#!/usr/bin/python3
########################################################################################################################
# Copyright 2022 Datum Technology Corporation
# All rights reserved
########################################################################################################################


########################################################################################################################
# IMPORTS
########################################################################################################################
from datetime import date
import os
import sys
import re
import jinja2
from jinja2 import Template
from mio import common
from mio import new
#from vsdx import VisioFile


########################################################################################################################
# GLOBALS
########################################################################################################################
dbg = False
uvm_gen_dir = re.sub("new_agent_advanced_parallel.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/sets/"
out_path = ""
default_copyright_owner = "Contributors"
default_license = "SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1"
name_of_copyright_owner = ""
name = ""
full_name = ""


########################################################################################################################
# TEMPLATE DATA
########################################################################################################################
parameters = { }

svg_agent_block_diagrams = [
    "agent_advanced_parallel/agent/docs/agent_block_diagram_1.svg",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_2.svg",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_3.svg",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_4.svg",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_5.svg",
    "agent_advanced_parallel/agent/docs/stream_agent_block_diagram_2.svg"
]

vsdx_agent_block_diagrams = [
    "agent_advanced_parallel/agent/docs/agent_block_diagram_1.vsdx",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_2.vsdx",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_3.vsdx",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_4.vsdx",
    "agent_advanced_parallel/agent/docs/mm_agent_block_diagram_5.vsdx",
    "agent_advanced_parallel/agent/docs/stream_agent_block_diagram_2.vsdx"
]

svg_stream_env_block_diagrams = [
    "agent_advanced_parallel/env/docs/stream_simplex_env_block_diagram.svg",
    "agent_advanced_parallel/env/docs/stream_duplex_env_block_diagram.svg"
]

vsdx_stream_env_block_diagrams = [
    "agent_advanced_parallel/env/docs/stream_simplex_env_block_diagram.vsdx",
    "agent_advanced_parallel/env/docs/stream_duplex_env_block_diagram.vsdx" 
]

segment_files = {
    "agent_advanced_parallel/agent/src/comps/rx_drv.sv"     : "uvma_{{ name }}/src/comps/uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_drv.sv",
    "agent_advanced_parallel/agent/src/comps/rx_sqr.sv"     : "uvma_{{ name }}/src/comps/uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_sqr.sv",
    "agent_advanced_parallel/agent/src/comps/tx_drv.sv"     : "uvma_{{ name }}/src/comps/uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv.sv",
    "agent_advanced_parallel/agent/src/comps/tx_sqr.sv"     : "uvma_{{ name }}/src/comps/uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_sqr.sv",
    "agent_advanced_parallel/agent/src/obj/rx_mon_trn.sv"   : "uvma_{{ name }}/src/obj/uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn.sv",
    "agent_advanced_parallel/agent/src/obj/tx_mon_trn.sv"   : "uvma_{{ name }}/src/obj/uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn.sv",
    "agent_advanced_parallel/agent/src/seq/rx_seq_item.sv"  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_seq_item.sv",
    "agent_advanced_parallel/agent/src/seq/tx_seq_item.sv"  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item.sv"
}

mm_agent_files = {
    #              SRC                                 DST
    "gitignore"                                                             : "uvma_{{ name }}/.gitignore",
    "agent_advanced_parallel/agent/docs/index.md"                           : "uvma_{{ name }}/docs/index.md",
    "agent_advanced_parallel/agent/docs/quick_start_guide.md"               : "uvma_{{ name }}/docs/quick_start_guide.md",
    "agent_advanced_parallel/agent/docs/user_guide.md"                      : "uvma_{{ name }}/docs/user_guide.md",
    "agent_advanced_parallel/agent/examples/instantiation.sv"               : "uvma_{{ name }}/examples/instantiation.sv",
    "agent_advanced_parallel/agent/examples/sequence.sv"                    : "uvma_{{ name }}/examples/sequence.sv",
    "agent_advanced_parallel/agent/src/comps/agent.sv"                      : "uvma_{{ name }}/src/comps/uvma_{{ name }}_agent.sv",
    "agent_advanced_parallel/agent/src/comps/cov_model.sv"                  : "uvma_{{ name }}/src/comps/uvma_{{ name }}_cov_model.sv",
    "agent_advanced_parallel/agent/src/comps/drv.sv"                        : "uvma_{{ name }}/src/comps/uvma_{{ name }}_drv.sv",
    "agent_advanced_parallel/agent/src/comps/logger.sv"                     : "uvma_{{ name }}/src/comps/uvma_{{ name }}_logger.sv",
    "agent_advanced_parallel/agent/src/comps/mon.sv"                        : "uvma_{{ name }}/src/comps/uvma_{{ name }}_mon.sv",
    "agent_advanced_parallel/agent/src/comps/vsqr.sv"                       : "uvma_{{ name }}/src/comps/uvma_{{ name }}_vsqr.sv",
    "agent_advanced_parallel/agent/src/obj/cfg.sv"                          : "uvma_{{ name }}/src/obj/uvma_{{ name }}_cfg.sv",
    "agent_advanced_parallel/agent/src/obj/cntxt.sv"                        : "uvma_{{ name }}/src/obj/uvma_{{ name }}_cntxt.sv",
    "agent_advanced_parallel/agent/src/obj/mm_mon_trn.sv"                   : "uvma_{{ name }}/src/obj/uvma_{{ name }}_mon_trn.sv",
    "agent_advanced_parallel/agent/src/reg/reg_adapter.sv"                  : "uvma_{{ name }}/src/reg/uvma_{{ name }}_reg_adapter.sv",
    "agent_advanced_parallel/agent/src/seq/base_vseq.sv"                    : "uvma_{{ name }}/src/seq/uvma_{{ name }}_base_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_idle_vseq.sv"                 : "uvma_{{ name }}/src/seq/uvma_{{ name }}_idle_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mon_vseq.sv"                  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_mon_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mode_1_drv_vseq.sv"           : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_1 }}_drv_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mode_2_drv_vseq.sv"           : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_2 }}_drv_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mode_2_handler_base_vseq.sv"  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mode_2_handler_mem_vseq.sv"   : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_2 }}_handler_mem_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_mode_2_handler_vseq_lib.sv"   : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_2 }}_handler_vseq_lib.sv",
    "agent_advanced_parallel/agent/src/seq/mm_rand_stim_vseq.sv"            : "uvma_{{ name }}/src/seq/uvma_{{ name }}_rand_stim_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/mm_seq_item.sv"                  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_seq_item.sv",
    "agent_advanced_parallel/agent/src/seq/transport_base_vseq.sv"          : "uvma_{{ name }}/src/seq/uvma_{{ name }}_transport_base_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/vseq_lib.sv"                     : "uvma_{{ name }}/src/seq/uvma_{{ name }}_vseq_lib.sv",
    "agent_advanced_parallel/agent/src/constants.sv"                        : "uvma_{{ name }}/src/uvma_{{ name }}_constants.sv",
    "agent_advanced_parallel/agent/src/if_chkr.sv"                          : "uvma_{{ name }}/src/uvma_{{ name }}_if_chkr.sv",
    "agent_advanced_parallel/agent/src/mm_if.sv"                            : "uvma_{{ name }}/src/uvma_{{ name }}_if.sv",
    "agent_advanced_parallel/agent/src/macros.svh"                          : "uvma_{{ name }}/src/uvma_{{ name }}_macros.svh",
    "agent_advanced_parallel/agent/src/pkg.flist"                           : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.flist",
    "agent_advanced_parallel/agent/src/pkg.flist.xsim"                      : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.flist.xsim",
    "agent_advanced_parallel/agent/src/pkg.sv"                              : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.sv",
    "agent_advanced_parallel/agent/src/tdefs.sv"                            : "uvma_{{ name }}/src/uvma_{{ name }}_tdefs.sv",
    "agent_advanced_parallel/agent/ip.yml"                                  : "uvma_{{ name }}/ip.yml",
    "agent_advanced_parallel/agent/mm_README.md"                            : "uvma_{{ name }}/README.md"
}

stream_agent_files = {
    #              SRC                                 DST
    "gitignore"                                                                 : "uvma_{{ name }}/.gitignore",
    "agent_advanced_parallel/agent/docs/index.md"                               : "uvma_{{ name }}/docs/index.md",
    "agent_advanced_parallel/agent/docs/quick_start_guide.md"                   : "uvma_{{ name }}/docs/quick_start_guide.md",
    "agent_advanced_parallel/agent/docs/user_guide.md"                          : "uvma_{{ name }}/docs/user_guide.md",
    "agent_advanced_parallel/agent/examples/instantiation.sv"                   : "uvma_{{ name }}/examples/instantiation.sv",
    "agent_advanced_parallel/agent/examples/sequence.sv"                        : "uvma_{{ name }}/examples/sequence.sv",
    "agent_advanced_parallel/agent/src/comps/agent.sv"                          : "uvma_{{ name }}/src/comps/uvma_{{ name }}_agent.sv",
    "agent_advanced_parallel/agent/src/comps/cov_model.sv"                      : "uvma_{{ name }}/src/comps/uvma_{{ name }}_cov_model.sv",
    "agent_advanced_parallel/agent/src/comps/drv.sv"                            : "uvma_{{ name }}/src/comps/uvma_{{ name }}_drv.sv",
    "agent_advanced_parallel/agent/src/comps/logger.sv"                         : "uvma_{{ name }}/src/comps/uvma_{{ name }}_logger.sv",
    "agent_advanced_parallel/agent/src/comps/mon.sv"                            : "uvma_{{ name }}/src/comps/uvma_{{ name }}_mon.sv",
    "agent_advanced_parallel/agent/src/comps/vsqr.sv"                           : "uvma_{{ name }}/src/comps/uvma_{{ name }}_vsqr.sv",
    "agent_advanced_parallel/agent/src/obj/cfg.sv"                              : "uvma_{{ name }}/src/obj/uvma_{{ name }}_cfg.sv",
    "agent_advanced_parallel/agent/src/obj/cntxt.sv"                            : "uvma_{{ name }}/src/obj/uvma_{{ name }}_cntxt.sv",
    "agent_advanced_parallel/agent/src/obj/stream_mon_trn.sv"                   : "uvma_{{ name }}/src/obj/uvma_{{ name }}_mon_trn.sv",
    "agent_advanced_parallel/agent/src/seq/base_vseq.sv"                        : "uvma_{{ name }}/src/seq/uvma_{{ name }}_base_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_idle_vseq.sv"                 : "uvma_{{ name }}/src/seq/uvma_{{ name }}_idle_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_mon_vseq.sv"                  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_mon_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_mode_1_drv_vseq.sv"           : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_1 }}_drv_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_mode_2_drv_vseq.sv"           : "uvma_{{ name }}/src/seq/uvma_{{ name }}_{{ mode_2 }}_drv_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_rand_stim_vseq.sv"            : "uvma_{{ name }}/src/seq/uvma_{{ name }}_rand_stim_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/stream_seq_item.sv"                  : "uvma_{{ name }}/src/seq/uvma_{{ name }}_seq_item.sv",
    "agent_advanced_parallel/agent/src/seq/transport_base_vseq.sv"              : "uvma_{{ name }}/src/seq/uvma_{{ name }}_transport_base_vseq.sv",
    "agent_advanced_parallel/agent/src/seq/vseq_lib.sv"                         : "uvma_{{ name }}/src/seq/uvma_{{ name }}_vseq_lib.sv",
    "agent_advanced_parallel/agent/src/constants.sv"                            : "uvma_{{ name }}/src/uvma_{{ name }}_constants.sv",
    "agent_advanced_parallel/agent/src/if_chkr.sv"                              : "uvma_{{ name }}/src/uvma_{{ name }}_if_chkr.sv",
    "agent_advanced_parallel/agent/src/stream_if.sv"                            : "uvma_{{ name }}/src/uvma_{{ name }}_if.sv",
    "agent_advanced_parallel/agent/src/macros.svh"                              : "uvma_{{ name }}/src/uvma_{{ name }}_macros.svh",
    "agent_advanced_parallel/agent/src/pkg.flist"                               : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.flist",
    "agent_advanced_parallel/agent/src/pkg.flist.xsim"                          : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.flist.xsim",
    "agent_advanced_parallel/agent/src/pkg.sv"                                  : "uvma_{{ name }}/src/uvma_{{ name }}_pkg.sv",
    "agent_advanced_parallel/agent/src/tdefs.sv"                                : "uvma_{{ name }}/src/uvma_{{ name }}_tdefs.sv",
    "agent_advanced_parallel/agent/ip.yml"                                      : "uvma_{{ name }}/ip.yml",
    "agent_advanced_parallel/agent/stream_README.md"                            : "uvma_{{ name }}/README.md"
}

mm_env_files = {
    "gitignore"                                                     : "uvme_{{ name }}_st/.gitignore",
    "agent_advanced_parallel/env/docs/mm_env_block_diagram.svg"     : "uvme_{{ name }}_st/docs/env_block_diagram.svg",
    "agent_advanced_parallel/env/docs/mm_env_block_diagram.vsdx"    : "uvme_{{ name }}_st/docs/env_block_diagram.vsdx",
    "agent_advanced_parallel/env/docs/index.md"                     : "uvme_{{ name }}_st/docs/index.md",
    "agent_advanced_parallel/env/examples/virtual_sequence.sv"      : "uvme_{{ name }}_st/examples/virtual_sequence.sv",
    "agent_advanced_parallel/env/src/comps/cov_model.sv"            : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_cov_model.sv",
    "agent_advanced_parallel/env/src/comps/env.sv"                  : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_env.sv",
    "agent_advanced_parallel/env/src/comps/mm_prd.sv"               : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_prd.sv",
    "agent_advanced_parallel/env/src/comps/mm_sb.sv"                : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_sb.sv",
    "agent_advanced_parallel/env/src/comps/vsqr.sv"                 : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_vsqr.sv",
    "agent_advanced_parallel/env/src/obj/cfg.sv"                    : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cfg.sv",
    "agent_advanced_parallel/env/src/obj/cntxt.sv"                  : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cntxt.sv",
    "agent_advanced_parallel/env/src/reg/full_reg.sv"               : "uvme_{{ name }}_st/src/reg/uvme_{{ name }}_st_full_reg.sv",
    "agent_advanced_parallel/env/src/reg/split_reg.sv"              : "uvme_{{ name }}_st/src/reg/uvme_{{ name }}_st_split_reg.sv",
    "agent_advanced_parallel/env/src/reg/reg_block.sv"              : "uvme_{{ name }}_st/src/reg/uvme_{{ name }}_st_reg_block.sv",
    "agent_advanced_parallel/env/src/seq/base_vseq.sv"              : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_base_vseq.sv",
    "agent_advanced_parallel/env/src/seq/mm_rand_stim_vseq.sv"      : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_rand_stim_vseq.sv",
    "agent_advanced_parallel/env/src/seq/reg_vseq.sv"               : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_reg_vseq.sv",
    "agent_advanced_parallel/env/src/seq/vseq_lib.sv"               : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_vseq_lib.sv",
    "agent_advanced_parallel/env/src/chkr.sv"                       : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_chkr.sv",
    "agent_advanced_parallel/env/src/constants.sv"                  : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_constants.sv",
    "agent_advanced_parallel/env/src/macros.svh"                    : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_macros.svh",
    "agent_advanced_parallel/env/src/pkg.flist"                     : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist",
    "agent_advanced_parallel/env/src/pkg.flist.xsim"                : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist.xsim",
    "agent_advanced_parallel/env/src/pkg.sv"                        : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.sv",
    "agent_advanced_parallel/env/src/tdefs.sv"                      : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_tdefs.sv",
    "agent_advanced_parallel/env/ip.yml"                            : "uvme_{{ name }}_st/ip.yml",
    "agent_advanced_parallel/env/mm_README.md"                      : "uvme_{{ name }}_st/README.md"
}

stream_env_files = {
    "gitignore"                                                       : "uvme_{{ name }}_st/.gitignore",
    "agent_advanced_parallel/env/docs/index.md"                       : "uvme_{{ name }}_st/docs/index.md",
    "agent_advanced_parallel/env/examples/virtual_sequence.sv"        : "uvme_{{ name }}_st/examples/virtual_sequence.sv",
    "agent_advanced_parallel/env/src/comps/cov_model.sv"              : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_cov_model.sv",
    "agent_advanced_parallel/env/src/comps/env.sv"                    : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_env.sv",
    "agent_advanced_parallel/env/src/comps/stream_prd.sv"             : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_prd.sv",
    "agent_advanced_parallel/env/src/comps/stream_sb.sv"              : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_sb.sv",
    "agent_advanced_parallel/env/src/comps/vsqr.sv"                   : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_vsqr.sv",
    "agent_advanced_parallel/env/src/obj/cfg.sv"                      : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cfg.sv",
    "agent_advanced_parallel/env/src/obj/cntxt.sv"                    : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cntxt.sv",
    "agent_advanced_parallel/env/src/reg/reg_block.sv"                : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_reg_block.sv",
    "agent_advanced_parallel/env/src/seq/base_vseq.sv"                : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_base_vseq.sv",
    "agent_advanced_parallel/env/src/seq/stream_rand_stim_vseq.sv"    : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_rand_stim_vseq.sv",
    "agent_advanced_parallel/env/src/seq/vseq_lib.sv"                 : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_vseq_lib.sv",
    "agent_advanced_parallel/env/src/chkr.sv"                         : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_chkr.sv",
    "agent_advanced_parallel/env/src/constants.sv"                    : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_constants.sv",
    "agent_advanced_parallel/env/src/macros.svh"                      : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_macros.svh",
    "agent_advanced_parallel/env/src/pkg.flist"                       : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist",
    "agent_advanced_parallel/env/src/pkg.flist.xsim"                  : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist.xsim",
    "agent_advanced_parallel/env/src/pkg.sv"                          : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.sv",
    "agent_advanced_parallel/env/src/tdefs.sv"                        : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_tdefs.sv",
    "agent_advanced_parallel/env/ip.yml"                              : "uvme_{{ name }}_st/ip.yml",
    "agent_advanced_parallel/env/stream_README.md"                    : "uvme_{{ name }}_st/README.md"
}

mm_tb_files = {
    "gitignore"                                                     : "uvmt_{{ name }}_st/.gitignore",
    "agent_advanced_parallel/tb/docs/tb_block_diagram.svg"          : "uvmt_{{ name }}_st/docs/tb_block_diagram.svg",
    "agent_advanced_parallel/tb/docs/tb_block_diagram.vsdx"         : "uvmt_{{ name }}_st/docs/tb_block_diagram.vsdx",
    "agent_advanced_parallel/tb/docs/index.md"                      : "uvmt_{{ name }}_st/docs/index.md",
    "agent_advanced_parallel/tb/examples/test.sv"                   : "uvmt_{{ name }}_st/examples/test.sv",
    "agent_advanced_parallel/tb/src/tb/clknrst_gen_if.sv"           : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_clknrst_gen_if.sv",
    "agent_advanced_parallel/tb/src/tb/dut_chkr.sv"                 : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_chkr.sv",
    "agent_advanced_parallel/tb/src/tb/mm_dut_wrap.sv"              : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_wrap.sv",
    "agent_advanced_parallel/tb/src/tb/tb.sv"                       : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_tb.sv",
    "agent_advanced_parallel/tb/src/tests/ts.yml"                   : "uvmt_{{ name }}_st/src/tests/ts.yml",
    "agent_advanced_parallel/tb/src/tests/base_test_workarounds.sv" : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test_workarounds.sv",
    "agent_advanced_parallel/tb/src/tests/base_test.sv"             : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test.sv",
    "agent_advanced_parallel/tb/src/tests/rand_stim_test.sv"        : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_rand_stim_test.sv",
    "agent_advanced_parallel/tb/src/tests/reg_test.sv"              : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_reg_test.sv",
    "agent_advanced_parallel/tb/src/tests/test_cfg.sv"              : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_test_cfg.sv",
    "agent_advanced_parallel/tb/src/constants.sv"                   : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_constants.sv",
    "agent_advanced_parallel/tb/src/macros.svh"                     : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_macros.svh",
    "agent_advanced_parallel/tb/src/pkg.flist"                      : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist",
    "agent_advanced_parallel/tb/src/pkg.flist.xsim"                 : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist.xsim",
    "agent_advanced_parallel/tb/src/pkg.sv"                         : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.sv",
    "agent_advanced_parallel/tb/src/tdefs.sv"                       : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_tdefs.sv",
    "agent_advanced_parallel/tb/ip.yml"                             : "uvmt_{{ name }}_st/ip.yml",
    "agent_advanced_parallel/tb/mm_README.md"                       : "uvmt_{{ name }}_st/README.md"
}

stream_tb_files = {
    "gitignore"                                                     : "uvmt_{{ name }}_st/.gitignore",
    "agent_advanced_parallel/tb/docs/tb_block_diagram.svg"          : "uvmt_{{ name }}_st/docs/tb_block_diagram.svg",
    "agent_advanced_parallel/tb/docs/tb_block_diagram.vsdx"         : "uvmt_{{ name }}_st/docs/tb_block_diagram.vsdx",
    "agent_advanced_parallel/tb/docs/index.md"                      : "uvmt_{{ name }}_st/docs/index.md",
    "agent_advanced_parallel/tb/examples/test.sv"                   : "uvmt_{{ name }}_st/examples/test.sv",
    "agent_advanced_parallel/tb/src/tb/clknrst_gen_if.sv"           : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_clknrst_gen_if.sv",
    "agent_advanced_parallel/tb/src/tb/dut_chkr.sv"                 : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_chkr.sv",
    "agent_advanced_parallel/tb/src/tb/stream_dut_wrap.sv"          : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_wrap.sv",
    "agent_advanced_parallel/tb/src/tb/tb.sv"                       : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_tb.sv",
    "agent_advanced_parallel/tb/src/tests/ts.yml"                   : "uvmt_{{ name }}_st/src/tests/ts.yml",
    "agent_advanced_parallel/tb/src/tests/base_test_workarounds.sv" : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test_workarounds.sv",
    "agent_advanced_parallel/tb/src/tests/base_test.sv"             : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test.sv",
    "agent_advanced_parallel/tb/src/tests/rand_stim_test.sv"        : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_rand_stim_test.sv",
    "agent_advanced_parallel/tb/src/tests/test_cfg.sv"              : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_test_cfg.sv",
    "agent_advanced_parallel/tb/src/constants.sv"                   : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_constants.sv",
    "agent_advanced_parallel/tb/src/macros.svh"                     : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_macros.svh",
    "agent_advanced_parallel/tb/src/pkg.flist"                      : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist",
    "agent_advanced_parallel/tb/src/pkg.flist.xsim"                 : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist.xsim",
    "agent_advanced_parallel/tb/src/pkg.sv"                         : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.sv",
    "agent_advanced_parallel/tb/src/tdefs.sv"                       : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_tdefs.sv",
    "agent_advanced_parallel/tb/ip.yml"                             : "uvmt_{{ name }}_st/ip.yml",
    "agent_advanced_parallel/tb/stream_README.md"                   : "uvmt_{{ name }}_st/README.md"
}

mm_directories = [
    "uvma_{{ name }}",
    "uvma_{{ name }}/bin",
    "uvma_{{ name }}/docs",
    "uvma_{{ name }}/examples",
    "uvma_{{ name }}/src",
    "uvma_{{ name }}/src/comps",
    "uvma_{{ name }}/src/obj",
    "uvma_{{ name }}/src/reg",
    "uvma_{{ name }}/src/seq",
    "uvme_{{ name }}_st",
    "uvme_{{ name }}_st/bin",
    "uvme_{{ name }}_st/docs",
    "uvme_{{ name }}_st/examples",
    "uvme_{{ name }}_st/src",
    "uvme_{{ name }}_st/src/comps",
    "uvme_{{ name }}_st/src/reg",
    "uvme_{{ name }}_st/src/obj",
    "uvme_{{ name }}_st/src/seq",
    "uvmt_{{ name }}_st",
    "uvmt_{{ name }}_st/bin",
    "uvmt_{{ name }}_st/docs",
    "uvmt_{{ name }}_st/examples",
    "uvmt_{{ name }}_st/src",
    "uvmt_{{ name }}_st/src/tb",
    "uvmt_{{ name }}_st/src/tests"
]

stream_directories = [
    "uvma_{{ name }}",
    "uvma_{{ name }}/bin",
    "uvma_{{ name }}/docs",
    "uvma_{{ name }}/examples",
    "uvma_{{ name }}/src",
    "uvma_{{ name }}/src/comps",
    "uvma_{{ name }}/src/obj",
    "uvma_{{ name }}/src/seq",
    "uvme_{{ name }}_st",
    "uvme_{{ name }}_st/bin",
    "uvme_{{ name }}_st/docs",
    "uvme_{{ name }}_st/examples",
    "uvme_{{ name }}_st/src",
    "uvme_{{ name }}_st/src/comps",
    "uvme_{{ name }}_st/src/obj",
    "uvme_{{ name }}_st/src/seq",
    "uvmt_{{ name }}_st",
    "uvmt_{{ name }}_st/bin",
    "uvmt_{{ name }}_st/docs",
    "uvmt_{{ name }}_st/examples",
    "uvmt_{{ name }}_st/src",
    "uvmt_{{ name }}_st/src/tb",
    "uvmt_{{ name }}_st/src/tests"
]


########################################################################################################################
# METHODS
########################################################################################################################
def combine_dict(d1, d2):
    d3 = {}
    for d in d1:
        d3[d] = d1[d]
    for d in d2:
        d3[d] = d2[d]
    return d3


def process_file(path, file_path_template):
    global parameters
    common.dbg("Processing file " + path + " with path template " + file_path_template)
    
    fin = open(relative_path_to_template + path, "rt")
    fout_path_template = Template(file_path_template)
    fout_path = fout_path_template.render(parameters)
    fout_path = out_path + "/" + fout_path
    common.dbg("Templated path: " + fout_path)
    fout = open(fout_path, "w+")
    
    if (fout_path[-5:] == ".vsdx"):
        new.j2_to_vsdx(parameters, relative_path_to_template + path, fout_path)
    else:
        data = fin.read()
        template = Template(data)
        data = template.render(parameters)
        common.dbg("Writing " + fout_path)
        fout.write(data)
        fout.close()
    fin.close()


def process_all_files():
    global parameters
    common.info("Working ...")
    if parameters['mm']:
        final_fileset = combine_dict(mm_agent_files, combine_dict(mm_env_files, mm_tb_files))
    else:
        final_fileset = combine_dict(stream_agent_files, combine_dict(stream_env_files, stream_tb_files))
    for file in final_fileset:
        process_file(file, final_fileset[file])
    for segment in parameters['segments']:
        for file in segment_files:
            parameters['segment'] = segment
            process_file(file, segment_files[file])


def create_directories():
    global parameters
    if parameters['mm']:
        directories = mm_directories
    else:
        directories = stream_directories
    for dir in directories:
        dir_name = out_path + "/" + dir
        dir_name_template = Template(dir_name)
        dir_name = dir_name_template.render(parameters)
        common.create_dir(dir_name)


def pick_out_path():
    global out_path
    global default_copyright_owner
    if len(sys.argv) > 1:
        out_path = sys.argv[1]
        print("Code will be output to " + out_path)
    if len(sys.argv) > 2:
        default_copyright_owner = sys.argv[2].replace('"', "")
        print("Default copyright owner is " + default_copyright_owner)


def prompt_user_values():
    global out_path
    global name
    global full_name
    global name_of_copyright_owner
    global default_copyright_owner
    global parameters
    channels = []
    dir_1_str = ""
    dir_2_str = ""
    is_duplex = False
    is_be     = False
    
    common.info("Moore.io Template Generator: UVM Advanced Parallel Agent - (v1p0)")
    common.info("******************************************************************")
    common.info("The answers to the following questionnaire will be used to generate the code for your new UVM Agent")
    common.info("")
    
    if out_path == "":
        out_path = common.prompt("Please enter the destination path for this new agent (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter a specific name for the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    
    license = common.prompt("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    
    name = common.prompt("Please enter the package name for this agent (ex: 'axi'); this name will be used for all agent types (ex: 'uvma_axi_agent_c'):\n").lower().strip()
    if name == "":
        sys.exit("ERROR: package name cannot be empty.  Exiting.")
    
    full_name = common.prompt("Please enter the full name for this agent (ex: 'Advanced eXtensible Interface'):\n").strip()
    if full_name == "":
        sys.exit("ERROR: full name cannot be empty.  Exiting.")
    
    is_mm_str = common.prompt("Is this agent memory mapped?  [N/y]").strip().lower()
    if is_mm_str == "":
        is_mm = False
    else:
        if is_mm_str == "n" or is_mm_str == "no":
            is_mm = False
        elif is_mm_str == "y" or is_mm_str == "yes":
            is_mm = True
        else:
            sys.exit("ERROR: please enter 'y' or 'n'")
    
    if is_mm:
        mode_1 = common.prompt("Please enter the first mode for this new agent (default: 'mstr'):\n").strip().lower()
        if mode_1 == "":
            mode_1 = "mstr"
        mode_2 = common.prompt("Please enter the second mode for this new agent (default: 'slv'):\n").strip().lower()
        if mode_2 == "":
            mode_2 = "slv"
        is_be_str = common.prompt("Does this agent support byte-enabled addressing?  [Y/n]").strip().lower()
        if is_be_str == "":
            is_be = True
        else:
            if is_be_str == "n" or is_be_str == "no":
                is_be = False
            elif is_be_str == "y" or is_be_str == "yes":
                is_be = True
            else:
                sys.exit("ERROR: please enter 'y' or 'n'")
        num_segments_str = common.prompt("How many channels are on the physical interface? (Ex: AXI has 5):\n").strip()
        num_segments = int(num_segments_str)
        if num_segments_str == "" or (num_segments <= 0):
            sys.exit("ERROR: please enter an integer larger than 0")
        if num_segments > 5:
            sys.exit("ERROR: please enter an integer smaller or equal to 5")
        if num_segments > 1:
            for ii in range(num_segments):
                channel_name = common.prompt("Please enter the name for channel #" + str(ii+1) + ":\n").strip().lower()
                if channel_name == "":
                    sys.exit("ERROR: channel name cannot be empty.  Exiting")
                tx_name = common.prompt("Please select which agent mode is the 'tx' for channel '" + channel_name + "': '" + mode_1 + "' or '" + mode_2 + "'\n").strip().lower()
                if tx_name != mode_1 and tx_name != mode_2:
                    sys.exit("ERROR: channel tx must be '" + mode_1 + "' or '" + mode_2 + "'.  Exiting")
                if tx_name == mode_1:
                    rx_name = mode_2
                else:
                    rx_name = mode_1
                channels.append({
                    "tx"   : tx_name,
                    "rx"   : rx_name,
                    "name" : channel_name,
                    "index": ii
                })
        else:
            channels.append({
                "tx"   : mode_1,
                "rx"   : mode_2,
                "name" : "phy",
                "index": 0
            })
    else:
        mode_1 = common.prompt("Please enter the first mode for this new agent (default: 'client'):\n").strip().lower()
        if mode_1 == "":
            mode_1 = "client"
        mode_2 = common.prompt("Please enter the second mode for this new agent (default: 'mac'):\n").strip().lower()
        if mode_2 == "":
            mode_2 = "mac"
        
        is_duplex_str = common.prompt("Is this agent duplex?  [Y/n]").strip().lower()
        if is_duplex_str == "":
            num_segments = 2
            is_duplex = False
        else:
            if is_duplex_str == "n" or is_duplex_str == "no":
                num_segments = 1
                is_duplex = False
            elif is_duplex_str == "y" or is_duplex_str == "yes":
                num_segments = 2
                is_duplex = True
            else:
                common.fatal("ERROR: please enter 'y' or 'n'")
        if is_duplex:
            dir_1_str = common.prompt("Please enter the first direction for this new agent (default: 'tx'):\n").strip().lower()
            if dir_1_str == "":
                dir_1_str = "tx"
            dir_2_str = common.prompt("Please enter the second direction for this new agent (default: 'rx'):\n").strip().lower()
            if dir_2_str == "":
                dir_2_str = "rx"
            channels.append({
                "tx"   : mode_1,
                "rx"   : mode_2,
                "name" : dir_1_str,
                "index": 0
            })
            channels.append({
                "tx"   : mode_2,
                "rx"   : mode_1,
                "name" : dir_2_str,
                "index": 1
            })
        else:
            channels.append({
                "tx"   : mode_1,
                "rx"   : mode_2,
                "name" : "phy",
                "index": 0
            })
    
    mm_agent_files[svg_agent_block_diagrams [num_segments-1]] = "uvma_{{ name }}/docs/agent_block_diagram.svg"
    mm_agent_files[vsdx_agent_block_diagrams[num_segments-1]] = "uvma_{{ name }}/docs/agent_block_diagram.vsdx"
    if is_duplex:
        stream_agent_files[svg_agent_block_diagrams   [-1]] = "uvma_{{ name }}/docs/agent_block_diagram.svg"
        stream_agent_files[vsdx_agent_block_diagrams  [-1]] = "uvma_{{ name }}/docs/agent_block_diagram.vsdx"
        stream_env_files[svg_stream_env_block_diagrams [1]] = "uvme_{{ name }}_st/docs/env_block_diagram.svg"
        stream_env_files[vsdx_stream_env_block_diagrams[1]] = "uvme_{{ name }}_st/docs/env_block_diagram.vsdx"
    else:
        stream_agent_files[svg_agent_block_diagrams    [0]] = "uvma_{{ name }}/docs/agent_block_diagram.svg"
        stream_agent_files[vsdx_agent_block_diagrams   [0]] = "uvma_{{ name }}/docs/agent_block_diagram.vsdx"
        stream_env_files[svg_stream_env_block_diagrams [0]] = "uvme_{{ name }}_st/docs/env_block_diagram.svg"
        stream_env_files[vsdx_stream_env_block_diagrams[0]] = "uvme_{{ name }}_st/docs/env_block_diagram.vsdx"
    
    parameters = {
        "name"                    : name,
        "full_name"               : full_name,
        "name_of_copyright_owner" : name_of_copyright_owner,
        "license"                 : license,
        "year"                    : date.today().year,
        "mm"                      : is_mm,
        "sbe"                     : is_be,
        "mode_1"                  : mode_1,
        "mode_2"                  : mode_2,
        "dir_1"                   : dir_1_str,
        "dir_2"                   : dir_2_str,
        "duplex"                  : is_duplex,
        "segments"                : channels,
        "num_segments"            : num_segments,
        "multisegment"            : (len(channels) > 1)
    }
    
    # Flatten some parameters for svg
    sequencer = 0
    driver = 0
    chan_id = 0
    for channel in channels:
        parameters['sqrn' + str(sequencer)] = mode_1 + "_" + channel['name']
        parameters['sqrt' + str(sequencer)] = name + "_" + mode_1 + "_" + channel['name']
        sequencer += 1
        parameters['sqrn' + str(sequencer)] = mode_2 + "_" + channel['name']
        parameters['sqrt' + str(sequencer)] = name + "_" + mode_2 + "_" + channel['name']
        sequencer += 1
        parameters['drvn' + str(driver)] = mode_1 + "_" + channel['name']
        parameters['drvt' + str(driver)] = name + "_" + mode_1 + "_" + channel['name']
        parameters['dmp' + str(driver)] = "drv_" + mode_1 + "_" + channel['name'] + "_mp"
        driver += 1
        parameters['drvn' + str(driver)] = mode_2 + "_" + channel['name']
        parameters['drvt' + str(driver)] = name + "_" + mode_2 + "_" + channel['name']
        parameters['dmp' + str(driver)] = "drv_" + mode_2 + "_" + channel['name'] + "_mp"
        driver += 1
        parameters['mmp' + str(chan_id)] = "mon_" + channel['name'] + "_mp"
        chan_id += 1


def print_end_message():
    global out_path
    global name
    global parameters
    
    common.banner("IPs successfully generated:")
    common.info("  * " + out_path + "/uvma_" + name)
    common.info("  * " + out_path + "/uvme_" + name + "_st")
    common.info("  * " + out_path + "/uvmt_" + name + "_st")


########################################################################################################################
# ENTRY POINT
########################################################################################################################
def main():
    #pick_out_path()
    prompt_user_values()
    create_directories()
    process_all_files()
    print_end_message()
