#!/usr/bin/python3
########################################################################################################################
# Copyright 2022 Datum Technology Corporation
# All rights reserved.
########################################################################################################################


########################################################################################################################
# IMPORTS
########################################################################################################################
from datetime import date
import os
import sys
import re
import jinja2
from jinja2 import Template
from mio import common
from mio import new
#from vsdx import VisioFile


########################################################################################################################
# GLOBALS
########################################################################################################################
dbg = False
uvm_gen_dir = re.sub("new_lib.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/sets/"
out_path = ""
default_copyright_owner = "Contributors"
default_license = "SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1"
name_of_copyright_owner = ""
name = ""
name_normal_case = ""


########################################################################################################################
# TEMPLATE DATA
########################################################################################################################
parameters = { }

files = {
    #              SRC                                 DST
    "gitignore"                            : "uvml_{{ name }}/.gitignore",
    "lib/lib/docs/index.md"                : "uvml_{{ name }}/docs/index.md",
    "lib/lib/docs/quick_start_guide.md"    : "uvml_{{ name }}/docs/quick_start_guide.md",
    "lib/lib/docs/user_guide.md"           : "uvml_{{ name }}/docs/user_guide.md",
    "lib/lib/examples/component.sv"        : "uvml_{{ name }}/examples/component.sv",
    "lib/lib/examples/object.sv"           : "uvml_{{ name }}/examples/object.sv",
    "lib/lib/src/constants.sv"             : "uvml_{{ name }}/src/uvml_{{ name }}_constants.sv",
    "lib/lib/src/macros.svh"               : "uvml_{{ name }}/src/uvml_{{ name }}_macros.svh",
    "lib/lib/src/pkg.flist"                : "uvml_{{ name }}/src/uvml_{{ name }}_pkg.flist",
    "lib/lib/src/pkg.flist.xsim"           : "uvml_{{ name }}/src/uvml_{{ name }}_pkg.flist.xsim",
    "lib/lib/src/pkg.sv"                   : "uvml_{{ name }}/src/uvml_{{ name }}_pkg.sv",
    "lib/lib/src/tdefs.sv"                 : "uvml_{{ name }}/src/uvml_{{ name }}_tdefs.sv",
    "lib/lib/ip.yml"                       : "uvml_{{ name }}/ip.yml",
    #"LICENSE_solderpad_v2p1.md"        : "uvml_{{ name }}/LICENSE.md",
    "lib/lib/README.md"                    : "uvml_{{ name }}/README.md",
    "gitignore"                                 : "uvme_{{ name }}_st/.gitignore",
    "lib/env/docs/env_block_diagram.svg"        : "uvme_{{ name }}_st/docs/env_block_diagram.svg",
    "lib/env/docs/env_block_diagram.vsdx"       : "uvme_{{ name }}_st/docs/env_block_diagram.vsdx",
    "lib/env/docs/index.md"                     : "uvme_{{ name }}_st/docs/index.md",
    "lib/env/examples/instantiation.sv"         : "uvme_{{ name }}_st/examples/instantiation.sv",
    "lib/env/examples/virtual_sequence.sv"      : "uvme_{{ name }}_st/examples/virtual_sequence.sv",
    "lib/env/src/comps/cov_model.sv"            : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_cov_model.sv",
    "lib/env/src/comps/env.sv"                  : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_env.sv",
    "lib/env/src/comps/prd.sv"                  : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_prd.sv",
    "lib/env/src/comps/sb.sv"                   : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_sb.sv",
    "lib/env/src/comps/vsqr.sv"                 : "uvme_{{ name }}_st/src/comps/uvme_{{ name }}_st_vsqr.sv",
    "lib/env/src/obj/cfg.sv"                    : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cfg.sv",
    "lib/env/src/obj/cntxt.sv"                  : "uvme_{{ name }}_st/src/obj/uvme_{{ name }}_st_cntxt.sv",
    "lib/env/src/seq/base_vseq.sv"              : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_base_vseq.sv",
    "lib/env/src/seq/vseq_lib.sv"               : "uvme_{{ name }}_st/src/seq/uvme_{{ name }}_st_vseq_lib.sv",
    "lib/env/src/chkr.sv"                       : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_chkr.sv",
    "lib/env/src/constants.sv"                  : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_constants.sv",
    "lib/env/src/macros.svh"                    : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_macros.svh",
    "lib/env/src/pkg.flist"                     : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist",
    "lib/env/src/pkg.flist.xsim"                : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.flist.xsim",
    "lib/env/src/pkg.sv"                        : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_pkg.sv",
    "lib/env/src/tdefs.sv"                      : "uvme_{{ name }}_st/src/uvme_{{ name }}_st_tdefs.sv",
    "lib/env/README.md"                         : "uvme_{{ name }}_st/README.md",
    "lib/env/ip.yml"                            : "uvme_{{ name }}_st/ip.yml",
    #"LICENSE_solderpad_v2p1.md"                         : "uvme_{{ name }}_st/LICENSE.md",
    "gitignore"                                 : "uvmt_{{ name }}_st/.gitignore",
    "lib/tb/docs/tb_block_diagram.svg"          : "uvmt_{{ name }}_st/docs/tb_block_diagram.svg",
    "lib/tb/docs/tb_block_diagram.vsdx"         : "uvmt_{{ name }}_st/docs/tb_block_diagram.vsdx",
    "lib/tb/docs/index.md"                      : "uvmt_{{ name }}_st/docs/index.md",
    "lib/tb/examples/test.sv"                   : "uvmt_{{ name }}_st/examples/test.sv",
    "lib/tb/src/tb/clknrst_gen_if.sv"           : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_clknrst_gen_if.sv",
    "lib/tb/src/tb/dut_chkr.sv"                 : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_chkr.sv",
    "lib/tb/src/tb/dut_wrap.sv"                 : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_dut_wrap.sv",
    "lib/tb/src/tb/tb.sv"                       : "uvmt_{{ name }}_st/src/tb/uvmt_{{ name }}_st_tb.sv",
    "lib/tb/src/tests/ts.yml"                   : "uvmt_{{ name }}_st/src/tests/ts.yml",
    "lib/tb/src/tests/base_test_workarounds.sv" : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test_workarounds.sv",
    "lib/tb/src/tests/base_test.sv"             : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_base_test.sv",
    "lib/tb/src/tests/test_cfg.sv"              : "uvmt_{{ name }}_st/src/tests/uvmt_{{ name }}_st_test_cfg.sv",
    "lib/tb/src/constants.sv"                   : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_constants.sv",
    "lib/tb/src/macros.svh"                     : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_macros.svh",
    "lib/tb/src/pkg.flist"                      : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist",
    "lib/tb/src/pkg.flist.xsim"                 : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.flist.xsim",
    "lib/tb/src/pkg.sv"                         : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_pkg.sv",
    "lib/tb/src/tdefs.sv"                       : "uvmt_{{ name }}_st/src/uvmt_{{ name }}_st_tdefs.sv",
    "lib/tb/ip.yml"                             : "uvmt_{{ name }}_st/ip.yml",
    #"LICENSE_solderpad_v2p1.md"                         : "uvmt_{{ name }}_st/LICENSE.md",
    "lib/tb/README.md"                          : "uvmt_{{ name }}_st/README.md"
}

directories = [
    "uvml_{{ name }}",
    "uvml_{{ name }}/bin",
    "uvml_{{ name }}/docs",
    "uvml_{{ name }}/examples",
    "uvml_{{ name }}/src",
    "uvml_{{ name }}/src/comps",
    "uvml_{{ name }}/src/obj",
    "uvml_{{ name }}/src/reg",
    "uvml_{{ name }}/src/seq",
    "uvme_{{ name }}_st",
    "uvme_{{ name }}_st/bin",
    "uvme_{{ name }}_st/docs",
    "uvme_{{ name }}_st/examples",
    "uvme_{{ name }}_st/src",
    "uvme_{{ name }}_st/src/comps",
    "uvme_{{ name }}_st/src/obj",
    "uvme_{{ name }}_st/src/reg",
    "uvme_{{ name }}_st/src/seq",
    "uvmt_{{ name }}_st",
    "uvmt_{{ name }}_st/bin",
    "uvmt_{{ name }}_st/docs",
    "uvmt_{{ name }}_st/examples",
    "uvmt_{{ name }}_st/src",
    "uvmt_{{ name }}_st/src/tb",
    "uvmt_{{ name }}_st/src/tests"
]


########################################################################################################################
# MAIN
########################################################################################################################
def process_file(path, file_path_template):
    global parameters
    common.dbg("Processing file " + path + " with path template " + file_path_template)
    
    fin = open(relative_path_to_template + path, "rt")
    fout_path_template = Template(file_path_template)
    fout_path = fout_path_template.render(parameters)
    fout_path = out_path + "/" + fout_path
    common.dbg("Templated path: " + fout_path)
    fout = open(fout_path, "w+")
    fout_path_len = len(fout_path)
    if (fout_path[-5:] == ".vsdx"):
        new.j2_to_vsdx(parameters, relative_path_to_template + path, fout_path)
    else:
        data = fin.read()
        template = Template(data)
        data = template.render(parameters)
        common.dbg("Writing " + fout_path)
        fout.write(data)
        fout.close()
    fin.close()


def process_all_files():
    common.info("Working ...")
    for file in files:
        process_file(file, files[file])


def create_directories():
    global parameters
    for dir in directories:
        dir_name = out_path + "/" + dir
        dir_name_template = Template(dir_name)
        dir_name = dir_name_template.render(parameters)
        common.create_dir(dir_name)


def pick_out_path():
    global out_path
    global default_copyright_owner
    if len(sys.argv) > 1:
        out_path = sys.argv[1]
        print("Code will be output to " + out_path)
    if len(sys.argv) > 2:
        default_copyright_owner = sys.argv[2].replace('"', "")
        print("Default copyright owner is " + default_copyright_owner)


def prompt_user_values():
    global out_path
    global name
    global name_normal_case
    global reset_agent_type
    global name_of_copyright_owner
    global default_copyright_owner
    global parameters
    
    common.info("Moore.io Template Generator: UVM Library (v1p1)")
    common.info("***********************************************************************")
    common.info("The answers to the following questionnaire will be used to generate the code for your new UVM Library")
    common.info("")
    
    if out_path == "":
        out_path = common.prompt("Please enter the destination path for this new Library (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    
    name_of_copyright_owner = common.prompt("Please enter a specific name for the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    
    license = common.prompt("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    
    name = common.prompt("Please enter the name of this new Library (ex: 'math'); this name will be used for all Library types (ex: 'uvml_math_c'):\n").lower().strip()
    if name == "":
        sys.exit("ERROR: package name cannot be empty.  Exiting.")
    
    name_normal_case = common.prompt("Please enter the (descriptive) name for this new Library (ex: 'Mathematical Objects'):\n").strip()
    if name_normal_case == "":
        sys.exit("ERROR: descriptive name cannot be empty.  Exiting.")
    
    parameters = {
        "name_normal_case"        : name_normal_case,
        "name"                    : name,
        "name_uppercase"          : name.upper(),
        "year"                    : date.today().year,
        "name_of_copyright_owner" : name_of_copyright_owner,
        "license"                 : license
    }


def print_end_message():
    global out_path
    global name
    global parameters
    
    common.banner("IPs successfully generated:")
    common.info("  * " + out_path + "/uvml_" + name)
    common.info("  * " + out_path + "/uvme_" + name + "_st")
    common.info("  * " + out_path + "/uvmt_" + name + "_st")


########################################################################################################################
# ENTRY POINT
########################################################################################################################
def main():
    #pick_out_path()
    prompt_user_values()
    create_directories()
    process_all_files()
    print_end_message()
