# Copyright 2021-2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


from mio import clean
from mio import cov
from mio import dox
from mio import results
from mio import sim
from mio import common

import jinja2
import os
import sys
import re
import pathlib

cli_args = None
dbg             = True
sim_debug       = False
sim_gui         = False
sim_waves       = False
sim_cov         = False
glb_args = {}
glb_cfg  = {}

version_str = 'Moore.io Client Command Line Interface 1.2.0'
mio_client_dir    = re.sub("cfg.py", "", os.path.realpath(__file__))
mio_data_src_dir  = str(pathlib.Path(os.path.join(mio_client_dir, "../data")).resolve())
mio_template_dir  = str(pathlib.Path(os.path.join(mio_client_dir, "../templates")).resolve())
vivado_home       = os.getenv("VIVADO_HOME" , '/tools/vivado/2022.1/Vivado/2022.1/bin/')
metrics_home      = os.getenv("METRICS_HOME", '/tools/metrics/')
vcs_home          = os.getenv("VCS_HOME"    , '/tools/vcs/'    )
xcelium_home      = os.getenv("XCELIUM_HOME", '/tools/xcelium/')
questa_home       = os.getenv("QUESTA_HOME" , '/tools/questa/' )
riviera_home      = os.getenv("RIVIERA_HOME", '/tools/riviera/')
uvm_dpi_so        = "uvm_dpi"
user_dir          = os.path.expanduser("~")
mio_user_dir      = user_dir + "/.mio"
user_global_ips_path = mio_user_dir + "/vendors"

pwd                   = ""
temp_path             = ""
project_dir           = ""
project_name          = ""
sim_dir               = ""
sim_results_dir       = ""
regr_results_dir      = ""
mio_data_dir          = ""
fsoc_dir              = ""
sim_output_dir        = ""
dependencies_path     = ""
ip_cache_file_path    = ""
fsoc_cache_file_path  = ""
job_history_file_path = ""
commands_file_path    = ""

default_simulator = "vivado"
regression_name = ""
test_suite_name = ""
test_results_path_template = ""

templateLoader = jinja2.FileSystemLoader(searchpath=mio_template_dir)
templateEnv    = jinja2.Environment(loader=templateLoader)

fresh_fsoc_cache = False
fresh_ip_cache = False

ip_paths = []
job_history = {}
configuration = {}
ip_local_cache = {}
ip_external_cache = {}
fsoc_cache = None
target_ip = None
target_ip_name = ""
target_ip_is_local = False


def set_pwd(wd):
    global pwd
    pwd = str(wd.resolve())

def set_project_dir(path):
    global project_dir 
    global mio_data_dir
    global temp_path
    global fsoc_dir
    global sim_output_dir
    global dependencies_path
    global ip_cache_file_path
    global fsoc_cache_file_path
    global job_history_file_path
    global commands_file_path
    project_dir           = path
    mio_data_dir          = project_dir + "/.mio"
    temp_path             = mio_data_dir + '/temp'
    fsoc_dir              = mio_data_dir + "/fsoc"
    sim_output_dir        = mio_data_dir + "/sim_out"
    dependencies_path     = mio_data_dir + "/vendors"
    ip_cache_file_path    = mio_data_dir + "/ip_cache.yml"
    fsoc_cache_file_path  = mio_data_dir + "/fsoc_cache.yml"
    job_history_file_path = mio_data_dir + "/job_history.yml"
    commands_file_path    = mio_data_dir + "/commands.yml"

