# Copyright 2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


from mio import cache
from mio import common
from mio import cfg
from mio import cov
from mio import dox
from mio import results
from mio import sim

import os


def do_clean():
    if cfg.cli_args.deep:
        common.banner(f"Deep cleaning IP '{cfg.target_ip_name}'")
        for dep in cfg.target_ip['data']['dependencies']:
            dep_name = dep['name']
            if dep_name not in cfg.ip_local_cache:
                clean_ip_compiled_code(dep_name)
    else:
        common.banner(f"Cleaning IP '{cfg.target_ip_name}'")
    for dep in cfg.target_ip['data']['dependencies']:
        dep_name = dep['name']
        if dep_name in cfg.ip_local_cache:
            clean_ip_compiled_code(dep_name)
    if 'dut' in cfg.target_ip['data']:
        if cfg.target_ip['data']['dut']['ip-type'].lower().strip() == "fsoc":
            clean_core_compiled_code(cfg.target_ip['data']['dut']['full-name'])
        else:
            clean_ip_compiled_code(cfg.target_ip['data']['dut']['name'])
    clean_ip_compiled_code(cfg.target_ip_name)


def clean_core_compiled_code(core_name):
    common.info(f"Deleting code and compiled code objects for FuseSoC core '{core_name}'")
    common.remove_dir(cfg.sim_output_dir + '/vivado/local/'  + core_name)
    common.remove_dir(cfg.sim_output_dir + '/metrics/local/' + core_name)
    common.remove_dir(cfg.sim_output_dir + '/vcs/local/'     + core_name)
    common.remove_dir(cfg.sim_output_dir + '/xcelium/local/' + core_name)
    common.remove_dir(cfg.sim_output_dir + '/questa/local/'  + core_name)
    common.remove_dir(cfg.sim_output_dir + '/riviera/local/' + core_name)
    common.remove_dir(cfg.fsoc_dir + "/" + core_name)
    cfg.fsoc_cache[core_name]['installed'] = False


def clean_ip_compiled_code(ip_name):
    if ip_name in cfg.ip_local_cache:
        common.info(f"Deleting compiled code objects for local IP '{ip_name}'")
        common.remove_dir(cfg.sim_output_dir + '/vivado/local/'  + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/metrics/local/' + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/vcs/local/'     + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/xcelium/local/' + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/questa/local/'  + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/riviera/local/' + ip_name)
    else:
        if ip_name == "uvm":
            return
        ip = cache.get_ip(ip_name, no_fatal=True)
        if ip == None:
            return
        common.info(f"Deleting compiled code objects for external IP '{ip_name}'")
        common.remove_dir(cfg.sim_output_dir + '/vivado/external/'  + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/metrics/external/' + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/vcs/external/'     + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/xcelium/external/' + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/questa/external/'  + ip_name)
        common.remove_dir(cfg.sim_output_dir + '/riviera/external/' + ip_name)
        ip['compiled']['vivado' ] = False
        ip['compiled']['metrics'] = False
        ip['compiled']['vcs'    ] = False
        ip['compiled']['xcelium'] = False
        ip['compiled']['questa' ] = False
        ip['compiled']['riviera'] = False
