# Copyright 2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


import jinja2
from jinja2 import Template
from mio import common
from mio import cfg
import re
import os
from datetime import date


uvm_gen_dir               = re.sub("init.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/"
project_toml_file_path    = relative_path_to_template + "mio.toml"
dv_ip_file_path           = relative_path_to_template + "dv_ip.yml"
rtl_ip_file_path          = relative_path_to_template + "rtl_ip.yml"
vivado_ip_file_path       = relative_path_to_template + "vivado_ip.yml"


def new_project():
    wd = cfg.pwd
    common.dbg(f"Using {wd} as working directory")
    common.banner("Welcome to the Moore.io Project Initializer v1.0")
    project_name = ""
    copyright_owner = ""
    while project_name == "":
        project_name = common.prompt("Please enter the project name:").strip().lower()
    while copyright_owner == "":
        copyright_owner = common.prompt("Please enter the name of the copyright owner:").strip().lower()
    try:
        fin = open(project_toml_file_path, "rt")
        data = fin.read()
        template = Template(data)
        data = template.render(name=project_name, year=date.today().year, name_of_copyright_owner=copyright_owner)
        fout_path = wd + "/mio.toml"
        fout = open(fout_path, "w+")
        common.dbg(f"Writing to {fout_path}")
        fout.write(data)
        fout.close()
        common.create_dir (wd + "/dft"           )
        common.create_dir (wd + "/docs"          )
        common.create_dir (wd + "/dv"            )
        common.create_dir (wd + "/lint"          )
        common.create_dir (wd + "/rtl"           )
        common.create_dir (wd + "/sim"           )
        common.create_file(wd + "/sim/.gitignore")
        common.create_dir (wd + "/syn"           )
        common.create_dir (wd + "/tools"         )
    except Exception as e:
        common.fatal("Failed to create project structure: " + str(e))
    common.info("Project created successfully.")


def new_ip():
    wd = cfg.pwd
    common.dbg(f"Using {wd} as working directory")
    common.info("Welcome to the Moore.io IP Initializer v1.0")
    common.info("  The following IP types are available:")
    common.info("  [0] - DV  IP")
    common.info("  [1] - RTL IP")
    common.info("  [2] - Vivado Project IP")
    ip_type = -1
    while ip_type != 0 and ip_type != 1 and ip_type != 2:
        ip_type = common.prompt("Please select the type of IP:").strip().lower()
        ip_type = int(ip_type)
    ip_name = ""
    copyright_owner = ""
    while ip_name == "":
        ip_name = common.prompt("Please enter the IP name:").strip().lower()
    while copyright_owner == "":
        copyright_owner = common.prompt("Please enter the name of the copyright owner:").strip().lower()
    
    try:
        if ip_type == 0:
            fin_path = dv_ip_file_path
            common.create_dir(wd + "/bin")
            common.create_dir(wd + "/docs")
            common.create_dir(wd + "/examples")
            common.create_dir(wd + "/src")
        elif ip_type == 1:
            fin_path = rtl_ip_file_path
            common.create_dir(wd + "/bin")
            common.create_dir(wd + "/docs")
            common.create_dir(wd + "/examples")
            common.create_dir(wd + "/src")
        elif ip_type == 2:
            fin_path = vivado_ip_file_path
        else:
            common.fatal("Invalid IP type: " + ip_type)
        fin = open(fin_path, "rt")
        data = fin.read()
        template = Template(data)
        data = template.render(name=ip_name, year=date.today().year, name_of_copyright_owner=copyright_owner)
        fout_path = wd + "/ip.yml"
        fout = open(fout_path, "w+")
        common.dbg(f"Writing to {fout_path}")
        fout.write(data)
        fout.close()
    except Exception as e:
        common.fatal("Failed to create IP structure: " + str(e))
    common.info("IP created successfully.")
