# Copyright 2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


from mio import common
from mio import cfg
from mio import cov
from mio import dox
from mio import results
from mio import sim
from gen import new_agent_advanced_serial
from gen import new_agent_advanced_parallel
from gen import new_agent_basic
from gen import new_block
from gen import new_lib
from gen import new_singleton
from gen import new_ss
from gen import new_ral

import jinja2
from jinja2 import Template
import zipfile
import shutil
import os
import sys


templates = ["basic_agent","parallel_agent","serial_agent","block_tb","ss_tb","lib","ral",
             "comp","obj","reg_adapter","reg_block","reg","seq_lib","seq","test","vseq_lib","vseq"]
singleton_names = ["component","object","reg_adapter","reg_block","reg","seq_lib","seq","test","vseq_lib","vseq"]

def menu(template_name=""):
    global singleton_names
    template_num = 0
    prompt = True
    if template_name != "":
        template_num = 0
        for template in templates:
            if template_name == template:
                prompt = False
                break
            template_num += 1
    else:
        prompt = True
    if prompt:
        common.info("***********************************************************************************************************************")
        common.info("  The following Moore.io UVM Framework templates are available:")
        common.info("***********************************************************************************************************************")
        common.info("  [ 0] - UVM Agent - Basic")
        common.info("  [ 1] - UVM Agent - Advanced, Parallel")
        common.info("  [ 2] - UVM Agent - Advanced, Serial")
        common.info("  [ 3] - UVM Block-Level Environment & Test Bench")
        common.info("  [ 4] - UVM Sub-System Environment & Test Bench")
        common.info("  [ 5] - UVM Library")
        common.info("  [ 6] - UVM Register Model (from .csv)")
        common.info("-------------------------------------------------")
        common.info("  [ 7] - UVM Component")
        common.info("  [ 8] - UVM Object")
        common.info("  [ 9] - UVM Register Adapter")
        common.info("  [10] - UVM Register Block")
        common.info("  [11] - UVM Register")
        common.info("  [12] - UVM Sequence Library")
        common.info("  [13] - UVM Sequence")
        common.info("  [14] - UVM Test")
        common.info("  [15] - UVM Virtual Sequence Library")
        common.info("  [16] - UVM Virtual Sequence")
        template_num_str = common.prompt("Please enter the index of the template you wish to run: ").strip()
        template_num = int(template_num_str)
    
    if template_num == 0:
        new_agent_basic.main()
    elif template_num == 1:
        new_agent_advanced_parallel.main()
    elif template_num == 2:
        new_agent_advanced_serial.main()
    elif template_num == 3:
        new_block.main()
    elif template_num == 4:
        new_ss.main()
    elif template_num == 5:
        new_lib.main()
    elif template_num == 6:
        new_ral.main()
        pass
    else:
        if template_num >= 7 and template_num <= 16:
            new_singleton.main(singleton_names[template_num-7])
        else:
            common.fatal("Invalid option: " + str(template_num))


def j2_to_vsdx(parameters, input_path, output_path):
    with zipfile.ZipFile(input_path) as inzip, zipfile.ZipFile(output_path, "w") as outzip:
        # Iterate the input files
        for inzipinfo in inzip.infolist():
            # Read input file
            with inzip.open(inzipinfo) as infile:
                if inzipinfo.filename == "visio/pages/page1.xml":
                    common.dbg("Processing vsdx file " + output_path)
                    content = infile.read()
                    content = str(content, 'utf-8')
                    #content = str(jinja2.escape(content))
                    #template = Template(content)
                    #content = template.render(parameters)
                    for param in parameters:
                        content = content.replace("{{ " + param + " }}", str(parameters[param]))
                    # Write content
                    outzip.writestr(inzipinfo.filename, content)
                else:
                    outzip.writestr(inzipinfo.filename, infile.read())

