# Hello from your Code Generator!
Thank you for using the Moore.io Advanced UVM Agent Code Template for Parallel protocols v1.0!

Your parameters are:
* Name: '{{ name }}'
* Full Name: '{{ full_name }}'
* Memory Mapped?: Yes
* Byte Enabled?: {% if sbe %}Yes{% else %}No{% endif %}
* Mode 1: '{{ mode_1 }}'
* Mode 2: '{{ mode_2 }}'
* Number of Channels: '{{ num_segments }}'
{% for segment in segments %}* Channel {{ segment.index }} - Name: '{{ segment.name }}'
* Channel {{ segment.index }} - Tx: '{{ segment.tx }}'
* Channel {{ segment.index }} - Rx: '{{ segment.rx }}'
{% endfor %}

If this is incorrect, it is recommended to delete the generated IP and re-generate with the correct parameters.


This code template is based on the ['Advanced' UVM methodology](https://www.linkedin.com/pulse/advanced-uvm-brian-hunter/).
This agent uses multiple Sequencers and Virtual Sequences to harness the full power of UVM and elegantly implement parallel
protocols.  It can also be combined with other Advanced UVM template instances to create VIP 'stacks'.

While the `rand_stim` test will pass out-of-the-box (scoreboards are disabled), there is some work to be done before your
agent is ready to roll.  What follows is a short developer guide; use `grep -nr TODO .` for a full list.


## 1 - Physical interface
 1. - [ ] Physical Interface - `uvma_{{ name }}_if.sv` - Add the signals from your protocol's physical interface definition.
{% for segment in segments %} 1. - [ ] '{{ mode_1.upper() }}' '{{ segment.name.upper() }}' Monitor Transaction - `obj/uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn.sv` - Add fields to match your interface definition.
 1. - [ ] '{{ mode_2.upper() }}' '{{ segment.name.upper() }}' Monitor Transaction - `obj/uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn.sv` - Add fields to match your interface definition.
 1. - [ ] '{{ mode_1.upper() }}' '{{ segment.name.upper() }}' Sequence Item - `seq/uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item.sv` - Add fields to match your interface definition.
 1. - [ ] '{{ mode_2.upper() }}' '{{ segment.name.upper() }}' Sequence Item - `seq/uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item.sv` - Add fields to match your interface definition.
 1. - [ ] '{{ mode_1.upper() }}' '{{ segment.name.upper() }}' Driver - `comps/uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_drv.sv` - Add code to drive the modport signals.
 1. - [ ] '{{ mode_2.upper() }}' '{{ segment.name.upper() }}' Driver - `comps/uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_drv.sv` - Add code to drive the modport signals.
{% endfor %} 1. - [ ] Monitor - `comps/uvma_{{ name }}_mon.sv` - Add code to sample the modports signals for each channel.

## 2 - Logical Transaction Model
 1. - [ ] Sequence Item - `seq/uvma_{{ name }}_seq_item.sv` - Modify the transaction to match your protocol model.
 1. - [ ] Monitor Transaction - `obj/uvma_{{ name }}_mon_trn.sv` - Modify the transaction to match your protocol model.
 1. - [ ] Register Adapter - `reg/uvma_{{ name }}_reg_adapter.sv` - Modify the register adapter to match your Sequence Item.


## 3 - Protocol Implementation
 1. - [ ] Configuration - `obj/uvma_{{ name }}_cfg.sv` - Add parameters if needed for protocol options.
 1. - [ ] Context - `obj/uvma_{{ name }}_cntxt.sv` - Add state variables if needed for protocol implementation.
 1. - [ ] '{{ mode_1.upper() }}' Driver Virtual Sequence - `seq/uvma_{{ name }}_{{ mode_1 }}_drv_vseq.sv` - Implement the Sequence driving the '{{ mode_1.upper() }}' interface.
 1. - [ ] '{{ mode_2.upper() }}' Driver Virtual Sequence - `seq/uvma_{{ name }}_{{ mode_2 }}_drv_vseq.sv` - Implement the Sequence driving the '{{ mode_2.upper() }}' interface.
 1. - [ ] Memory Handler Virtual Sequence - `seq/uvma_{{ name }}_{{ mode_2 }}_handler_mem_vseq.sv` - Implement the Sequence modeling a {{ mode_2.upper() }} memory.
 1. - [ ] Monitor Virtual Sequence - `seq/uvma_{{ name }}_mon_vseq.sv` - Implement the Sequence rebuilding Monitor Transactions from the interface.
 1. - [ ] Idle Virtual Sequence - `seq/uvma_{{ name }}_idle_vseq.sv` - Implement Sequence driving the interface on idle cycles.


## Have fun!
The codebase is ready to test using the other 2 IPs generated along with this agent:
* `uvme_{{ name }}_st`
* `uvmt_{{ name }}_st`




# {{ name_of_copyright_owner }} {{ full_name }} UVM Agent

# About
This IP contains the {{ name_of_copyright_owner }} {{ full_name }} UVM Agent.
TODO Describe {{ full_name }} UVM Agent

# Block Diagram
![alt text](./docs/agent_block_diagram.png "{{ full_name }} UVM Agent Block Diagram")

# Directory Structure
* `bin` - Scripts, metadata and other miscellaneous files
* `docs` - Documents describing the {{ full_name }} UVM Agent
* `examples` - Code samples for using and extending this agent
* `src` - Source code


# Dependencies
It is dependent on the following IP:

* `uvm`
* `uvml`
* `uvml_logs`
