// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_AGENT_SV__
`define __UVMA_{{ name.upper() }}_AGENT_SV__


/**
 * Top-level component that encapsulates, builds and connects all others.  Capable of driving/monitoring the {{ full_name }}.
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_agent_c extends uvml_agent_c;

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}

   /// @name Components
   /// @{
   uvma_{{ name }}_vsqr_c       vsequencer; ///< Provides sequence items for #driver.
   uvma_{{ name }}_drv_c        driver    ; ///< Drives Virtual Interface (uvma_{{ name }}_if) with stimulus obtained from #sequencer.
   uvma_{{ name }}_mon_c        monitor   ; ///< Samples transactions from Virtual Interface (uvma_{{ name }}_if).
   uvma_{{ name }}_cov_model_c  cov_model ; ///< Functional coverage model sampling #cfg, #cntxt and all transactions.
   uvma_{{ name }}_logger_c     logger    ; ///< Transaction logger for #vsequencer, #driver and #monitor.
   /// @}

   /// @name TLM
   /// @{
   uvm_seq_item_pull_port #(uvm_sequence_item)  upstream_sqr_port; ///< Upstream Sequencer Port for transport sequences.
   uvm_analysis_port      #(uvma_{{ name }}_seq_item_c)  seq_item_ap; ///< Output Port for Sequence Items.
   {% if duplex %}uvm_analysis_port      #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_mon_trn_ap ; ///< Output Port for {{ dir_1.upper() }} Monitor Transactions.
   uvm_analysis_port      #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_mon_trn_ap ; ///< Output Port for {{ dir_2.upper() }} Monitor Transactions.
   {% else %}uvm_analysis_port      #(uvma_{{ name }}_mon_trn_c )  mon_trn_ap ; ///< Output Port for Monitor Transactions.
   {% endif %}{% for segment in segments %}uvm_analysis_port      #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_seq_item_ap; ///< Output Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items from #driver.
   {% endfor %}{% for segment in segments %}uvm_analysis_port      #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_seq_item_ap; ///< Output Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items from #driver.
   {% endfor %}{% for segment in segments %}uvm_analysis_port      #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_mon_trn_ap; ///< Output Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transactions from #monitor.
   {% endfor %}{% for segment in segments %}uvm_analysis_port      #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_mon_trn_ap; ///< Output Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transactions from #monitor.
   {% endfor %}/// @}

   `uvm_component_utils_begin(uvma_{{ name }}_agent_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_agent", uvm_component parent=null);

   /**
    * 1. Ensures cfg & cntxt handles are not null
    * 2. Builds all components
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * 1. Links agent's analysis ports to sub-components'
    * 2. Connects coverage models and loggers
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Starts Virtual Sequences on #vsequencer vital both for driving and monitoring.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg and hand out to sub-components.
    */
   extern function void get_and_set_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt and hand out to sub-components.
    */
   extern function void get_and_set_cntxt();

   /**
    * Uses uvm_config_db to retrieve the Virtual Interface (vif) associated with this agent.
    */
   extern function void retrieve_vif();

   /**
    * Creates sub-components.
    */
   extern function void create_components();

   /**
    * Connects sequencer to monitor and driver's TLM ports.
    */
   extern function void connect_sequencer;

   /**
    * Connects agent's TLM ports to driver's and monitor's.
    */
   extern function void connect_analysis_ports();

   /**
    * Connects coverage model to monitor and driver's analysis ports.
    */
   extern function void connect_cov_model();

   /**
    * Connects transaction logger to monitor and driver's analysis ports.
    */
   extern function void connect_logger();

   /**
    * Starts Monitoring Virtual Sequence.
    */
   extern task start_mon_vseq();

   /**
    * Starts IDLE Generating Virtual Sequence.
    */
   extern task start_idle_vseq();

   /**
    * Starts {{ mode_1.upper() }} Driver Virtual Sequence.
    */
   extern task start_{{ mode_1 }}_drv_vseq();

   /**
    * Starts {{ mode_2.upper() }} Driver Virtual Sequence.
    */
   extern task start_{{ mode_2 }}_drv_vseq();

endclass : uvma_{{ name }}_agent_c


function uvma_{{ name }}_agent_c::new(string name="uvma_{{ name }}_agent", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_agent_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_and_set_cfg  ();
   get_and_set_cntxt();
   retrieve_vif     ();
   create_components();

endfunction : build_phase


function void uvma_{{ name }}_agent_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_sequencer     ();
   connect_analysis_ports();
   if (cfg.cov_model_enabled) begin
      connect_cov_model();
   end
   if (cfg.trn_log_enabled) begin
      connect_logger();
   end

endfunction: connect_phase


task uvma_{{ name }}_agent_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   if (cfg.enabled) begin
      start_mon_vseq();
      if (cfg.is_active) begin
         start_idle_vseq();
         case (cfg.drv_mode)
            UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_1.upper() }}: start_{{ mode_1 }}_drv_vseq();
            UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_2.upper() }}: start_{{ mode_2 }}_drv_vseq();
            default: begin
               `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Invalid cfg.drv_mode: %s", cfg.drv_mode.name()))
            end
         endcase
      end
   end

endtask : run_phase


function void uvma_{{ name }}_agent_c::get_and_set_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", "Configuration handle is null")
   end
   else begin
      `uvm_info("{{ name.upper() }}_AGENT", $sformatf("Found configuration handle:\n%s", cfg.sprint()), UVM_DEBUG)
      uvm_config_db#(uvma_{{ name }}_cfg_c)::set(this, "*", "cfg", cfg);
   end

endfunction : get_and_set_cfg


function void uvma_{{ name }}_agent_c::get_and_set_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_info("{{ name.upper() }}_AGENT", "Context handle is null; creating.", UVM_DEBUG)
      cntxt = uvma_{{ name }}_cntxt_c::type_id::create("cntxt");
   end
   uvm_config_db#(uvma_{{ name }}_cntxt_c)::set(this, "*", "cntxt", cntxt);

endfunction : get_and_set_cntxt


function void uvma_{{ name }}_agent_c::retrieve_vif();

   if (!uvm_config_db#(virtual uvma_{{ name }}_if)::get(this, "", "vif", cntxt.vif)) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Could not find vif handle of type %s in uvm_config_db", $typename(cntxt.vif)))
   end
   else begin
      `uvm_info("{{ name.upper() }}_AGENT", $sformatf("Found vif handle of type %s in uvm_config_db", $typename(cntxt.vif)), UVM_DEBUG)
   end

endfunction : retrieve_vif


function void uvma_{{ name }}_agent_c::create_components();

   monitor    = uvma_{{ name }}_mon_c      ::type_id::create("monitor"   , this);
   vsequencer = uvma_{{ name }}_vsqr_c     ::type_id::create("vsequencer", this);
   driver     = uvma_{{ name }}_drv_c      ::type_id::create("driver"    , this);
   cov_model  = uvma_{{ name }}_cov_model_c::type_id::create("cov_model" , this);
   logger     = uvma_{{ name }}_logger_c   ::type_id::create("logger"    , this);

endfunction : create_components


function void uvma_{{ name }}_agent_c::connect_sequencer();

   vsequencer.set_arbitration(cfg.sqr_arb_mode);
   {% for segment in segments %}vsequencer.{{ mode_1 }}_{{ segment.name }}_sequencer.set_arbitration(UVM_SEQ_ARB_STRICT_FIFO);
   {% endfor %}{% for segment in segments %}vsequencer.{{ mode_2 }}_{{ segment.name }}_sequencer.set_arbitration(UVM_SEQ_ARB_STRICT_FIFO);
   {% endfor %}if (!cfg.bypass_mode) begin
   {% for segment in segments %}   driver.{{ mode_1 }}_{{ segment.name }}_driver.seq_item_port.connect(vsequencer.{{ mode_1 }}_{{ segment.name }}_sequencer.seq_item_export);
   {% endfor %}{% for segment in segments %}   driver.{{ mode_2 }}_{{ segment.name }}_driver.seq_item_port.connect(vsequencer.{{ mode_2 }}_{{ segment.name }}_sequencer.seq_item_export);
   {% endfor %}{% for segment in segments %}   monitor.{{ mode_1 }}_{{ segment.name }}_ap.connect(vsequencer.{{ mode_1 }}_{{ segment.name }}_mon_trn_export);
   {% endfor %}{% for segment in segments %}   monitor.{{ mode_2 }}_{{ segment.name }}_ap.connect(vsequencer.{{ mode_2 }}_{{ segment.name }}_mon_trn_export);
   {% endfor %}end

endfunction : connect_sequencer


function void uvma_{{ name }}_agent_c::connect_analysis_ports();

   upstream_sqr_port = vsequencer.upstream_sqr_port;
   {% if duplex %}{{ dir_1 }}_mon_trn_ap = vsequencer.{{ dir_1 }}_mon_trn_ap;
   {{ dir_2 }}_mon_trn_ap = vsequencer.{{ dir_2 }}_mon_trn_ap;
   {% else %}mon_trn_ap        = vsequencer.mon_trn_ap       ;
   {% endif %}seq_item_ap       = vsequencer.seq_item_ap      ;
   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_ap = driver.{{ mode_1 }}_{{ segment.name }}_driver.ap;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_ap = driver.{{ mode_2 }}_{{ segment.name }}_driver.ap;
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_ap = monitor.{{ mode_1 }}_{{ segment.name }}_ap;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_ap = monitor.{{ mode_2 }}_{{ segment.name }}_ap;
   {% endfor %}
endfunction : connect_analysis_ports


function void uvma_{{ name }}_agent_c::connect_cov_model();

   seq_item_ap.connect(cov_model.seq_item_fifo.analysis_export);
   {% if duplex %}{{ dir_1 }}_mon_trn_ap.connect(cov_model.{{ dir_1 }}_mon_trn_fifo.analysis_export);
   {{ dir_2 }}_mon_trn_ap.connect(cov_model.{{ dir_2 }}_mon_trn_fifo.analysis_export);
   {% else %}mon_trn_ap .connect(cov_model.mon_trn_fifo .analysis_export);
   {% endif %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_ap.connect(cov_model.{{ mode_1 }}_{{ segment.name }}_seq_item_fifo.analysis_export);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_ap.connect(cov_model.{{ mode_2 }}_{{ segment.name }}_seq_item_fifo.analysis_export);
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_ap.connect(cov_model.{{ mode_1 }}_{{ segment.name }}_mon_trn_fifo.analysis_export);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_ap.connect(cov_model.{{ mode_2 }}_{{ segment.name }}_mon_trn_fifo.analysis_export);
   {% endfor %}
endfunction : connect_cov_model


function void uvma_{{ name }}_agent_c::connect_logger();

   seq_item_ap.connect(logger.seq_item_export);
   {% if duplex %}{{ dir_1 }}_mon_trn_ap.connect(logger.{{ dir_1 }}_mon_trn_export);
   {{ dir_2 }}_mon_trn_ap.connect(logger.{{ dir_2 }}_mon_trn_export);
   {% else %}mon_trn_ap .connect(logger.mon_trn_export);
   {% endif %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_ap.connect(logger.{{ mode_1 }}_{{ segment.name }}_seq_item_export);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_ap.connect(logger.{{ mode_2 }}_{{ segment.name }}_seq_item_export);
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_ap.connect(logger.{{ mode_1 }}_{{ segment.name }}_mon_trn_export);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_ap.connect(logger.{{ mode_2 }}_{{ segment.name }}_mon_trn_export);
   {% endfor %}
endfunction : connect_logger


task uvma_{{ name }}_agent_c::start_mon_vseq();

   uvm_coreservice_t cs = uvm_coreservice_t::get();
   uvm_factory       f  = cs.get_factory();
   uvm_object        temp_obj;
   temp_obj = f.create_object_by_type(cfg.mon_vseq_type, get_full_name(), cfg.mon_vseq_type.get_type_name());
   if (!$cast(cntxt.mon_vseq, temp_obj)) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Could not cast 'temp_obj' (%s) to 'cntxt.mon_vseq' (%s)", $typename(temp_obj), $typename(cntxt.mon_vseq)))
   end
   if (!cntxt.mon_vseq.randomize()) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", "Failed to randomize cntxt.mon_vseq")
   end
   fork
      cntxt.mon_vseq.start(vsequencer);
   join_none

endtask : start_mon_vseq


task uvma_{{ name }}_agent_c::start_idle_vseq();

   uvm_coreservice_t cs = uvm_coreservice_t::get();
   uvm_factory       f  = cs.get_factory();
   uvm_object        temp_obj;
   temp_obj = f.create_object_by_type(cfg.idle_vseq_type, get_full_name(), cfg.idle_vseq_type.get_type_name());
   if (!$cast(cntxt.idle_vseq, temp_obj)) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Could not cast 'temp_obj' (%s) to 'cntxt.idle_vseq' (%s)", $typename(temp_obj), $typename(cntxt.idle_vseq)))
   end
   if (!cntxt.idle_vseq.randomize()) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", "Failed to randomize cntxt.idle_vseq")
   end
   fork
      cntxt.idle_vseq.start(vsequencer);
   join_none

endtask : start_idle_vseq


task uvma_{{ name }}_agent_c::start_{{ mode_1 }}_drv_vseq();

   uvm_coreservice_t cs = uvm_coreservice_t::get();
   uvm_factory       f  = cs.get_factory();
   uvm_object        temp_obj;
   temp_obj = f.create_object_by_type(cfg.{{ mode_1 }}_drv_vseq_type, get_full_name(), cfg.{{ mode_1 }}_drv_vseq_type.get_type_name());
   if (!$cast(cntxt.{{ mode_1 }}_drv_vseq, temp_obj)) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Could not cast 'temp_obj' (%s) to 'cntxt.{{ mode_1 }}_drv_vseq' (%s)", $typename(temp_obj), $typename(cntxt.{{ mode_1 }}_drv_vseq)))
   end
   if (!cntxt.{{ mode_1 }}_drv_vseq.randomize()) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", "Failed to randomize cntxt.{{ mode_1 }}_drv_vseq")
   end
   fork
      cntxt.{{ mode_1 }}_drv_vseq.start(vsequencer);
   join_none

endtask : start_{{ mode_1 }}_drv_vseq


task uvma_{{ name }}_agent_c::start_{{ mode_2 }}_drv_vseq();

   uvm_coreservice_t cs = uvm_coreservice_t::get();
   uvm_factory       f  = cs.get_factory();
   uvm_object        temp_obj;
   temp_obj = f.create_object_by_type(cfg.{{ mode_2 }}_drv_vseq_type, get_full_name(), cfg.{{ mode_2 }}_drv_vseq_type.get_type_name());
   if (!$cast(cntxt.{{ mode_2 }}_drv_vseq, temp_obj)) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", $sformatf("Could not cast 'temp_obj' (%s) to 'cntxt.{{ mode_2 }}_drv_vseq' (%s)", $typename(temp_obj), $typename(cntxt.{{ mode_2 }}_drv_vseq)))
   end
   if (!cntxt.{{ mode_2 }}_drv_vseq.randomize()) begin
      `uvm_fatal("{{ name.upper() }}_AGENT", "Failed to randomize cntxt.{{ mode_2 }}_drv_vseq")
   end
   fork
      cntxt.{{ mode_2 }}_drv_vseq.start(vsequencer);
   join_none

endtask : start_{{ mode_2 }}_drv_vseq


`endif // __UVMA_{{ name.upper() }}_AGENT_SV__
