// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_COV_MODEL_SV__
`define __UVMA_{{ name.upper() }}_COV_MODEL_SV__


/**
 * Abstract component providing a base class for {{ full_name }} Agent functional coverage models.
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_cov_model_c extends uvm_component;

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}

   /// @name Covergroup variables
   /// @{
   uvma_{{ name }}_seq_item_c  seq_item; ///< Sequence Item currently being sampled.
   {% if duplex %}uvma_{{ name }}_mon_trn_c  {{ dir_1 }}_mon_trn; ///< {{ dir_1.upper() }} Monitor Transaction currently being sampled.
   uvma_{{ name }}_mon_trn_c  {{ dir_2 }}_mon_trn; ///< {{ dir_2.upper() }} Monitor Transaction currently being sampled.
   {% else %}uvma_{{ name }}_mon_trn_c  mon_trn; ///< Monitor Transaction currently being sampled.
   {% endif %}{% for segment in segments %}uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c  {{ mode_1 }}_{{ segment.name }}_seq_item; ///< {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Item currently being sampled.
   {% endfor %}{% for segment in segments %}uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c  {{ mode_2 }}_{{ segment.name }}_seq_item; ///< {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Item currently being sampled.
   {% endfor %}{% for segment in segments %}uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c  {{ mode_1 }}_{{ segment.name }}_mon_trn; ///< {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transaction currently being sampled.
   {% endfor %}{% for segment in segments %}uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c  {{ mode_2 }}_{{ segment.name }}_mon_trn; ///< {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transaction currently being sampled.
   {% endfor %}/// @}

   /// @name TLM FIFOs
   /// @{
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_seq_item_c)  seq_item_fifo; ///< FIFO of Sequence Items to be sampled.
   {% if duplex %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c)  {{ dir_1 }}_mon_trn_fifo; ///< FIFO of {{ dir_1.upper() }} Monitor Transactions to be sampled.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c)  {{ dir_2 }}_mon_trn_fifo; ///< FIFO of {{ dir_2.upper() }} Monitor Transactions to be sampled.
   {% else %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c)  mon_trn_fifo; ///< FIFO of Monitor Transactions to be sampled.
   {% endif %}{% for segment in segments %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_seq_item_fifo; ///< FIFO of {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items to be sampled.
   {% endfor %}{% for segment in segments %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_seq_item_fifo; ///< FIFO of {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items to be sampled.
   {% endfor %}{% for segment in segments %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_mon_trn_fifo; ///< FIFO of {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transaction to be sampled.
   {% endfor %}{% for segment in segments %}uvm_tlm_analysis_fifo #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_mon_trn_fifo; ///< FIFO of {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transaction to be sampled.
   {% endfor %}/// @}

   /// @name TLM Exports
   /// @{
   uvm_analysis_export #(uvma_{{ name }}_seq_item_c)  seq_item_export; ///< Input Port for Sequence Items to be sampled.
   {% if duplex %}uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_1 }}_mon_trn_export; ///< Input Port for {{ dir_1.upper() }} Monitor Transactions to be sampled.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_2 }}_mon_trn_export; ///< Input Port for {{ dir_2.upper() }} Monitor Transactions to be sampled.
   {% else %}uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  mon_trn_export; ///< Input Port for Monitor Transactions to be sampled.
   {% endif %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_seq_item_export; ///< Input Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items to be sampled.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_seq_item_export; ///< Input Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items to be sampled.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_mon_trn_export; ///< Input Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transactions to be sampled.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_mon_trn_export; ///< Input Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transactions to be sampled.
   {% endfor %}/// @}

   `uvm_component_utils_begin(uvma_{{ name }}_cov_model_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_cov_model", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Builds FIFOs.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects TLM Objects.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Forks all sampling loops
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates Exports and FIFOs.
    */
   extern function void create_tlm_objects();

   /**
    * Connects Exports to FIFOs.
    */
   extern function void connect_fifos();

   /**
    * User hook for sampling #cfg.
    */
   extern virtual function void sample_cfg();

   /**
    * User hook for sampling #cntxt.
    */
   extern virtual function void sample_cntxt();

   /**
    * User hook for sampling #seq_item.
    */
   extern virtual function void sample_seq_item();

   {% if duplex %}/**
    * User hook for sampling #{{ dir_1 }}_mon_trn.
    */
   extern virtual function void sample_{{ dir_1 }}_mon_trn();

   /**
    * User hook for sampling #{{ dir_2 }}_mon_trn.
    */
   extern virtual function void sample_{{ dir_2 }}_mon_trn();
   {% else %}/**
    * User hook for sampling #mon_trn.
    */
   extern virtual function void sample_mon_trn();
{% endif %}
{% for segment in segments %}   /**
    * User hook for sampling #{{ mode_1 }}_{{ segment.name }}_seq_item.
    */
   extern virtual function void sample_{{ mode_1 }}_{{ segment.name }}_seq_item();

{% endfor %}{% for segment in segments %}   /**
    * User hook for sampling #{{ mode_2 }}_{{ segment.name }}_seq_item.
    */
   extern virtual function void sample_{{ mode_2 }}_{{ segment.name }}_seq_item();

{% endfor %}{% for segment in segments %}   /**
    * User hook for sampling #{{ mode_1 }}_{{ segment.name }}_mon_trn.
    */
   extern virtual function void sample_{{ mode_1 }}_{{ segment.name }}_mon_trn();

{% endfor %}{% for segment in segments %}   /**
    * User hook for sampling #{{ mode_2 }}_{{ segment.name }}_mon_trn.
    */
   extern virtual function void sample_{{ mode_2 }}_{{ segment.name }}_mon_trn();

{% endfor %}endclass : uvma_{{ name }}_cov_model_c


function uvma_{{ name }}_cov_model_c::new(string name="uvma_{{ name }}_cov_model", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_cov_model_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg           ();
   get_cntxt         ();
   create_tlm_objects();

endfunction : build_phase


function void uvma_{{ name }}_cov_model_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_fifos();

endfunction : connect_phase


task uvma_{{ name }}_cov_model_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   if (cfg.enabled && cfg.cov_model_enabled) begin
      fork
         forever begin
            cntxt.sample_cfg_e.wait_trigger();
            sample_cfg();
         end
         forever begin
            cntxt.sample_cntxt_e.wait_trigger();
            sample_cntxt();
         end
         forever begin
            seq_item_fifo.get(seq_item);
            sample_seq_item();
         end
{% if duplex %}         forever begin
            {{ dir_1 }}_mon_trn_fifo.get({{ dir_1 }}_mon_trn);
            sample_{{ dir_1 }}_mon_trn();
         end
         forever begin
            {{ dir_2 }}_mon_trn_fifo.get({{ dir_2 }}_mon_trn);
            sample_{{ dir_2 }}_mon_trn();
         end
{% else %}         forever begin
            mon_trn_fifo.get(mon_trn);
            sample_mon_trn();
         end
{% endif %}{% for segment in segments %}         forever begin
            {{ mode_1 }}_{{ segment.name }}_seq_item_fifo.get({{ mode_1 }}_{{ segment.name }}_seq_item);
            sample_{{ mode_1 }}_{{ segment.name }}_seq_item();
         end
{% endfor %}{% for segment in segments %}         forever begin
            {{ mode_2 }}_{{ segment.name }}_seq_item_fifo.get({{ mode_2 }}_{{ segment.name }}_seq_item);
            sample_{{ mode_2 }}_{{ segment.name }}_seq_item();
         end
{% endfor %}{% for segment in segments %}         forever begin
            {{ mode_1 }}_{{ segment.name }}_mon_trn_fifo.get({{ mode_1 }}_{{ segment.name }}_mon_trn);
            sample_{{ mode_1 }}_{{ segment.name }}_mon_trn();
         end
{% endfor %}{% for segment in segments %}         forever begin
            {{ mode_2 }}_{{ segment.name }}_mon_trn_fifo.get({{ mode_2 }}_{{ segment.name }}_mon_trn);
            sample_{{ mode_2 }}_{{ segment.name }}_mon_trn();
         end
{% endfor %}      join_none
   end

endtask : run_phase


function void uvma_{{ name }}_cov_model_c::get_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_COV_MODEL", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvma_{{ name }}_cov_model_c::get_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_COV_MODEL", "Context handle is null")
   end

endfunction : get_cntxt


function void uvma_{{ name }}_cov_model_c::create_tlm_objects();

   seq_item_fifo = new("seq_item_fifo", this);
   {% if duplex %}{{ dir_1 }}_mon_trn_fifo = new("{{ dir_1 }}_mon_trn_fifo", this);
   {{ dir_2 }}_mon_trn_fifo = new("{{ dir_2 }}_mon_trn_fifo", this);
   {% else %}mon_trn_fifo = new("mon_trn_fifo", this);
   {% endif %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_fifo = new("{{ mode_1 }}_{{ segment.name }}_mon_trn_fifo", this);
   {{ mode_2 }}_{{ segment.name }}_mon_trn_fifo = new("{{ mode_2 }}_{{ segment.name }}_mon_trn_fifo", this);
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_fifo = new("{{ mode_1 }}_{{ segment.name }}_seq_item_fifo", this);
   {{ mode_2 }}_{{ segment.name }}_seq_item_fifo = new("{{ mode_2 }}_{{ segment.name }}_seq_item_fifo", this);
   {% endfor %}
endfunction : create_tlm_objects


function void uvma_{{ name }}_cov_model_c::connect_fifos();

   {% if duplex %}{{ dir_1 }}_mon_trn_export = {{ dir_1 }}_mon_trn_fifo.analysis_export;
   {{ dir_2 }}_mon_trn_export = {{ dir_2 }}_mon_trn_fifo.analysis_export;
   {% else %}mon_trn_export  = mon_trn_fifo .analysis_export;
   {% endif %}seq_item_export = seq_item_fifo.analysis_export;
   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_export = {{ mode_1 }}_{{ segment.name }}_mon_trn_fifo.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_export = {{ mode_2 }}_{{ segment.name }}_mon_trn_fifo.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_export = {{ mode_1 }}_{{ segment.name }}_seq_item_fifo.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_export = {{ mode_2 }}_{{ segment.name }}_seq_item_fifo.analysis_export;
   {% endfor %}
endfunction : connect_fifos


function void uvma_{{ name }}_cov_model_c::sample_cfg();
endfunction : sample_cfg


function void uvma_{{ name }}_cov_model_c::sample_cntxt();
endfunction : sample_cntxt


function void uvma_{{ name }}_cov_model_c::sample_seq_item();
endfunction : sample_seq_item


{% if duplex %}
function void uvma_{{ name }}_cov_model_c::sample_{{ dir_1 }}_mon_trn();
endfunction : sample_{{ dir_1 }}_mon_trn


function void uvma_{{ name }}_cov_model_c::sample_{{ dir_2 }}_mon_trn();
endfunction : sample_{{ dir_2 }}_mon_trn


{% else %}function void uvma_{{ name }}_cov_model_c::sample_mon_trn();
endfunction : sample_mon_trn

{% endif %}
{% for segment in segments %}function void uvma_{{ name }}_cov_model_c::sample_{{ mode_1 }}_{{ segment.name }}_seq_item();
endfunction : sample_{{ mode_1 }}_{{ segment.name }}_seq_item


{% endfor %}{% for segment in segments %}function void uvma_{{ name }}_cov_model_c::sample_{{ mode_2 }}_{{ segment.name }}_seq_item();
endfunction : sample_{{ mode_2 }}_{{ segment.name }}_seq_item


{% endfor %}{% for segment in segments %}function void uvma_{{ name }}_cov_model_c::sample_{{ mode_1 }}_{{ segment.name }}_mon_trn();
endfunction : sample_{{ mode_1 }}_{{ segment.name }}_mon_trn


{% endfor %}{% for segment in segments %}function void uvma_{{ name }}_cov_model_c::sample_{{ mode_2 }}_{{ segment.name }}_mon_trn();
endfunction : sample_{{ mode_2 }}_{{ segment.name }}_mon_trn


{% endfor %}`endif // __UVMA_{{ name.upper() }}_COV_MODEL_SV__
