// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_DRV_SV__
`define __UVMA_{{ name.upper() }}_DRV_SV__


/**
 * Component driving {{ full_name }} Interface (uvma_{{ name }}_if) in either direction.
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_drv_c extends uvm_component;

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   // @}

   /// @name Components
   /// @{
   {% for segment in segments %}uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_drv_c  {{ mode_1 }}_{{ segment.name }}_driver; ///< Drives Virtual Interface with {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items
   {% endfor %}{% for segment in segments %}uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_drv_c  {{ mode_2 }}_{{ segment.name }}_driver; ///< Drives Virtual Interface with {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items
   {% endfor %}/// @}

   /// @name TLM Ports
   /// @{
   {% for segment in segments %}uvm_analysis_port#(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_ap; ///< Output Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items after they've been driven
   {% endfor %}{% for segment in segments %}uvm_analysis_port#(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_ap; ///< Output Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items after they've been driven
   {% endfor %}/// @}

   `uvm_component_utils_begin(uvma_{{ name }}_drv_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_drv", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Creates sub-components.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects TLM Ports.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg and hand out to sub-components.
    */
   extern function void get_and_set_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt and hand out to sub-components.
    */
   extern function void get_and_set_cntxt();

   /**
    * Creates sub-drivers.
    */
   extern function void create_components();

   /**
    * Connects analysis ports to sub-drivers.
    */
   extern function void connect_ports();

endclass : uvma_{{ name }}_drv_c


function uvma_{{ name }}_drv_c::new(string name="uvma_{{ name }}_drv", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_drv_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_and_set_cfg  ();
   get_and_set_cntxt();
   create_components();

endfunction : build_phase


function void uvma_{{ name }}_drv_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_ports();

endfunction : connect_phase


function void uvma_{{ name }}_drv_c::get_and_set_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_DRV", "Configuration handle is null")
   end
   uvm_config_db#(uvma_{{ name }}_cfg_c)::set(this, "*", "cfg", cfg);

endfunction : get_and_set_cfg


function void uvma_{{ name }}_drv_c::get_and_set_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_DRV", "Context handle is null")
   end
   uvm_config_db#(uvma_{{ name }}_cntxt_c)::set(this, "*", "cntxt", cntxt);

endfunction : get_and_set_cntxt


function void uvma_{{ name }}_drv_c::create_components();

   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_driver = uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_drv_c::type_id::create("{{ mode_1 }}_{{ segment.name }}_driver", this);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_driver = uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_drv_c::type_id::create("{{ mode_2 }}_{{ segment.name }}_driver", this);
   {% endfor %}
endfunction : create_components


function void uvma_{{ name }}_drv_c::connect_ports();

   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_ap = {{ mode_1 }}_{{ segment.name }}_driver.ap;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_ap = {{ mode_2 }}_{{ segment.name }}_driver.ap;
   {% endfor %}
endfunction : connect_ports


`endif // __UVMA_{{ name.upper() }}_DRV_SV__
