// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_LOGGER_SV__
`define __UVMA_{{ name.upper() }}_LOGGER_SV__


/**
 * Component which logs to disk information of the transactions generated and monitored by uvma_{{ name }}_agent_c.
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_logger_c extends uvm_component;

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}

   /// @name Components
   /// @{
   uvml_logs_metadata_logger_c #(uvma_{{ name }}_seq_item_c)  seq_item_logger; ///< Logs Sequence Items from vsequencer.
   {% if duplex %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_mon_trn_logger; ///< Logs {{ dir_1.upper() }} Monitor Transactions from vsequencer.
   uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_mon_trn_logger; ///< Logs {{ dir_2.upper() }} Monitor Transactions from vsequencer.
   {% else %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )  mon_trn_logger; ///< Logs Monitor Transactions from vsequencer.
   {% endif %}{% for segment in segments %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_seq_item_logger; ///< Logs {{ segment.name.upper() }} Sequence Items from {{ mode_1.upper() }} Driver.
   {% endfor %}{% for segment in segments %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_seq_item_logger; ///< Logs {{ segment.name.upper() }} Sequence Items from {{ mode_2.upper() }} Driver.
   {% endfor %}{% for segment in segments %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_mon_trn_logger; ///< Logs {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transactions from Monitor.
   {% endfor %}{% for segment in segments %}uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_mon_trn_logger; ///< Logs {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transactions from Monitor.
   {% endfor %}/// @}

   /// @name TLM Exports
   /// @{
   uvm_analysis_export #(uvma_{{ name }}_seq_item_c)  seq_item_export; ///< Input Port for #seq_item_logger.
   {% if duplex %}uvm_analysis_export #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_mon_trn_export; ///< Input Port for #{{ dir_1 }}_mon_trn_logger.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_mon_trn_export; ///< Input Port for #{{ dir_2 }}_mon_trn_logger.
   {% else %}uvm_analysis_export #(uvma_{{ name }}_mon_trn_c )  mon_trn_export; ///< Input Port for #mon_trn_logger.
   {% endif %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)  {{ mode_1 }}_{{ segment.name }}_seq_item_export; ///< Input Port for #{{ mode_1 }}_{{ segment.name }}_seq_item_logger.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)  {{ mode_2 }}_{{ segment.name }}_seq_item_export; ///< Input Port for #{{ mode_2 }}_{{ segment.name }}_seq_item_logger.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_mon_trn_export; ///< Input Port for #{{ mode_1 }}_{{ segment.name }}_mon_trn_logger.
   {% endfor %}{% for segment in segments %}uvm_analysis_export #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_mon_trn_export; ///< Input Port for #{{ mode_2 }}_{{ segment.name }}_mon_trn_logger.
   {% endfor %}/// @}


   `uvm_component_utils_begin(uvma_{{ name }}_logger_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_logger", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Builds loggers.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects TLM ports.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates logger components.
    */
   extern function void create_components();

   /**
    * Sets filenames for logger components.
    */
   extern function void configure_loggers();

   /**
    * Connects TLM ports to logger components.
    */
   extern function void connect_loggers();

endclass : uvma_{{ name }}_logger_c


function uvma_{{ name }}_logger_c::new(string name="uvma_{{ name }}_logger", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_logger_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg  ();
   get_cntxt();
   create_components();

endfunction : build_phase


function void uvma_{{ name }}_logger_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   configure_loggers();
   connect_loggers  ();

endfunction : connect_phase


function void uvma_{{ name }}_logger_c::get_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_LOGGER", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvma_{{ name }}_logger_c::get_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_LOGGER", "Context handle is null")
   end

endfunction : get_cntxt


function void uvma_{{ name }}_logger_c::create_components();

   {% if duplex %}{{ dir_1 }}_mon_trn_logger  = uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )::type_id::create("{{ dir_1 }}_mon_trn_logger", this);
   {{ dir_2 }}_mon_trn_logger  = uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )::type_id::create("{{ dir_2 }}_mon_trn_logger", this);
   {% else %}mon_trn_logger  = uvml_logs_metadata_logger_c #(uvma_{{ name }}_mon_trn_c )::type_id::create("mon_trn_logger" , this);
   {% endif %}seq_item_logger = uvml_logs_metadata_logger_c #(uvma_{{ name }}_seq_item_c)::type_id::create("seq_item_logger", this);
   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_logger = uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c)::type_id::create("{{ mode_1 }}_{{ segment.name }}_mon_trn_logger", this);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_logger = uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c)::type_id::create("{{ mode_2 }}_{{ segment.name }}_mon_trn_logger", this);
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_logger = uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c)::type_id::create("{{ mode_1 }}_{{ segment.name }}_seq_item_logger", this);
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_logger = uvml_logs_metadata_logger_c #(uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c)::type_id::create("{{ mode_2 }}_{{ segment.name }}_seq_item_logger", this);
   {% endfor %}
endfunction : create_components


function void uvma_{{ name }}_logger_c::configure_loggers();

   {% if duplex %}{{ dir_1 }}_mon_trn_logger.set_file_name({get_parent().get_full_name(), ".{{ dir_1 }}_mon_trn" });
   {{ dir_2 }}_mon_trn_logger.set_file_name({get_parent().get_full_name(), ".{{ dir_2 }}_mon_trn" });
   {% else %}mon_trn_logger .set_file_name({get_parent().get_full_name(), ".mon_trn" });
   {% endif %}seq_item_logger.set_file_name({get_parent().get_full_name(), ".seq_item"});
   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_logger.set_file_name({get_parent().get_full_name(), ".{{ mode_1 }}_{{ segment.name }}.mon_trn"});
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_logger.set_file_name({get_parent().get_full_name(), ".{{ mode_2 }}_{{ segment.name }}.mon_trn"});
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_logger.set_file_name({get_parent().get_full_name(), ".{{ mode_1 }}_{{ segment.name }}.seq_item"});
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_logger.set_file_name({get_parent().get_full_name(), ".{{ mode_2 }}_{{ segment.name }}.seq_item"});
   {% endfor %}
endfunction : configure_loggers


function void uvma_{{ name }}_logger_c::connect_loggers();

   {% if duplex %}{{ dir_1 }}_mon_trn_export = {{ dir_1 }}_mon_trn_logger.analysis_export;
   {{ dir_2 }}_mon_trn_export = {{ dir_2 }}_mon_trn_logger.analysis_export;
   {% else %}mon_trn_export  = mon_trn_logger .analysis_export;
   {% endif %}seq_item_export = seq_item_logger.analysis_export;
   {% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_mon_trn_export = {{ mode_1 }}_{{ segment.name }}_mon_trn_logger.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_mon_trn_export = {{ mode_2 }}_{{ segment.name }}_mon_trn_logger.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_1 }}_{{ segment.name }}_seq_item_export = {{ mode_1 }}_{{ segment.name }}_seq_item_logger.analysis_export;
   {% endfor %}{% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_seq_item_export = {{ mode_2 }}_{{ segment.name }}_seq_item_logger.analysis_export;
   {% endfor %}
endfunction : connect_loggers


`endif // __UVMA_{{ name.upper() }}_LOGGER_SV__
