// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_MON_SV__
`define __UVMA_{{ name.upper() }}_MON_SV__


/**
 * Component sampling transactions from an {{ full_name }} (uvma_{{ name }}_if).
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_mon_c extends uvml_mon_c;

   /// @name Modports
   /// @{
{% for segment in segments %}   virtual uvma_{{ name }}_if.mon_{{ segment.name }}_mp  mp_{{ segment.name }}; ///< Handle to '{{ segment.name.upper() }}' modport
{% endfor %}   /// @}

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}

   /// @name TLM Ports
   /// @{
{% for segment in segments %}   uvm_analysis_port#(uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c)  {{ mode_1 }}_{{ segment.name }}_ap; ///< Output Port for {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor transactions.
{% endfor %}   {% for segment in segments %}uvm_analysis_port#(uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c)  {{ mode_2 }}_{{ segment.name }}_ap; ///< Output Port for {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor transactions.
   {% endfor %}/// @}

   `uvm_component_utils_begin(uvma_{{ name }}_mon_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_mon", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Builds TLM Ports.
    * 3. Retrieves modport handles from #cntxt.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Oversees monitoring, depending on the reset state, by calling `mon_x_y_<pre|in|post>_reset()` tasks.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates TLM Analysis POrts.
    */
   extern function void create_tlm_ports();

   /**
    * Retrieves modports from #cntxt.
    */
   extern function void retrieve_modports();

   /**
    * Updates the #cntxt.reset state.
    */
   extern task observe_reset();

   /**
    * Synchronous reset thread.
    */
   extern task observe_reset_sync();

   /**
    * Asynchronous reset thread.
    */
   extern task observe_reset_async();

{% for segment in segments %}   /**
    * Infinite loop that monitors #{{ segment.name }}_mp {{ segment.tx.upper() }} during run_phase().
    */
   extern task mon_{{ segment.tx }}_{{ segment.name }}();

{% endfor %}{% for segment in segments %}   /**
    * Infinite loop that monitors #{{ segment.name }}_mp {{ segment.rx.upper() }} during run_phase().
    */
   extern task mon_{{ segment.rx }}_{{ segment.name }}();

{% endfor %}{% for segment in segments %}   /**
    * Waits out each {{ segment.tx.upper() }} {{ segment.name.upper() }} clock cycle before reset.
    */
   extern task mon_{{ segment.tx }}_{{ segment.name }}_pre_reset();

{% endfor %}{% for segment in segments %}   /**
    * Waits out each {{ segment.rx.upper() }} {{ segment.name.upper() }} clock cycle before reset.
    */
   extern task mon_{{ segment.rx }}_{{ segment.name }}_pre_reset();

{% endfor %}{% for segment in segments %}   /**
    * Waits out each {{ segment.tx.upper() }} {{ segment.name.upper() }} clock cycle before reset.
    */
   extern task mon_{{ segment.tx }}_{{ segment.name }}_in_reset();

{% endfor %}{% for segment in segments %}   /**
    * Waits out each {{ segment.rx.upper() }} {{ segment.name.upper() }} clock cycle before reset.
    */
   extern task mon_{{ segment.rx }}_{{ segment.name }}_in_reset();

{% endfor %}{% for segment in segments %}   /**
    * Samples #mp_{{ segment.name }} {{ segment.name.upper() }} after reset.
    */
   extern task mon_{{ segment.tx }}_{{ segment.name }}_post_reset();

{% endfor %}{% for segment in segments %}   /**
    * Samples #mp_{{ segment.name }} {{ segment.name.upper() }} after reset.
    */
   extern task mon_{{ segment.rx }}_{{ segment.name }}_post_reset();

{% endfor %}{% for segment in segments %}   /**
    * Creates {{ segment.tx.upper() }} {{ segment.name.upper() }} Monitor Transaction by sampling #{{ segment.name }}_mp signals.
    */
   extern task sample_{{ segment.tx }}_{{ segment.name }}_trn(output uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}{% for segment in segments %}   /**
    * Creates {{ segment.rx.upper() }} {{ segment.name.upper() }} Monitor Transaction by sampling #{{ segment.name }}_mp signals.
    */
   extern task sample_{{ segment.rx }}_{{ segment.name }}_trn(output uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}{% for segment in segments %}   /**
    * Appends #cfg to #trn and prints it out.
    */
   extern function void process_{{ segment.tx }}_{{ segment.name }}_trn(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}{% for segment in segments %}   /**
    * Appends #cfg to #trn and prints it out.
    */
   extern function void process_{{ segment.rx }}_{{ segment.name }}_trn(ref uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}endclass : uvma_{{ name }}_mon_c


function uvma_{{ name }}_mon_c::new(string name="uvma_{{ name }}_mon", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_mon_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg          ();
   get_cntxt        ();
   create_tlm_ports ();
   retrieve_modports();

endfunction : build_phase


task uvma_{{ name }}_mon_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   if (cfg.enabled) begin
      fork
         observe_reset();
{% for segment in segments %}         mon_{{ segment.tx }}_{{ segment.name }}();
{% endfor %}{% for segment in segments %}         mon_{{ segment.rx }}_{{ segment.name }}();
{% endfor %}      join_none
   end

endtask : run_phase


function void uvma_{{ name }}_mon_c::get_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_MON", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvma_{{ name }}_mon_c::get_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_MON", "Context handle is null")
   end

endfunction : get_cntxt


function void uvma_{{ name }}_mon_c::create_tlm_ports();

{% for segment in segments %}   {{ mode_1 }}_{{ segment.name }}_ap = new("{{ mode_1 }}_{{ segment.name }}_ap", this);
{% endfor %}   {% for segment in segments %}{{ mode_2 }}_{{ segment.name }}_ap = new("{{ mode_2 }}_{{ segment.name }}_ap", this);
   {% endfor %}
endfunction : create_tlm_ports


function void uvma_{{ name }}_mon_c::retrieve_modports();

{% for segment in segments %}   mp_{{ segment.name }} = cntxt.vif.mon_{{ segment.name }}_mp;
{% endfor %}
endfunction : retrieve_modports


task uvma_{{ name }}_mon_c::observe_reset();

   case (cfg.reset_type)
      UVML_RESET_TYPE_SYNCHRONOUS : observe_reset_sync ();
      UVML_RESET_TYPE_ASYNCHRONOUS: observe_reset_async();
      default: begin
         `uvm_fatal("{{ name.upper() }}_MON", $sformatf("Illegal cfg.reset_type: %s", cfg.reset_type.name()))
      end
   endcase

endtask : observe_reset


task uvma_{{ name }}_mon_c::observe_reset_sync();

   forever begin
      while (cntxt.vif.reset_n !== 1'b0) begin
         wait (cntxt.vif.clk === 0);
         wait (cntxt.vif.clk === 1);
      end
      cntxt.reset_state = UVML_RESET_STATE_IN_RESET;
      `uvm_info("{{ name.upper() }}_MON", "Entered IN_RESET state", UVM_MEDIUM)
      while (cntxt.vif.reset_n !== 1'b1) begin
         wait (cntxt.vif.clk === 0);
         wait (cntxt.vif.clk === 1);
      end
      cntxt.reset_state = UVML_RESET_STATE_POST_RESET;
      `uvm_info("{{ name.upper() }}_MON", "Entered POST_RESET state", UVM_MEDIUM)
   end

endtask : observe_reset_sync


task uvma_{{ name }}_mon_c::observe_reset_async();

   forever begin
      wait (cntxt.vif.reset_n === 0);
      cntxt.reset_state = UVML_RESET_STATE_IN_RESET;
      `uvm_info("{{ name.upper() }}_MON", "Entered IN_RESET state", UVM_MEDIUM)
      wait (cntxt.vif.reset_n === 1);
      cntxt.reset_state = UVML_RESET_STATE_POST_RESET;
      `uvm_info("{{ name.upper() }}_MON", "Entered POST_RESET state", UVM_MEDIUM)
   end

endtask : observe_reset_async

{% for segment in segments %}
task uvma_{{ name }}_mon_c::mon_{{ segment.tx }}_{{ segment.name }}();

   forever begin
      case (cntxt.reset_state)
         UVML_RESET_STATE_PRE_RESET : mon_{{ segment.tx }}_{{ segment.name }}_pre_reset ();
         UVML_RESET_STATE_IN_RESET  : mon_{{ segment.tx }}_{{ segment.name }}_in_reset  ();
         UVML_RESET_STATE_POST_RESET: mon_{{ segment.tx }}_{{ segment.name }}_post_reset();
      endcase
   end

endtask : mon_{{ segment.tx }}_{{ segment.name }}

{% endfor %}{% for segment in segments %}
task uvma_{{ name }}_mon_c::mon_{{ segment.rx }}_{{ segment.name }}();

   forever begin
      case (cntxt.reset_state)
         UVML_RESET_STATE_PRE_RESET : mon_{{ segment.rx }}_{{ segment.name }}_pre_reset ();
         UVML_RESET_STATE_IN_RESET  : mon_{{ segment.rx }}_{{ segment.name }}_in_reset  ();
         UVML_RESET_STATE_POST_RESET: mon_{{ segment.rx }}_{{ segment.name }}_post_reset();
      endcase
   end

endtask : mon_{{ segment.rx }}_{{ segment.name }}


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.tx }}_{{ segment.name }}_pre_reset();

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);

endtask : mon_{{ segment.tx }}_{{ segment.name }}_pre_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.rx }}_{{ segment.name }}_pre_reset();

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);

endtask : mon_{{ segment.rx }}_{{ segment.name }}_pre_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.tx }}_{{ segment.name }}_in_reset();

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);

endtask : mon_{{ segment.tx }}_{{ segment.name }}_in_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.rx }}_{{ segment.name }}_in_reset();

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);

endtask : mon_{{ segment.rx }}_{{ segment.name }}_in_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.tx }}_{{ segment.name }}_post_reset();

   uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c  trn;
   sample_{{ segment.tx }}_{{ segment.name }}_trn (trn);
   process_{{ segment.tx }}_{{ segment.name }}_trn(trn);
   {{ segment.tx }}_{{ segment.name }}_ap.write   (trn);

endtask : mon_{{ segment.tx }}_{{ segment.name }}_post_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::mon_{{ segment.rx }}_{{ segment.name }}_post_reset();

   uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c  trn;
   sample_{{ segment.rx }}_{{ segment.name }}_trn (trn);
   process_{{ segment.rx }}_{{ segment.name }}_trn(trn);
   {{ segment.rx }}_{{ segment.name }}_ap.write   (trn);

endtask : mon_{{ segment.rx }}_{{ segment.name }}_post_reset


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::sample_{{ segment.tx }}_{{ segment.name }}_trn(output uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c trn);

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);
   `uvm_info("{{ name.upper() }}_MON", "Sampling {{ segment.tx }} {{ segment.name }} transaction", UVM_DEBUG)
   trn = uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c::type_id::create("trn");
   // TODO Implement sampling {{ segment.tx }} {{ segment.name }} monitor transaction from the interface
   {% if mm %}//      Ex: trn.{{ segment.name }}req = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}req;
   //          trn.{{ segment.name }}we = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}we;
   //          for (int unsigned ii=0; ii<cfg.addr_width; ii++) begin
   //             trn.{{ segment.name }}addr[ii] = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}addr[ii];
   //          end
   {% if sbe %}//          for (int unsigned ii=0; ii<(cfg.data_width/8); ii++) begin
   //             req.{{ segment.name }}be[ii] = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}be[ii];
   //          end
   {% endif %}//          for (int unsigned ii=0; ii<cfg.data_width; ii++) begin
   //             trn.{{ segment.name }}data[ii] = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}data[ii];
   //          end
   {% else %}//      Ex: trn.{{ segment.name }}_req = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}_req;
   //          for (int unsigned ii=0; ii<(cfg.data_width/8); ii++) begin
   //             req.{{ segment.name }}_be[ii] = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}_be[ii];
   //          end
   //          for (int unsigned ii=0; ii<cfg.data_width; ii++) begin
   //             trn.{{ segment.name }}_data[ii] = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}_data[ii];
   //          end
{% endif %}
endtask : sample_{{ segment.tx }}_{{ segment.name }}_trn


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_mon_c::sample_{{ segment.rx }}_{{ segment.name }}_trn(output uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c trn);

   @(mp_{{ segment.name }}.mon_{{ segment.name }}_cb);
   `uvm_info("{{ name.upper() }}_MON", "Sampling {{ segment.rx }} {{ segment.name }} transaction", UVM_DEBUG)
   trn = uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c::type_id::create("trn");
   // TODO Implement sampling {{ segment.rx }} {{ segment.name }} monitor transaction from the interface
   {% if mm %}//      Ex: trn.{{ segment.name }}gnt = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}gnt;
   {% else %}//      Ex: trn.{{ segment.name }}_gnt = mp_{{ segment.name }}.mon_{{ segment.name }}_cb.{{ segment.name }}_gnt;
{% endif %}
endtask : sample_{{ segment.rx }}_{{ segment.name }}_trn


{% endfor %}{% for segment in segments %}function void uvma_{{ name }}_mon_c::process_{{ segment.tx }}_{{ segment.name }}_trn(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_mon_trn_c trn);

   trn.cfg = cfg;
   trn.set_initiator(this);
   trn.set_timestamp_end($realtime());
   `uvm_info("{{ name.upper() }}_MON", $sformatf("Sampled {{ segment.tx.upper() }} {{ segment.name.upper() }} transaction from the virtual interface:\n%s", trn.sprint()), UVM_HIGH)

endfunction : process_{{ segment.tx }}_{{ segment.name }}_trn


{% endfor %}{% for segment in segments %}function void uvma_{{ name }}_mon_c::process_{{ segment.rx }}_{{ segment.name }}_trn(ref uvma_{{ name }}_{{ segment.rx }}_{{ segment.name }}_mon_trn_c trn);

   trn.cfg = cfg;
   trn.set_initiator(this);
   trn.set_timestamp_end($realtime());
   `uvm_info("{{ name.upper() }}_MON", $sformatf("Sampled {{ segment.rx.upper() }} {{ segment.name.upper() }} transaction from the virtual interface:\n%s", trn.sprint()), UVM_HIGH)

endfunction : process_{{ segment.rx }}_{{ segment.name }}_trn


{% endfor %}`endif // __UVMA_{{ name.upper() }}_MON_SV__
