// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV_SV__
`define __UVMA_{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV_SV__


/**
 * Driver driving {{ full_name }} Virtual Interface (uvma_{{ name }}_if) {{ segment.tx.upper() }} {{ segment.name.upper() }}.
 * @ingroup uvma_{{ name }}_comps
 */
class uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c extends uvml_drv_c #(
   .REQ(uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c),
   .RSP(uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c)
);

   virtual uvma_{{ name }}_if.drv_{{ segment.tx }}_{{ segment.name }}_mp  mp; ///< Handle to Virtual Interface {{ segment.tx.upper() }} {{ segment.name.upper() }} modport

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}

   /// @name TLM
   /// @{
   uvm_analysis_port#(uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c)  ap; ///< Output Port for {{ segment.tx.upper() }} {{ segment.name.upper() }} Sequence Items after they've been used to drive #mp.
   /// @}


   `uvm_component_utils_begin(uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Builds TLM Ports.
    * 3. Retrieves #mp from #cntxt.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Infinite loop taking in #req and driving #mp on each clock cycle.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates #ap.
    */
   extern function void create_tlm_ports();

   /**
    * Retrieves #mp from #cntxt.
    */
   extern function void retrieve_modports();

   /**
    * Appends #cfg to #req and prints it out.
    */
   extern virtual function void process_req(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

   /**
    * Drives #mp signals using #req's contents on the next clock cycle.
    */
   extern virtual task drv_req(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

   /**
    * Samples #mp signals on the next clock edge for the last transaction.
    */
   extern virtual task sample_post_clk(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

endclass : uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c


function uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::new(string name="uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg          ();
   get_cntxt        ();
   create_tlm_ports ();
   retrieve_modports();

endfunction : build_phase


task uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   if (cfg.enabled && cfg.is_active && (cfg.drv_mode == UVMA_{{ name.upper() }}_DRV_MODE_{{ segment.tx.upper() }})) begin
      forever begin
         seq_item_port.get_next_item(req);
         process_req                (req);
         drv_req                    (req);
         ap.write                   (req);
         @(mp.drv_{{ segment.tx }}_{{ segment.name }}_cb);
         sample_post_clk(req);
         seq_item_port.item_done();
      end
   end

endtask : run_phase


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::get_cfg();

   void'(uvm_config_db#(uvma_{{ name }}_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV", "Configuration handle is null")
   end
   uvm_config_db#(uvma_{{ name }}_cfg_c)::set(this, "*", "cfg", cfg);

endfunction : get_cfg


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::get_cntxt();

   void'(uvm_config_db#(uvma_{{ name }}_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV", "Context handle is null")
   end
   uvm_config_db#(uvma_{{ name }}_cntxt_c)::set(this, "*", "cntxt", cntxt);

endfunction : get_cntxt


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::create_tlm_ports();

   ap = new("ap", this);

endfunction : create_tlm_ports


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::retrieve_modports();

   mp = cntxt.vif.drv_{{ segment.tx }}_{{ segment.name }}_mp;

endfunction : retrieve_modports


function void uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::process_req(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

   req.cfg = cfg;
   `uvm_info("{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV", $sformatf("Got new req from the sequencer:\n%s", req.sprint()), UVM_DEBUG)

endfunction : process_req


task uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::drv_req(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

   // TODO uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::drv_req()
   {% if mm %}//      Ex: mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}req <= req.{{ segment.name }}req;
   //          mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}we <= req.{{ segment.name }}we;
   //          for (int unsigned ii=0; ii<cfg.addr_width; ii++) begin
   //             mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}addr[ii] <= req.{{ segment.name }}addr[ii];
   //          end
   {% if sbe %}//          for (int unsigned ii=0; ii<(cfg.data_width/8); ii++) begin
   //             mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}be[ii] <= req.{{ segment.name }}be[ii];
   //          end
   {% endif %}//          for (int unsigned ii=0; ii<cfg.data_width; ii++) begin
   //             mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}data[ii] <= req.{{ segment.name }}data[ii];
   //          end
   {% else %}//      Ex: for (int unsigned ii=0; ii<(cfg.data_width); ii++) begin
   //             mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}_be[ii] <= req.{{ segment.name }}_be[ii];
   //          end
   //          for (int unsigned ii=0; ii<cfg.data_width; ii++) begin
   //             mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}_data[ii] <= req.{{ segment.name }}_data[ii];
   //          end
{% endif %}
endtask : drv_req


task uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::sample_post_clk(ref uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_seq_item_c req);

   // TODO Implement uvma_{{ name }}_{{ segment.tx }}_{{ segment.name }}_drv_c::sample_post_clk()
   {% if mm %}//      Ex: req.gnt = mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}gnt;
   {% else %}//      Ex: req.gnt = mp.drv_{{ segment.tx }}_{{ segment.name }}_cb.{{ segment.name }}_gnt;
{% endif %}
endtask : sample_post_clk


`endif // __UVMA_{{ name.upper() }}_{{ segment.tx.upper() }}_{{ segment.name.upper() }}_DRV_SV__
