// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_CFG_SV__
`define __UVMA_{{ name.upper() }}_CFG_SV__


// Default sequences
typedef class uvma_{{ name }}_mon_vseq_c ;
typedef class uvma_{{ name }}_idle_vseq_c;
typedef class uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c;
typedef class uvma_{{ name }}_{{ mode_2 }}_drv_vseq_c ;


/**
 * Object encapsulating all parameters for creating, connecting and running all {{ full_name }} agent (uvma_{{ name }}_agent_c) components.
 * @ingroup uvma_{{ name }}_obj
 */
class uvma_{{ name }}_cfg_c extends uvml_cfg_c;

   /// @name Generic options
   /// @{
   rand bit                      enabled          ; ///< Sub-components are not created if '0'.
   rand bit                      bypass_mode      ; ///< Does not drive interface when '1' (only TLM).
   rand uvm_active_passive_enum  is_active        ; ///< Driver is not created if 'UVM_PASSIVE'.
   rand uvml_reset_type_enum     reset_type       ; ///< Sets whether the monitor is sensitive to a/synchronous reset.
   rand uvm_sequencer_arb_mode   sqr_arb_mode     ; ///< TODO Describe uvma_{{ name }}_cfg_c::sqr_arb_mode
   rand bit                      cov_model_enabled; ///< Gates the creation of the functional coverage model.
   rand bit                      trn_log_enabled  ; ///< Gates the creation of the transaction logger.
   /// @}

   /// @name Protocol Options
   /// @{
   rand uvma_{{ name }}_mode_enum      drv_mode; ///< Specifies which data stream to drive when in active mode.
   rand uvma_{{ name }}_drv_idle_enum  drv_idle; ///< Specifies the type of data to send during idle cycles
   {% if mm %}rand int unsigned  addr_width; ///< Specifies the width for the addr signals.
   {% endif %}rand int unsigned  data_width; ///< Specifies the width for the data signals.
   /// @}

   /// @name Sequence Types
   /// @{
   uvm_object_wrapper  mon_vseq_type ; ///< Type for Monitor Virtual Sequence to be started by agent at start of run_phase().
   uvm_object_wrapper  idle_vseq_type; ///< Type for Idle Virtual Sequence to be started by agent at start of run_phase().
   uvm_object_wrapper  {{ mode_1 }}_drv_vseq_type; ///< Type for {{ mode_1.upper() }} Driver Virtual Sequence to be started by agent at start of run_phase().
   uvm_object_wrapper  {{ mode_2 }}_drv_vseq_type ; ///< Type for {{ mode_2.upper() }} Driver Virtual Sequence to be started by agent at start of run_phase().
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_cfg_c)
      `uvm_field_int (                         enabled          , UVM_DEFAULT)
      `uvm_field_enum(uvm_active_passive_enum, is_active        , UVM_DEFAULT)
      `uvm_field_enum(uvml_reset_type_enum   , reset_type       , UVM_DEFAULT)
      `uvm_field_enum(uvm_sequencer_arb_mode , sqr_arb_mode     , UVM_DEFAULT)
      `uvm_field_int (                         cov_model_enabled, UVM_DEFAULT)
      `uvm_field_int (                         trn_log_enabled  , UVM_DEFAULT)

      `uvm_field_enum(uvma_{{ name }}_mode_enum    , drv_mode, UVM_DEFAULT)
      `uvm_field_enum(uvma_{{ name }}_drv_idle_enum, drv_idle, UVM_DEFAULT)
       {% if mm %}`uvm_field_int(addr_width, UVM_DEFAULT + UVM_DEC)
      {% endif %}`uvm_field_int(data_width, UVM_DEFAULT + UVM_DEC)
   `uvm_object_utils_end


   /**
    * Sets usual values for generic options.
    */
   constraint defaults_cons {
      soft enabled           == 1;
      soft is_active         == UVM_PASSIVE;
      soft reset_type        == UVML_RESET_TYPE_SYNCHRONOUS;
      soft sqr_arb_mode      == UVM_SEQ_ARB_FIFO;
      soft cov_model_enabled == 0;
      soft trn_log_enabled   == 1;
      soft drv_idle          == UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS;
      {% if mm %}soft addr_width  == uvma_{{ name }}_default_addr_width;
      {% endif %}soft data_width  == uvma_{{ name }}_default_data_width;
   }

   /**
    * Rules for signal widths.
    */
   constraint limits_cons {
      {% if mm %}addr_width >= 4;
      addr_width <= `UVMA_{{ name.upper() }}_ADDR_MAX_WIDTH;
      data_width % 8 == 0;
      data_width >= 8;
      data_width <= `UVMA_{{ name.upper() }}_DATA_MAX_WIDTH;
      {% else %}data_width > 0;
      data_width <= `UVMA_{{ name.upper() }}_DATA_MAX_WIDTH;
   {% endif %}}


   /**
    * Creates objects.
    */
   extern function new(string name="uvma_{{ name }}_cfg");

endclass : uvma_{{ name }}_cfg_c


function uvma_{{ name }}_cfg_c::new(string name="uvma_{{ name }}_cfg");

   super.new(name);
   mon_vseq_type  = uvma_{{ name }}_mon_vseq_c ::get_type();
   idle_vseq_type = uvma_{{ name }}_idle_vseq_c::get_type();
   {{ mode_1 }}_drv_vseq_type = uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::get_type();
   {{ mode_2 }}_drv_vseq_type = uvma_{{ name }}_{{ mode_2 }}_drv_vseq_c ::get_type();

endfunction : new


`endif // __UVMA_{{ name.upper() }}_CFG_SV__
