// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_MON_TRN_SV__
`define __UVMA_{{ name.upper() }}_MON_TRN_SV__


/**
 * Monitor Transaction rebuilt by the Monitor Virtual Sequence (uvma_{{ name }}_mon_vseq_c).
 * Analog of uvma_{{ name }}_seq_item_c.
 * @ingroup uvma_{{ name }}_obj
 */
class uvma_{{ name }}_mon_trn_c extends uvml_mon_trn_c;

   uvma_{{ name }}_cfg_c  cfg; ///< Agent configuration handle

   /// @name Data
   /// @{
   uvma_{{ name }}_access_enum  access_type; ///< Read or write
   uvma_{{ name }}_addr_l_t     address    ; ///< Read/Write Address
   {% if sbe %}uvma_{{ name }}_be_l_t       be         ; ///< Byte Enabled
   {% endif %}uvma_{{ name }}_data_l_t      data       ; ///< Read/Write Data
   // TODO Add data fields
   //      Ex: uvma_{{ name }}_user_l_t  user; ///< User Data
   /// @}

   /// @name Metadata
   /// @{
   int unsigned  gnt_latency; ///< Number of cycles before gnt is asserted
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_mon_trn_c)
      `uvm_field_enum(uvma_{{ name }}_access_enum, access_type, UVM_DEFAULT)
      `uvm_field_int(address, UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
      {% if sbe %}`uvm_field_int(be  , UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
      {% endif %}`uvm_field_int(data, UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
   `uvm_object_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_mon_trn");

   /**
    * Compares variable width fields.
    */
   extern virtual function bit do_compare(uvm_object rhs, uvm_comparer comparer);

   /**
    * Prints variable width fields.
    */
   extern virtual function void do_print(uvm_printer printer);

   /**
    * Describes transaction as metadata for uvml_logs_metadata_logger_c.
    */
   extern virtual function uvml_metadata_t get_metadata();

endclass : uvma_{{ name }}_mon_trn_c


function uvma_{{ name }}_mon_trn_c::new(string name="uvma_{{ name }}_mon_trn");

   super.new(name);

endfunction : new


function bit uvma_{{ name }}_mon_trn_c::do_compare(uvm_object rhs, uvm_comparer comparer);

   uvma_{{ name }}_mon_trn_c  rhs_;
   if (!$cast(rhs_, rhs)) begin
      `uvm_fatal("{{ name.upper() }}_MON_TRN", $sformatf("Could not cast 'rhs' (%s) to 'rhs_' (%s)", $typename(rhs), $typename(rhs_)))
   end
   if (cfg == null) begin
      `uvm_warning("{{ name.upper() }}_MON_TRN", "cfg handle is null, skipping do_compare()")
   end
   else begin
      do_compare = 1;
      // TODO Implement uvma_{{ name }}_mon_trn_c::do_compare()
      //      Ex: do_compare &= comparer.compare_field_int("address", address, rhs_.address, cfg.addr_width);
      //          {% if sbe %}do_compare &= comparer.compare_field_int("be", be, rhs_.be, cfg.data_width/8);
      //          {% endif %}do_compare &= comparer.compare_field_int("data", data, rhs_.data, cfg.data_width);
   end

endfunction : do_compare


function void uvma_{{ name }}_mon_trn_c::do_print(uvm_printer printer);

   super.do_print(printer);
   if (cfg == null) begin
      `uvm_warning("{{ name.upper() }}_MON_TRN", "cfg handle is null, skipping do_print()")
   end
   else begin
      printer.print_field("address", address, cfg.addr_width);
      {% if sbe %}printer.print_field("be", be, cfg.data_width/8);
      {% endif %}printer.print_field("data", data, cfg.data_width);
   end

endfunction : do_print


function uvml_metadata_t uvma_{{ name }}_mon_trn_c::get_metadata();

   string  access_str, address_str, {% if sbe %}be_str, {% endif %}data_str;
   int unsigned  field_count = 0;
   if (cfg == null) begin
      `uvm_warning("{{ name.upper() }}_MON_TRN", "cfg handle is null, skipping get_metadata()")
   end
   else begin
      access_str = (access_type == UVMA_{{ name.upper() }}_ACCESS_READ) ? "READ" : "WRITE";
      get_metadata.push_back('{
         index     : field_count,
         value     : access_str,
         col_name  : "access",
         col_width : access_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_ENUM
      });
      field_count++;
      address_str = $sformatf("%h", address);
      address_str = address_str.substr(address_str.len() - (cfg.addr_width/4), address_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : address_str,
         col_name  : "address",
         col_width : address_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
      {% if sbe %}be_str = $sformatf("%b", be);
      be_str = be_str.substr(be_str.len() - cfg.data_width/8, be_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : be_str,
         col_name  : "be",
         col_width : be_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
      {% endif %}data_str = $sformatf("%h", data);
      data_str = data_str.substr(data_str.len() - (cfg.data_width/4), data_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : data_str,
         col_name  : "data",
         col_width : data_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
   end

endfunction : get_metadata


`endif // __UVMA_{{ name.upper() }}_MON_TRN_SV__
