// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_PKG_SV__
`define __UVMA_{{ name.upper() }}_PKG_SV__


// Doxygen Groups
/**
 * @defgroup uvma_{{ name }}_pkg {{ full_name }} UVM Agent
 * @{
 * @defgroup uvma_{{ name }}_comps Components
 * @defgroup uvma_{{ name }}_misc  Miscellaneous
 * @defgroup uvma_{{ name }}_obj   Objects
{% if mm %} * @defgroup uvma_{{ name }}_reg   Register Modeling
{% endif %} * @defgroup uvma_{{ name }}_seq   Sequences & Sequence Items
 * @}
 */


// Pre-processor macros
`include "uvm_macros.svh"
`include "uvml_macros.svh"
`include "uvml_logs_macros.svh"
{% if mm %}`include "uvml_ral_macros.svh"
{% endif %}`include "uvma_{{ name }}_macros.svh"

// Interface(s)
`include "uvma_{{ name }}_if.sv"


/**
 * Encapsulates all the types needed for an UVM agent capable of driving and/or monitoring {{ full_name }} Interface.
 * @ingroup uvma_{{ name }}_pkg
 */
package uvma_{{ name }}_pkg;

   import uvm_pkg      ::*;
   import uvml_pkg     ::*;
   {% if mm %}import uvml_ral_pkg::*;
   {% endif %}import uvml_logs_pkg ::*;

   // Constants / Structs / Enums
   `include "uvma_{{ name }}_tdefs.sv"
   `include "uvma_{{ name }}_constants.sv"

   // Objects
   `include "uvma_{{ name }}_cfg.sv"
   `include "uvma_{{ name }}_cntxt.sv"

   // Transactions
   `include "uvma_{{ name }}_mon_trn.sv"
   {% for segment in segments %}`include "uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn.sv"
   {% endfor %}{% for segment in segments %}`include "uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn.sv"
   {% endfor %}`include "uvma_{{ name }}_seq_item.sv"
   {% for segment in segments %}`include "uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item.sv"
   {% endfor %}{% for segment in segments %}   `include "uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item.sv"
{% endfor %}
   // Driver
   {% for segment in segments %}`include "uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_drv.sv"
   {% endfor %}{% for segment in segments %}   `include "uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_drv.sv"
{% endfor %}
   // Virtual Sequencer
   {% for segment in segments %}`include "uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_sqr.sv"
   {% endfor %}{% for segment in segments %}   `include "uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_sqr.sv"
{% endfor %}
   // Agent-Level Components
   `include "uvma_{{ name }}_mon.sv"
   `include "uvma_{{ name }}_drv.sv"
   `include "uvma_{{ name }}_vsqr.sv"
   `include "uvma_{{ name }}_logger.sv"
   `include "uvma_{{ name }}_cov_model.sv"
   `include "uvma_{{ name }}_agent.sv"

   // Sequences
   `include "uvma_{{ name }}_base_vseq.sv"
   `include "uvma_{{ name }}_mon_vseq.sv"
   `include "uvma_{{ name }}_{{ mode_1 }}_drv_vseq.sv"
   `include "uvma_{{ name }}_{{ mode_2 }}_drv_vseq.sv"
   `include "uvma_{{ name }}_idle_vseq.sv"
   `include "uvma_{{ name }}_vseq_lib.sv"
   {% if mm %}`include "uvma_{{ name }}_{{ mode_2 }}_handler_vseq_lib.sv"

   // Register-related
   `include "uvma_{{ name }}_reg_adapter.sv"
{% endif %}
endpackage : uvma_{{ name }}_pkg


// Module(s) / Checker(s)
`ifdef UVMA_{{ name.upper() }}_INC_CHKR
`include "uvma_{{ name }}_chkr.sv"
`endif


`endif // __UVMA_{{ name.upper() }}_PKG_SV__
