// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_REG_ADAPTER_SV__
`define __UVMA_{{ name.upper() }}_REG_ADAPTER_SV__


/**
 * Object that converts between UVM's abstract register operations (uvm_reg_bus_op) and uvma_{{ name }}_seq_item_c.
 * @ingroup uvma_{{ name }}_reg
 */
class uvma_{{ name }}_reg_adapter_c extends uvml_ral_reg_adapter_c;

   `uvm_object_utils(uvma_{{ name }}_reg_adapter_c)

   /**
    * Default constructor
    */
   extern function new(string name="uvma_{{ name }}_reg_adapter");

   /**
    * Converts from UVM register operation to {{ full_name }} ({{ name.upper() }}) Agent.
    */
   extern virtual function uvm_sequence_item reg2bus(const ref uvm_reg_bus_op rw);

   /**
    * Converts from {{ full_name }} ({{ name.upper() }}) Agent to UVM register operation.
    */
   extern virtual function void bus2reg(uvm_sequence_item bus_item, ref uvm_reg_bus_op rw);

endclass : uvma_{{ name }}_reg_adapter_c


function uvma_{{ name }}_reg_adapter_c::new(string name="uvma_{{ name }}_reg_adapter");

   super.new(name);
   {% if sbe %}supports_byte_enable = 1;
   {% else %}supports_byte_enable = 0;
{% endif %}
endfunction : new


function uvm_sequence_item uvma_{{ name }}_reg_adapter_c::reg2bus(const ref uvm_reg_bus_op rw);

   uvma_{{ name }}_seq_item_c  {{ name }}_seq_item;
   super.reg2bus(rw);
   {{ name }}_seq_item = uvma_{{ name }}_seq_item_c::type_id::create("{{ name }}_seq_item");
   {{ name }}_seq_item.access_type = (rw.kind == UVM_READ) ? UVMA_{{ name.upper() }}_ACCESS_READ : UVMA_{{ name.upper() }}_ACCESS_WRITE;
   {{ name }}_seq_item.address     = rw.addr;
   {{ name }}_seq_item.data        = rw.data;
   {% if sbe %}{{ name }}_seq_item.be          = rw.byte_en;
   {% endif %}if (rw.status == UVM_NOT_OK) begin
      {{ name }}_seq_item.set_error(1);
   end
   return {{ name }}_seq_item;

endfunction : reg2bus


function void uvma_{{ name }}_reg_adapter_c::bus2reg(uvm_sequence_item bus_item, ref uvm_reg_bus_op rw);

   uvma_{{ name }}_seq_item_c  {{ name }}_seq_item;
   super.bus2reg(bus_item, rw);
   if (!$cast({{ name }}_seq_item, bus_item)) begin
      `uvm_fatal("{{ name.upper() }}_REG_ADAPTER", $sformatf("Could not cast bus_item (%s) into {{ name }}_seq_item (%s)", $typename(bus_item), $typename({{ name }}_seq_item)))
   end
   rw.kind = ({{ name }}_seq_item.access_type == UVMA_{{ name.upper() }}_ACCESS_READ) ? UVM_READ : UVM_WRITE;
   rw.addr = {{ name }}_seq_item.address;
   rw.data = {{ name }}_seq_item.data;
   {% if sbe %}rw.byte_en = {{ name }}_seq_item.be;
   {% endif %}if ({{ name }}_seq_item.has_error()) begin
      rw.status = UVM_NOT_OK;
   end
   else begin
      rw.status = UVM_IS_OK;
   end

endfunction : bus2reg


`endif // __UVMA_{{ name.upper() }}_REG_ADAPTER_SV__
