// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_BASE_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_BASE_VSEQ_SV__


/**
 * Abstract Virtual Sequence from which all other {{ full_name }} Virtual Sequences must extend.
 * Classes extending this one must be run on {{ full_name }} Virtual Sequencer (uvma_{{ name }}_vsqr_c) instance.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_base_vseq_c extends uvml_vseq_c #(
   .REQ(uvma_{{ name }}_seq_item_c),
   .RSP(uvma_{{ name }}_seq_item_c)
);

   /// @name Objects
   /// @{
   uvma_{{ name }}_cfg_c    cfg  ; ///< Agent configuration handle
   uvma_{{ name }}_cntxt_c  cntxt; ///< Agent context handle
   /// @}


   `uvm_object_utils(uvma_{{ name }}_base_vseq_c)
   `uvm_declare_p_sequencer(uvma_{{ name }}_vsqr_c)


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_base_vseq");

   /**
    * Assigns #cfg and #cntxt handles from #p_sequencer.
    */
   extern virtual task pre_start();

   /**
    * Gets the next #req for transport sequences.
    */
   extern task upstream_get_next_item(ref uvm_sequence_item req);

   /**
    * Signals to the upstream sequencer that we're done driving #req.
    */
   extern task upstream_item_done(ref uvm_sequence_item req);

{% if duplex %}   /**
    * Writes #trn to #p_sequencer's {{ dir_1.upper() }} Monitor Transaction analysis port.
    */
   extern task write_{{ dir_1 }}_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

   /**
    * Writes #trn to #p_sequencer's {{ dir_2.upper() }} Monitor Transaction analysis port.
    */
   extern task write_{{ dir_2 }}_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

{% else %}   /**
    * Writes #trn to #p_sequencer's Monitor Transaction analysis port.
    */
   extern task write_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

{% endif %}{% for segment in segments %}   /**
    * Gets the next {{ mode_1.upper() }} {{ segment.name.upper() }} transaction from the Monitor.
    */
   extern task get_{{ mode_1 }}_{{ segment.name }}_mon_trn(output uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}{% for segment in segments %}   /**
    * Gets the next {{ mode_2.upper() }} {{ segment.name.upper() }} transaction from the Monitor.
    */
   extern task get_{{ mode_2 }}_{{ segment.name }}_mon_trn(output uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c trn);

{% endfor %}endclass : uvma_{{ name }}_base_vseq_c


function uvma_{{ name }}_base_vseq_c::new(string name="uvma_{{ name }}_base_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_base_vseq_c::pre_start();

   cfg   = p_sequencer.cfg  ;
   cntxt = p_sequencer.cntxt;

endtask : pre_start


task uvma_{{ name }}_base_vseq_c::upstream_get_next_item(ref uvm_sequence_item req);

   p_sequencer.upstream_sqr_port.get_next_item(req);

endtask : upstream_get_next_item


task uvma_{{ name }}_base_vseq_c::upstream_item_done(ref uvm_sequence_item req);

   p_sequencer.upstream_sqr_port.item_done(req);

endtask : upstream_item_done


{% if duplex %}task uvma_{{ name }}_base_vseq_c::write_{{ dir_1 }}_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

   p_sequencer.{{ dir_1 }}_mon_trn_ap.write(trn);

endtask : write_{{ dir_1 }}_mon_trn


task uvma_{{ name }}_base_vseq_c::write_{{ dir_2 }}_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

   p_sequencer.{{ dir_2 }}_mon_trn_ap.write(trn);

endtask : write_{{ dir_2 }}_mon_trn

{% else %}task uvma_{{ name }}_base_vseq_c::write_mon_trn(ref uvma_{{ name }}_mon_trn_c trn);

   p_sequencer.mon_trn_ap.write(trn);

endtask : write_mon_trn

{% endif %}
{% for segment in segments %}task uvma_{{ name }}_base_vseq_c::get_{{ mode_1 }}_{{ segment.name }}_mon_trn(output uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_mon_trn_c trn);

   p_sequencer.{{ mode_1 }}_{{ segment.name }}_mon_trn_fifo.get(trn);

endtask : get_{{ mode_1 }}_{{ segment.name }}_mon_trn


{% endfor %}{% for segment in segments %}task uvma_{{ name }}_base_vseq_c::get_{{ mode_2 }}_{{ segment.name }}_mon_trn(output uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_mon_trn_c trn);

   p_sequencer.{{ mode_2 }}_{{ segment.name }}_mon_trn_fifo.get(trn);

endtask : get_{{ mode_2 }}_{{ segment.name }}_mon_trn


{% endfor %}`endif // __UVMA_{{ name.upper() }}_BASE_SEQ_SV__
