// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__


/**
 * Virtual Sequence taking in uvma_{{ name }}_seq_item_c instances and driving uvma_{{ name }}_drv_c with {{ mode_1.upper() }} Sequence Items.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c extends uvma_{{ name }}_base_vseq_c;

   `uvm_object_utils(uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c)

   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_{{ mode_1 }}_drv_vseq");

   /**
    * Infinite loop maintaining 2 threads:
    * 1. Main sequence: wait for reset, drive traffic
    * 2. Reset trigger for mid-sim reset which resets the main sequence.
    */
   extern virtual task body();

   /**
    * Appends #cfg to seq_item and trims data outside the configured widths.
    */
   extern function void process_req(ref uvma_{{ name }}_seq_item_c seq_item);

   /**
    * Converts uvma_{{ name }}_seq_item_c instances into {{ mode_1.upper() }} Sequence Items.
    */
   extern task drive(ref uvma_{{ name }}_seq_item_c seq_item);

endclass : uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c


function uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::new(string name="uvma_{{ name }}_{{ mode_1 }}_drv_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::body();

   uvma_{{ name }}_seq_item_c  seq_item;
   forever begin
      fork
         begin
            `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", "Waiting for post_reset", UVM_DEBUG)
            wait (cntxt.reset_state == UVML_RESET_STATE_POST_RESET);
            forever begin
               `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", "Waiting for next sequence item", UVM_DEBUG)
               p_sequencer.get_next_item    (seq_item);
               `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", {"Got next sequence item:\n", seq_item.sprint()}, UVM_DEBUG)
               process_req                  (seq_item);
               drive                        (seq_item);
               p_sequencer.seq_item_ap.write(seq_item);
               p_sequencer.item_done();
            end
         end

         begin
            wait (cntxt.reset_state == UVML_RESET_STATE_POST_RESET);
            wait (cntxt.reset_state != UVML_RESET_STATE_POST_RESET);
         end
      join_any
      disable fork;
   end

endtask : body


function void uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::process_req(ref uvma_{{ name }}_seq_item_c seq_item);

   seq_item.cfg = cfg;
   for (int unsigned ii=cfg.addr_width; ii<`UVMA_{{ name.upper() }}_ADDR_MAX_WIDTH; ii++) begin
      seq_item.address[ii] = 0;
   end
   {% if sbe %}for (int unsigned ii=(cfg.data_width/8); ii<(`UVMA_{{ name.upper() }}_DATA_MAX_WIDTH/8); ii++) begin
      seq_item.be[ii] = 0;
   end
   {% endif %}for (int unsigned ii=cfg.data_width; ii<`UVMA_{{ name.upper() }}_DATA_MAX_WIDTH; ii++) begin
      seq_item.data[ii] = 0;
   end
   `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", $sformatf("Processed sequence item:\n%s", seq_item.sprint()), UVM_HIGH)
   `uvml_hrtbt_owner(p_sequencer)

endfunction : process_req


task uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive(ref uvma_{{ name }}_seq_item_c seq_item);

{% for segment in segments %}   uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c  {{ mode_1 }}_{{ segment.name }}_seq_item;
{% endfor %}
   // TODO Implement uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive()
   //      Ex: do begin
   //        `uvm_create_on({{ mode_1 }}_{{ segments[0].name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segments[0].name }}_sequencer)
   //        `uvm_rand_send_pri_with({{ mode_1 }}_{{ segments[0].name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_SEQ_ITEM_PRI, {
   //           {{ segments[0].name }}req  == 1'b1;
   //           {{ segments[0].name }}addr == seq_item.address    ;
   //           {{ segments[0].name }}we   == seq_item.access_type;
   //           {% if sbe %}{{ segments[0].name }}be   == seq_item.be         ;
   //           {% endif %}{{ segments[0].name }}data == seq_item.data       ;
   //        })
   //     end while ({{ mode_1 }}_{{ segments[0].name }}_seq_item.{{ segments[0].name }}gnt !== 1'b1);
   {% if multisegment %}//     do begin
   //     `uvm_create_on({{ mode_1 }}_{{ segments[1].name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segments[1].name }}_sequencer)
   //     `uvm_rand_send_pri_with({{ mode_1 }}_{{ segments[1].name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_SEQ_ITEM_PRI, {
   //        {{ segments[1].name }}gnt == 1'b1;
   //     })
   //     end while({{ mode_1 }}_{{ segments[1].name }}_seq_item.{{ segments[1].name }}req !== 1'b1);
   //     if (seq_item.access_type == UVMA_{{ name.upper() }}_ACCESS_READ) begin
   //        seq_item.data = {{ mode_1 }}_{{ segments[1].name }}_seq_item.{{ segments[1].name }}data;
   //     end
{% endif %}
endtask : drive


`endif // __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__
