// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_SLV_HANDLER_BASE_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_SLV_HANDLER_BASE_VSEQ_SV__


/**
 * Abstract Virtual Sequence from which all other {{ mode_2.upper() }} response handler Virtual Sequences must extend.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c extends uvma_{{ name }}_base_vseq_c;

   `uvm_object_utils(uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c)

   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq");

   /**
    * 1. Adds this instance to the response handler queue.
    * 2. Infinite loop to keep the sequence running.
    */
   extern virtual task body();

   /**
    * Hook for subclasses to implement.
    */
   extern virtual task handle_{{ mode_1 }}_req(ref uvma_{{ name }}_{{ mode_1 }}_{{ segments[0].name }}_mon_trn_c trn, output bit handled);

   /**
    * Performs read from memory model.
    */
   extern function uvma_{{ name }}_data_b_t mem_read(uvma_{{ name }}_addr_b_t address{% if sbe %}, uvma_{{ name }}_be_b_t be{% endif %});

   /**
    * Performs write to memory model.
    */
   extern function void mem_write(uvma_{{ name }}_addr_b_t address, uvma_{{ name }}_data_b_t data{% if sbe %}, uvma_{{ name }}_be_b_t be{% endif %});

   /**
    * Waits for one clock cycle.
    */
   extern task wait_clk();

endclass : uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c


function uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::new(string name="uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::body();

   p_sequencer.cntxt.{{ mode_2 }}_handlers.push_front(this);
   forever begin
      wait_clk();
   end

endtask : body


task uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::handle_{{ mode_1 }}_req(ref uvma_{{ name }}_{{ mode_1 }}_{{ segments[0].name }}_mon_trn_c trn, output bit handled);
endtask : handle_{{ mode_1 }}_req


function uvma_{{ name }}_data_b_t uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::mem_read(uvma_{{ name }}_addr_b_t address{% if sbe %}, uvma_{{ name }}_be_b_t be{% endif %});

   bit [7:0]  current_byte;
   for (int unsigned ii=0; ii<(cfg.data_width/8); ii++) begin
      {% if sbe %}if (be[ii]) begin
         current_byte = cntxt.memory.read(address+ii);
         for (int unsigned jj=0; jj<8; jj++) begin
            mem_read[(ii*8)+jj] = current_byte[jj];
         end
      end{% else %}current_byte = cntxt.memory.read(address+ii);
      for (int unsigned jj=0; jj<8; jj++) begin
         mem_read[(ii*8)+jj] = current_byte[jj];
      end
{% endif %}   end

endfunction : mem_read


function void uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::mem_write(uvma_{{ name }}_addr_b_t address, uvma_{{ name }}_data_b_t data{% if sbe %}, uvma_{{ name }}_be_b_t be{% endif %});

   bit [7:0]  current_byte;
   for (int unsigned ii=0; ii<(cfg.data_width/8); ii++) begin
      current_byte = 0;
      {% if sbe %}if (be[ii]) begin
         for (int unsigned jj=0; jj<8; jj++) begin
            current_byte[jj] = data[(ii*8)+jj];
         end
         cntxt.memory.write(address+ii, current_byte);
      end{% else %}for (int unsigned jj=0; jj<8; jj++) begin
         current_byte[jj] = data[(ii*8)+jj];
      end
      cntxt.memory.write(address+ii, current_byte);
{% endif %}   end

endfunction : mem_write


task uvma_{{ name }}_{{ mode_2 }}_handler_base_vseq_c::wait_clk();

   @(cntxt.vif.drv_{{ mode_2 }}_{{ segments[0].name }}_cb);

endtask : wait_clk


`endif // __UVMA_{{ name.upper() }}_SLV_HANDLER_BASE_VSEQ_SV__
