// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__


/**
 * Virtual Sequence generating a set number of random accesses with knobs for reads/writes ratio and gap control.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_rand_stim_vseq_c extends uvma_{{ name }}_base_vseq_c;

   /// @name Knobs
   /// @{
   rand int unsigned  num_access; ///< Number of access requests to generate.
   rand int unsigned  pct_reads ; ///< Percentage of accesses which are UVMA_{{ name.upper() }}_ACCESS_READ
   rand int unsigned  pct_writes; ///< Percentage of accesses which are UVMA_{{ name.upper() }}_ACCESS_WRITE
   rand int unsigned  min_gap   ; ///< Minimum number of cycles between accesses.
   rand int unsigned  max_gap   ; ///< Maximum number of cycles between accesses.
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_rand_stim_vseq_c)
      `uvm_field_int(num_access, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(pct_reads , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(pct_writes, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_gap   , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_gap   , UVM_DEFAULT + UVM_DEC)
   `uvm_object_utils_end


   /**
    * Ensures pct, mins & maxes and bounds #num_access to an upper limit.
    */
   constraint limits_cons {
      pct_reads + pct_writes == 100;
      num_access inside {[1:uvma_{{ name }}_rand_stim_vseq_max_num_access]};
      min_gap >= 0;
      min_gap <= max_gap;
      max_gap <= uvma_{{ name }}_rand_stim_vseq_max_gap;
   }


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_rand_stim_vseq");

   /**
    * Generates #num_access Sequence Items on uvma_{{ name }}_vsqr_c according to knobs values.
    */
   extern virtual task body();

endclass : uvma_{{ name }}_rand_stim_vseq_c


function uvma_{{ name }}_rand_stim_vseq_c::new(string name="uvma_{{ name }}_rand_stim_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_rand_stim_vseq_c::body();

   uvma_{{ name }}_seq_item_c  req;
   int unsigned  gap_size;
   `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", "Starting sequence", UVM_MEDIUM)
   for (int unsigned ii=0; ii<num_access; ii++) begin
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Starting access %0d/%0d", (ii+1), num_access), UVM_HIGH)
      `uvm_create_on(req, p_sequencer)
      if ($urandom_range(0,100) > pct_reads) begin
         `uvm_rand_send_with(req, {
            access_type == UVMA_{{ name.upper() }}_ACCESS_WRITE;
         })
      end
      else begin
         `uvm_rand_send_with(req, {
            access_type == UVMA_{{ name.upper() }}_ACCESS_READ;
         })
      end
      gap_size = $urandom_range(min_gap, max_gap);
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Done driving access %0d/%0d data - Waiting %0d gap cycle(s)", (ii+1), num_access, gap_size), UVM_DEBUG)
      for (int unsigned jj=0; jj<gap_size; jj++) begin
         `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Access %0d/%0d - Waiting %0d/%0d gap cycle(s)", (ii+1), num_access, (jj+1), gap_size), UVM_DEBUG)
         @(cntxt.vif.mon_{{ segments[0].name }}_cb);
         `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Access %0d/%0d - Done waiting %0d/%0d gap cycle(s)", (ii+1), num_access, (jj+1), gap_size), UVM_DEBUG)
      end
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Finished access %0d/%0d", (ii+1), num_access), UVM_HIGH)
   end

endtask : body


`endif // __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__
