// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__
`define __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__


/**
 * Sequence Item created by {{ full_name }} Sequences. Analog of uvma_{{ name }}_mon_trn_c
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_seq_item_c extends uvml_seq_item_c;

   uvma_{{ name }}_cfg_c  cfg; ///< Agent configuration handle

   /// @name Data
   /// @{
   rand uvma_{{ name }}_access_enum  access_type; ///< Read or write
   rand uvma_{{ name }}_addr_b_t     address    ; ///< Read/Write Address
   {% if sbe %}rand uvma_{{ name }}_be_b_t       be         ; ///< Byte enabled
   {% endif %}rand uvma_{{ name }}_data_b_t     data       ; ///< Write Data
   // TODO Add data fields
   //      Ex: rand uvma_{{ name }}_user_b_t  user; ///< User Data
   /// @}

   /// @name Metadata
   /// @{
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_seq_item_c)
      `uvm_field_enum(uvma_{{ name }}_access_enum, access_type, UVM_DEFAULT)
      `uvm_field_int(address, UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
      {% if sbe %}`uvm_field_int(be  , UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
      {% endif %}`uvm_field_int(data, UVM_DEFAULT + UVM_NOCOMPARE + UVM_NOPRINT)
   `uvm_object_utils_end


   /**
    * Sets default values for data.
    */
   constraint default_cons {
   }


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_seq_item");

   /**
    * Prints variable width fields.
    */
   extern virtual function void do_print(uvm_printer printer);

   /**
    * Describes transaction as metadata for uvml_logs_metadata_logger_c.
    */
   extern function uvml_metadata_t get_metadata();

endclass : uvma_{{ name }}_seq_item_c


function uvma_{{ name }}_seq_item_c::new(string name="uvma_{{ name }}_seq_item");

   super.new(name);

endfunction : new


function void uvma_{{ name }}_seq_item_c::do_print(uvm_printer printer);

   super.do_print(printer);
   if (cfg == null) begin
      `uvm_warning("{{ name.upper() }}_SEQ_ITEM", "cfg handle is null, skipping do_print()")
   end
   else begin
      printer.print_field("address", address, cfg.addr_width);
      {% if sbe %}printer.print_field("be", be, cfg.data_width/8);
      {% endif %}printer.print_field("data", data, cfg.data_width);
   end

endfunction : do_print


function uvml_metadata_t uvma_{{ name }}_seq_item_c::get_metadata();

   string  access_str, address_str, {% if sbe %}be_str, {% endif %}data_str;
   int unsigned  field_count = 0;
   if (cfg == null) begin
      `uvm_warning("{{ name.upper() }}_SEQ_ITEM", "cfg handle is null, skipping get_metadata()")
   end
   else begin
      access_str = (access_type == UVMA_{{ name.upper() }}_ACCESS_READ) ? "READ" : "WRITE";
      get_metadata.push_back('{
         index     : field_count,
         value     : access_str,
         col_name  : "access",
         col_width : access_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_ENUM
      });
      field_count++;
      address_str = $sformatf("%h", address);
      address_str = address_str.substr(address_str.len() - (cfg.addr_width/4), address_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : address_str,
         col_name  : "address",
         col_width : address_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
      {% if sbe %}be_str = $sformatf("%b", be);
      be_str = be_str.substr(be_str.len() - cfg.data_width/8, be_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : be_str,
         col_name  : "be",
         col_width : be_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
      {% endif %}data_str = $sformatf("%h", data);
      data_str = data_str.substr(data_str.len() - (cfg.data_width/4), data_str.len()-1);
      get_metadata.push_back('{
         index     : field_count,
         value     : data_str,
         col_name  : "data",
         col_width : data_str.len(),
         col_align : UVML_TEXT_ALIGN_RIGHT,
         data_type : UVML_FIELD_INT
      });
      field_count++;
   end

endfunction : get_metadata


`endif // __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__
