// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_IDLE_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_IDLE_VSEQ_SV__


/**
 * Virtual Sequence generating idle Sequence Items at the lowest sequencer priority possible.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_idle_vseq_c extends uvma_{{ name }}_base_vseq_c;

   `uvm_object_utils(uvma_{{ name }}_idle_vseq_c)

   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_idle_vseq");

   /**
    * Launches `do_idle_x()` tasks.
    */
   extern virtual task body();

   /**
    * Infinite loop generating idle {{ mode_1.upper() }} Sequence Items.
    */
   extern task do_idle_{{ mode_1 }}();

   /**
    * Infinite loop generating idle {{ mode_2.upper() }} Sequence Items.
    */
   extern task do_idle_{{ mode_2 }}();

{% for segment in segments %}   /**
    * Infinite loop generating idle {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Items.
    */
   extern task do_idle_{{ mode_1 }}_{{ segment.name }}();

{% endfor %}{% for segment in segments %}   /**
    * Infinite loop generating idle {{ mode_2.upper() }} {{ segment.name.upper() }} Sequence Items.
    */
   extern task do_idle_{{ mode_2 }}_{{ segment.name }}();

{% endfor %}endclass : uvma_{{ name }}_idle_vseq_c


function uvma_{{ name }}_idle_vseq_c::new(string name="uvma_{{ name }}_idle_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_idle_vseq_c::body();

   `uvm_info("{{ name.upper() }}_IDLE_VSEQ", "Idle virtual sequence has started", UVM_HIGH)
   case (cfg.drv_mode)
      UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_1.upper() }}: do_idle_{{ mode_1 }}();
      UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_2.upper() }}: do_idle_{{ mode_2 }}();
   endcase

endtask : body


task uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_1 }}();

   fork
      begin
{% for segment in segments %}         forever begin
            do_idle_{{ mode_1 }}_{{ segment.name }}();
         end
{% endfor %}      end
   join

endtask : do_idle_{{ mode_1 }}


task uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_2 }}();

   fork
      begin
{% for segment in segments %}         forever begin
            do_idle_{{ mode_2 }}_{{ segment.name }}();
         end
{% endfor %}      end
   join

endtask : do_idle_{{ mode_2 }}


{% for segment in segments %}task uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_1 }}_{{ segment.name }}();
{% if segment.tx == mode_1 %}
   uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c  {{ mode_1 }}_{{ segment.name }}_seq_item;
   `uvm_create_on({{ mode_1 }}_{{ segment.name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segment.name }}_sequencer)
   case (cfg.drv_idle)
      UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS: begin
         `uvm_rand_send_pri_with({{ mode_1 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
            // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_1 }}_{{ segment.name }}() for UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS
            //      Ex: {{ segment.name }}_req  == 0;
            //          {{ segment.name }}_be   == 0;
            //          {{ segment.name }}_data == 0;
         })
      end
      UVMA_{{ name.upper() }}_DRV_IDLE_RANDOM: begin
         `uvm_rand_send_pri_with({{ mode_1 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
            // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_1 }}_{{ segment.name }}() for UVMA_{{ name.upper() }}_DRV_IDLE_RANDOM
            // {{ segment.name }}_req == 0;
         })
      end
   endcase
{% else %}
   uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c  {{ mode_1 }}_{{ segment.name }}_seq_item;
   `uvm_create_on({{ mode_1 }}_{{ segment.name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segment.name }}_sequencer)
   `uvm_rand_send_pri_with({{ mode_1 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
      // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_1 }}_{{ segment.name }}()
      //      Ex: {{ segment.name }}_gnt == 0;
   })
{% endif %}
endtask : do_idle_{{ mode_1 }}_{{ segment.name }}

{% endfor %}
{% for segment in segments %}task uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_2 }}_{{ segment.name }}();
{% if segment.tx == mode_2 %}
   uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c  {{ mode_2 }}_{{ segment.name }}_seq_item;
   `uvm_create_on({{ mode_2 }}_{{ segment.name }}_seq_item, p_sequencer.{{ mode_2 }}_{{ segment.name }}_sequencer)
   case (cfg.drv_idle)
      UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS: begin
         `uvm_rand_send_pri_with({{ mode_2 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_2.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
            // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_2 }}_{{ segment.name }}() for UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS
            //      Ex: {{ segment.name }}_req  == 0;
            //      Ex: {{ segment.name }}_be   == 0;
            //      Ex: {{ segment.name }}_data == 0;
         })
      end
      UVMA_{{ name.upper() }}_DRV_IDLE_RANDOM: begin
         `uvm_rand_send_pri_with({{ mode_2 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_2.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
            // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_2 }}_{{ segment.name }}() for UVMA_{{ name.upper() }}_DRV_IDLE_RANDOM
            //      Ex: {{ segment.name }}_req == 0;
         })
      end
   endcase
{% else %}
   uvma_{{ name }}_{{ mode_2 }}_{{ segment.name }}_seq_item_c  {{ mode_2 }}_{{ segment.name }}_seq_item;
   `uvm_create_on({{ mode_2 }}_{{ segment.name }}_seq_item, p_sequencer.{{ mode_2 }}_{{ segment.name }}_sequencer)
   `uvm_rand_send_pri_with({{ mode_2 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_2.upper() }}_{{ segment.name.upper() }}_IDLE_SEQ_ITEM_PRI, {
      // TODO Implement uvma_{{ name }}_idle_vseq_c::do_idle_{{ mode_2 }}_{{ segment.name }}()
      //      Ex: {{ segment.name }}_gnt == 0;
   })
{% endif %}
endtask : do_idle_{{ mode_2 }}_{{ segment.name }}

{% endfor %}
`endif // __UVMA_{{ name.upper() }}_BASE_SEQ_SV__
