// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__


/**
 * Virtual Sequence taking in uvma_{{ name }}_seq_item_c instances and driving uvma_{{ name }}_drv_c with {{ mode_1.upper() }} Sequence Items.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c extends uvma_{{ name }}_base_vseq_c;

   `uvm_object_utils(uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c)

   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_{{ mode_1 }}_drv_vseq");

   /**
    * Infinite loop maintaining 2 threads:
    * 1. Main sequence: wait for reset, drive traffic
    * 2. Reset trigger for mid-sim reset which resets the main sequence.
    */
   extern virtual task body();

   /**
    * Appends #cfg to seq_item and resets heartbeat monitor.
    */
   extern function void process_req(ref uvma_{{ name }}_seq_item_c seq_item);

   /**
    * Converts uvma_{{ name }}_seq_item_c instances into {{ mode_1.upper() }} Sequence Items.
    */
   extern task drive_req(ref uvma_{{ name }}_seq_item_c seq_item);

   {% for segment in segments %}{% if segment.rx == mode_1 %}/**
    * Drives uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_drv_c.
    */
   extern task drive_{{ segment.name }}();
{% endif %}{% endfor %}
endclass : uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c


function uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::new(string name="uvma_{{ name }}_{{ mode_1 }}_drv_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::body();

   uvma_{{ name }}_seq_item_c  seq_item;
   forever begin
      fork
         begin
            `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", "Waiting for post_reset", UVM_DEBUG)
            wait (cntxt.reset_state == UVML_RESET_STATE_POST_RESET);
            forever begin
               `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", "Waiting for next sequence item", UVM_DEBUG)
               p_sequencer.get_next_item    (seq_item);
               `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", {"Got next sequence item:\n", seq_item.sprint()}, UVM_DEBUG)
               process_req                  (seq_item);
               drive_req                    (seq_item);
               p_sequencer.seq_item_ap.write(seq_item);
               p_sequencer.item_done();
            end
         end
{% for segment in segments %}{% if segment.rx == mode_1 %}         begin
            drive_{{ segment.name }}();
         end
{% endif %}{% endfor %}         begin
            wait (cntxt.reset_state == UVML_RESET_STATE_POST_RESET);
            wait (cntxt.reset_state != UVML_RESET_STATE_POST_RESET);
         end
      join_any
      disable fork;
   end

endtask : body


function void uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::process_req(ref uvma_{{ name }}_seq_item_c seq_item);

   seq_item.cfg = cfg;
   `uvm_info("{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ", $sformatf("Processed sequence item:\n%s", seq_item.sprint()), UVM_HIGH)
   `uvml_hrtbt_owner(p_sequencer)

endfunction : process_req


task uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive_req(ref uvma_{{ name }}_seq_item_c seq_item);

   // TODO Implement uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive()
   //      Ex: int unsigned  num_packed_bits;
   //          uvma_{{ name }}_data_b_t  data_cycle;
   //          uvma_{{ name }}_be_b_t    be_cycle;
   //          uvma_{{ name }}_word_b_t  packed_bytes[];
   //          uvma_{{ name }}_{{ mode_1 }}_{{ segments[0].name }}_seq_item_c  {{ mode_1 }}_{{ segments[0].name }}_seq_item;
   //          num_packed_bits = seq_item.pack_bytes(packed_bytes);
   //          for (int unsigned ii=0; ii<packed_bytes.size(); ii++) begin
   //             cntxt.drv_data_q.push_back(packed_bytes[ii]);
   //          end
   //          do begin
   //             for (int unsigned ii=0; ii<cfg.data_width; ii++) begin
   //                if ($urandom_range(0,100)>seq_item.bandwidth_pct) begin
   //                   be_cycle[ii] = 0;
   //                end
   //                else begin
   //                   if (cntxt.drv_data_q.size()) begin
   //                      data_cycle[ii] = cntxt.drv_data_q.pop_front();
   //                      be_cycle  [ii] = 1;
   //                   end
   //                end
   //             end
   //             do begin
   //                `uvm_create_on({{ mode_1 }}_{{ segments[0].name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segments[0].name }}_sequencer)
   //                `uvm_rand_send_pri_with({{ mode_1 }}_{{ segments[0].name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_SEQ_ITEM_PRI, {
   //                   {{ segments[0].name }}_req  == 1'b1;
   //                   {{ segments[0].name }}_be   == be_cycle  ;
   //                   {{ segments[0].name }}_data == data_cycle;
   //                })
   //             end while ({{ mode_1 }}_{{ segments[0].name }}_seq_item.{{ segments[0].name }}_gnt !== 1'b1);
   //          end while (cntxt.drv_data_q.size());

endtask : drive_req


{% for segment in segments %}{% if segment.rx == mode_1 %}task uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive_{{ segment.name }}();

   uvma_{{ name }}_{{ mode_1 }}_{{ segment.name }}_seq_item_c  {{ mode_1 }}_{{ segment.name }}_seq_item;
   wait (cntxt.reset_state == UVML_RESET_STATE_POST_RESET);
   forever begin
      `uvm_create_on({{ mode_1 }}_{{ segment.name }}_seq_item, p_sequencer.{{ mode_1 }}_{{ segment.name }}_sequencer)
      `uvm_rand_send_pri_with({{ mode_1 }}_{{ segment.name }}_seq_item, `UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_SEQ_ITEM_PRI, {
         // TODO Constrain uvma_{{ name }}_{{ mode_1 }}_drv_vseq_c::drive_{{ segment.name }}()
      })
   end

endtask : drive_{{ segment.name }}


{% endif %}{% endfor %}`endif // __UVMA_{{ name.upper() }}_{{ mode_1.upper() }}_DRV_VSEQ_SV__
