// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__
`define __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__


/**
 * Virtual Sequence generating a set number of random data 'packets' with min/max knobs for payload size and gap control.
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_rand_stim_vseq_c extends uvma_{{ name }}_base_vseq_c;

   /// @name Knobs
   /// @{
   rand int unsigned  num_pkts; ///< Number of packets to generate.
   rand int unsigned  min_size; ///< Minimum packet payload size.
   rand int unsigned  max_size; ///< Maximum packet payload size.
   rand int unsigned  min_gap ; ///< Minimum number of cycles between packets.
   rand int unsigned  max_gap ; ///< Maximum number of cycles between packets.
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_rand_stim_vseq_c)
      `uvm_field_int(num_pkts, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_size, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_size, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_gap , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_gap , UVM_DEFAULT + UVM_DEC)
   `uvm_object_utils_end


   /**
    * Enforces min/max and bounds #num_pkts to an upper limit.
    */
   constraint limits_cons {
      num_pkts inside {[1:uvma_{{ name }}_rand_stim_vseq_max_num_pkts]};
      min_size >= uvma_{{ name }}_rand_stim_vseq_min_size;
      min_size <= max_size;
      max_size <= uvma_{{ name }}_rand_stim_vseq_max_size;
      min_gap <= max_gap;
      max_gap <= uvma_{{ name }}_rand_stim_vseq_max_gap;
   }


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_rand_stim_vseq");

   /**
    * Generates #num_pkts Sequence Items on uvma_{{ name }}_vsqr_c according to knobs values.
    */
   extern virtual task body();

endclass : uvma_{{ name }}_rand_stim_vseq_c


function uvma_{{ name }}_rand_stim_vseq_c::new(string name="uvma_{{ name }}_rand_stim_vseq");

   super.new(name);

endfunction : new


task uvma_{{ name }}_rand_stim_vseq_c::body();

   uvma_{{ name }}_seq_item_c  req;
   int unsigned  gap_size;
   `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", "Starting sequence", UVM_MEDIUM)
   for (int unsigned ii=0; ii<num_pkts; ii++) begin
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Starting packet %0d/%0d", (ii+1), num_pkts), UVM_HIGH)
      `uvm_create_on(req, p_sequencer)
      `uvm_rand_send_with(req, {
         data_size inside {[min_size:max_size]};
      })
      gap_size = $urandom_range(min_gap, max_gap);
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Done driving packet %0d/%0d data - Waiting %0d gap cycle(s)", (ii+1), num_pkts, gap_size), UVM_DEBUG)
      for (int unsigned jj=0; jj<gap_size; jj++) begin
         `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Packet %0d/%0d - Waiting %0d/%0d gap cycle(s)", (ii+1), num_pkts, (jj+1), gap_size), UVM_DEBUG)
         @(cntxt.vif.mon_{{ segments[0].name }}_cb);
         `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Packet %0d/%0d - Done waiting %0d/%0d gap cycle(s)", (ii+1), num_pkts, (jj+1), gap_size), UVM_DEBUG)
      end
      `uvm_info("{{ name.upper() }}_RAND_STIM_VSEQ", $sformatf("Finished packet %0d/%0d", (ii+1), num_pkts), UVM_HIGH)
   end

endtask : body


`endif // __UVMA_{{ name.upper() }}_RAND_STIM_VSEQ_SV__
