// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__
`define __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__


/**
 * Sequence Item created by {{ full_name }} Sequences. Analog of uvma_{{ name }}_mon_trn_c
 * @ingroup uvma_{{ name }}_seq
 */
class uvma_{{ name }}_seq_item_c extends uvml_seq_item_c;

   uvma_{{ name }}_cfg_c  cfg; ///< Agent configuration handle

   /// @name Data
   /// @{
   rand uvma_{{ name }}_word_b_t  data[]; ///< Payload Data
   // TODO Add data fields
   //      Ex: rand uvma_{{ name }}_user_b_t  user; ///< User Data
   /// @}

   /// @name Metadata
   /// @{
   rand int unsigned  data_size    ; ///< Size of #data
   rand int unsigned  bandwidth_pct; ///< % of #be to be used on average per active cycle
   /// @}


   `uvm_object_utils_begin(uvma_{{ name }}_seq_item_c)
      `uvm_field_int(data_size, UVM_DEFAULT + UVM_DEC + UVM_NOPACK)
      `uvm_field_int(bandwidth_pct, UVM_DEFAULT + UVM_DEC + UVM_NOPACK)
      `uvm_field_array_int(data, UVM_DEFAULT)
   `uvm_object_utils_end


   /**
    * Bounds #data_size and #data.size().  Ensures #bandwitdh_pct is a legal value.
    */
   constraint limits_cons {
      data.size() == data_size;
      data_size >= `UVMA_{{ name.upper() }}_DATA_MIN_SIZE;
      data_size <= `UVMA_{{ name.upper() }}_DATA_MAX_SIZE;
      bandwidth_pct > 0;
      bandwidth_pct <= 100;
   }


   /**
    * Default constructor.
    */
   extern function new(string name="uvma_{{ name }}_seq_item");

   /**
    * Describes transaction as metadata for uvml_logs_metadata_logger_c.
    */
   extern function uvml_metadata_t get_metadata();

endclass : uvma_{{ name }}_seq_item_c


function uvma_{{ name }}_seq_item_c::new(string name="uvma_{{ name }}_seq_item");

   super.new(name);

endfunction : new


function uvml_metadata_t uvma_{{ name }}_seq_item_c::get_metadata();

   string        data_size_str, bandwidth_pct_str, data_str;
   int unsigned  field_count = 0;
   data_size_str = $sformatf("%0d", data_size);
   get_metadata.push_back('{
      index     : field_count,
      value     : data_size_str,
      col_name  : "size",
      col_width : data_size_str.len(),
      col_align : UVML_TEXT_ALIGN_RIGHT,
      data_type : UVML_FIELD_INT
   });
   field_count++;
   bandwidth_pct_str = $sformatf("%0d", bandwidth_pct);
   get_metadata.push_back('{
      index     : field_count,
      value     : bandwidth_pct_str,
      col_name  : "BAND%",
      col_width : bandwidth_pct_str.len(),
      col_align : UVML_TEXT_ALIGN_RIGHT,
      data_type : UVML_FIELD_INT
   });
   field_count++;
   data_str = $sformatf("%h%h ... %h%h", data[data_size-1], data[data_size-2], data[1], data[0]);
   get_metadata.push_back('{
      index     : field_count,
      value     : data_str,
      col_name  : "data",
      col_width : data_str.len(),
      col_align : UVML_TEXT_ALIGN_RIGHT,
      data_type : UVML_FIELD_INT
   });
   field_count++;

endfunction : get_metadata


`endif // __UVMA_{{ name.upper() }}_SEQ_ITEM_SV__
