// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVMA_{{ name.upper() }}_IF_SV__
`define __UVMA_{{ name.upper() }}_IF_SV__


/**
 * Encapsulates all signals and clocking of the {{ full_name }}.
 * @ingroup uvma_{{ name }}_misc
 */
interface uvma_{{ name }}_if #(
   parameter DATA_WIDTH  = `UVMA_{{ name.upper() }}_DATA_MAX_WIDTH  ///< Width of the data signal in bytes
)
(
   input  clk    , ///< All signal timings are related to the rising edge of #clk
   input  reset_n  ///< Active LOW reset signal
);

   {% for segment in segments %}/// @name '{{ segment.name.upper() }}' signals
   /// @{
   // TODO Add {{ segment.name }} signals to interface
   //      Ex: wire                           {{ segment.name }}_req ; ///< {{ segment.name.upper() }} Request
   //          wire                           {{ segment.name }}_gnt ; ///< {{ segment.name.upper() }} Grant
   //          wire [(DATA_WIDTH-1):0]        {{ segment.name }}_be  ; ///< {{ segment.name.upper() }} Byte Enabled
   //          wire [(DATA_WIDTH-1):0][7:0]   {{ segment.name }}_data; ///< {{ segment.name.upper() }} Data Transferred
   /// @}

{% endfor %}

   /**
    * Used by DUT in '{{ mode_1 }}' mode.
    */
   clocking dut_{{ mode_1 }}_cb @(posedge clk);
      // TODO Define dut_{{ mode_1 }}_cb signals
      //      Ex: input  abc, xyz;
      //          output def, qrs;
   endclocking : dut_{{ mode_1 }}_cb

   /**
    * Used by DUT in '{{ mode_2 }}' mode.
    */
   clocking dut_{{ mode_2 }}_cb @(posedge clk);
      // TODO Define dut_{{ mode_2 }}_cb signals
      //      Ex: input  def, qrs;
      //          output abc, xyz;
   endclocking : dut_{{ mode_2 }}_cb

   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_drv_{{ segment.tx }}_{{ segment.name }}_c.
    */
   clocking drv_{{ segment.tx }}_{{ segment.name }}_cb @(posedge clk);
      // TODO Define drv_{{ segment.tx }}_{{ segment.name }}_cb signals
      //      Ex: output {{ segment.name }}_req, {{ segment.name }}_be, {{ segment.name }}_data;
      //          input  {{ segment.name }}_gnt;
   endclocking : drv_{{ segment.tx }}_{{ segment.name }}_cb
{% endfor %}
   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_drv_{{ segment.rx }}_{{ segment.name }}_c.
    */
   clocking drv_{{ segment.rx }}_{{ segment.name }}_cb @(posedge clk);
      // TODO Define drv_{{ segment.rx }}_{{ segment.name }}_cb signals
      //      Ex: input  {{ segment.name }}_req, {{ segment.name }}_be, {{ segment.name }}_data;
      //          output {{ segment.name }}_gnt;
   endclocking : drv_{{ segment.rx }}_{{ segment.name }}_cb
{% endfor %}
   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_mon_c.
    */
   clocking mon_{{ segment.name }}_cb @(posedge clk);
      // TODO Define mon_{{ segment.name }}_cb signals
      //      Ex: input  {{ segment.name }}_gnt, {{ segment.name }}_req, {{ segment.name }}_be, {{ segment.name }}_data;
   endclocking : mon_{{ segment.name }}_cb
{% endfor %}

   /**
    * Used by target DUT.
    */
   modport dut_{{ mode_1 }}_mp (
      clocking dut_{{ mode_1 }}_cb,
      input    clk    ,
      input    reset_n
   );

   /**
    * Used by target DUT.
    */
   modport dut_{{ mode_2 }}_mp (
      clocking dut_{{ mode_2 }}_cb,
      input    clk    ,
      input    reset_n
   );

   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_drv_{{ segment.tx }}_{{ segment.name }}_c.
    */
   modport drv_{{ segment.tx }}_{{ segment.name }}_mp (
      clocking drv_{{ segment.tx }}_{{ segment.name }}_cb,
      input    clk    ,
      input    reset_n
   );
{% endfor %}
   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_drv_{{ segment.rx }}_{{ segment.name }}_c.
    */
   modport drv_{{ segment.rx }}_{{ segment.name }}_mp (
      clocking drv_{{ segment.rx }}_{{ segment.name }}_cb,
      input    clk    ,
      input    reset_n
   );
{% endfor %}
   {% for segment in segments %}/**
    * Used by uvma_{{ name }}_mon_c.
    */
   modport mon_{{ segment.name }}_mp(
      clocking mon_{{ segment.name }}_cb,
      input    clk    ,
      input    reset_n
   );
{% endfor %}
endinterface : uvma_{{ name }}_if


`endif // __UVMA_{{ name.upper() }}_IF_SV__
