// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_COV_MODEL_SV__
`define __UVME_{{ name.upper() }}_ST_COV_MODEL_SV__


/**
 * Component encapsulating {{ full_name }} Agent Self-Test Environment functional coverage model.
 * @ingroup uvme_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_cov_model_c extends uvma_{{ name }}_cov_model_c;

   `uvm_component_utils(uvme_{{ name }}_st_cov_model_c)


   /**
    * Agent configuration functional coverage.
    */
   covergroup {{ name }}_st_cfg_cg;
      bypass_mode_cpt     : coverpoint cfg.bypass_mode    ;
      is_active_cpt       : coverpoint cfg.is_active      ;
      reset_type_cpt      : coverpoint cfg.reset_type     ;
      trn_log_enabled_cpt : coverpoint cfg.trn_log_enabled;
      drv_mode_cpt        : coverpoint cfg.drv_mode       ;
      drv_idle_cpt        : coverpoint cfg.drv_idle       ;
   endgroup : {{ name }}_st_cfg_cg

   /**
    * Agent context functional coverage.
    */
   covergroup {{ name }}_st_cntxt_cg;
      reset_state_cpt : coverpoint cntxt.reset_state;
   endgroup : {{ name }}_st_cntxt_cg

   /**
    * Sequence item functional coverage.
    */
   covergroup {{ name }}_st_seq_item_cg;
      {% if mm %}access_type_cpt : coverpoint seq_item.access_type;
      {% if sbe %}be_cpt : coverpoint seq_item.be;
      {% endif %}{% else %}data_size_cpt : coverpoint seq_item.data_size;
{% endif %}   endgroup : {{ name }}_st_seq_item_cg

{% if mm or not duplex %}   /**
    *  Monitor Transaction functional coverage.
    */
   covergroup {{ name }}_st_mon_trn_cg;
      {% if mm %}access_type_cpt : coverpoint mon_trn.access_type;
      {% if sbe %}be_cpt : coverpoint mon_trn.be;
      {% endif %}{% else %}data_size_cpt : coverpoint mon_trn.data_size;
{% endif %}endgroup : {{ name }}_st_mon_trn_cg
{% else %}   /**
    * {{ dir_1.upper() }} Monitor Transaction functional coverage.
    */
   covergroup {{ name }}_st_{{ dir_1 }}_mon_trn_cg;
      data_size_cpt : coverpoint {{ dir_1 }}_mon_trn.data_size;
   endgroup : {{ name }}_st_{{ dir_1 }}_mon_trn_cg

   /**
    * {{ dir_2.upper() }} Monitor Transaction functional coverage.
    */
   covergroup {{ name }}_st_{{ dir_2 }}_mon_trn_cg;
      data_size_cpt : coverpoint {{ dir_2 }}_mon_trn.data_size;
   endgroup : {{ name }}_st_{{ dir_2 }}_mon_trn_cg
{% endif %}
{% for segment in segments %}   /**
    * {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Item functional coverage.
    */
   covergroup {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg;
      // TODO Implement {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg
      //      Ex: abc_cpt : coverpoint {{ mode_1 }}_{{ segment.name }}_seq_item.abc;
   endgroup : {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg

{% endfor %}{% for segment in segments %}   /**
    * {{ mode_1.upper() }} {{ segment.name.upper() }} Sequence Item functional coverage.
    */
   covergroup {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg;
      // TODO Implement {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg
      //      Ex: abc_cpt : coverpoint {{ mode_2 }}_{{ segment.name }}_seq_item.abc;
   endgroup : {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg

{% endfor %}{% for segment in segments %}   /**
    * {{ mode_1.upper() }} {{ segment.name.upper() }} Monitor Transaction functional coverage.
    */
   covergroup {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg;
      // TODO Implement {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg
      //      Ex: abc_cpt : coverpoint {{ mode_1 }}_{{ segment.name }}_mon_trn.abc;
   endgroup : {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg

{% endfor %}{% for segment in segments %}   /**
    * {{ mode_2.upper() }} {{ segment.name.upper() }} Monitor Transaction functional coverage.
    */
   covergroup {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg;
      // TODO Implement {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg
      //      Ex: abc_cpt : coverpoint {{ mode_2 }}_{{ segment.name }}_mon_trn.abc;
   endgroup : {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg

{% endfor %}
   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_cov_model", uvm_component parent=null);

   /**
    * Samples #cfg.
    */
   extern virtual function void end_of_elaboration_phase(uvm_phase phase);

   /**
    * Samples {{ name }}_st_cfg_cg.
    */
   extern virtual function void sample_cfg();

   /**
    * Samples {{ name }}_st_cntxt_cg.
    */
   extern virtual function void sample_cntxt();

   /**
    * Samples {{ name }}_st_seq_item_cg.
    */
   extern virtual function void sample_seq_item();

{% if mm or not duplex %}
   /**
    * Samples {{ name }}_st_mon_trn_cg.
    */
   extern virtual function void sample_mon_trn();

{% else %}   /**
    * Samples {{ name }}_st_{{ dir_1 }}_mon_trn_cg.
    */
   extern virtual function void sample_{{ dir_1 }}_mon_trn();

   /**
    * Samples {{ name }}_st_{{ dir_2 }}_mon_trn_cg.
    */
   extern virtual function void sample_{{ dir_2 }}_mon_trn();

{% endif %}{% for segment in segments %}   /**
    * Samples {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg.
    */
   extern virtual function void sample_{{ mode_1 }}_{{ segment.name }}_seq_item();

{% endfor %}{% for segment in segments %}   /**
    * Samples {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg.
    */
   extern virtual function void sample_{{ mode_2 }}_{{ segment.name }}_seq_item();

{% endfor %}{% for segment in segments %}   /**
    * Samples {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg.
    */
   extern virtual function void sample_{{ mode_1 }}_{{ segment.name }}_mon_trn();

{% endfor %}{% for segment in segments %}   /**
    * Samples {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg.
    */
   extern virtual function void sample_{{ mode_2 }}_{{ segment.name }}_mon_trn();

{% endfor %}endclass : uvme_{{ name }}_st_cov_model_c


function uvme_{{ name }}_st_cov_model_c::new(string name="uvme_{{ name }}_st_cov_model", uvm_component parent=null);

   super.new(name, parent);
   {{ name }}_st_cfg_cg      = new();
   {{ name }}_st_cntxt_cg    = new();
   {{ name }}_st_seq_item_cg = new();
{% if mm or not duplex %}   {{ name }}_st_mon_trn_cg  = new();
{% else %}   {{ name }}_st_{{ dir_1 }}_mon_trn_cg = new();
   {{ name }}_st_{{ dir_2 }}_mon_trn_cg = new();
{% endif %}{% for segment in segments %}   {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg = new();
{% endfor %}{% for segment in segments %}   {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg = new();
{% endfor %}{% for segment in segments %}   {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg  = new();
{% endfor %}{% for segment in segments %}   {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg  = new();
{% endfor %}
endfunction : new


function void uvme_{{ name }}_st_cov_model_c::end_of_elaboration_phase(uvm_phase phase);

   super.end_of_elaboration_phase(phase);
   sample_cfg();

endfunction : end_of_elaboration_phase


function void uvme_{{ name }}_st_cov_model_c::sample_cfg();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling Configuration:\n%s", cfg.sprint()), UVM_DEBUG)
   {{ name }}_st_cfg_cg.sample();

endfunction : sample_cfg


function void uvme_{{ name }}_st_cov_model_c::sample_cntxt();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling Context:\n%s", cntxt.sprint()), UVM_DEBUG)
   {{ name }}_st_cntxt_cg.sample();

endfunction : sample_cntxt


function void uvme_{{ name }}_st_cov_model_c::sample_seq_item();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling Sequence Item:\n%s", seq_item.sprint()), UVM_DEBUG)
   {{ name }}_st_seq_item_cg.sample();

endfunction : sample_seq_item


{% if mm or not duplex %}function void uvme_{{ name }}_st_cov_model_c::sample_mon_trn();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling Monitor Transaction:\n%s", mon_trn.sprint()), UVM_DEBUG)
   {{ name }}_st_mon_trn_cg.sample();

endfunction : sample_mon_trn

{% else %}function void uvme_{{ name }}_st_cov_model_c::sample_{{ dir_1 }}_mon_trn();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ dir_1.upper() }} Monitor Transaction:\n%s", {{ dir_1 }}_mon_trn.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ dir_1 }}_mon_trn_cg.sample();

endfunction : sample_{{ dir_1 }}_mon_trn


function void uvme_{{ name }}_st_cov_model_c::sample_{{ dir_2 }}_mon_trn();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ dir_2.upper() }} Monitor Transaction:\n%s", {{ dir_2 }}_mon_trn.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ dir_2 }}_mon_trn_cg.sample();

endfunction : sample_{{ dir_2 }}_mon_trn

{% endif %}
{% for segment in segments %}function void uvme_{{ name }}_st_cov_model_c::sample_{{ mode_1 }}_{{ segment.name }}_seq_item();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ mode_1.upper() }} '{{ segment.name.upper() }}' Sequence Item:\n%s", {{ mode_1 }}_{{ segment.name }}_seq_item.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_seq_item_cg.sample();

endfunction : sample_{{ mode_1 }}_{{ segment.name }}_seq_item


{% endfor %}{% for segment in segments %}function void uvme_{{ name }}_st_cov_model_c::sample_{{ mode_2 }}_{{ segment.name }}_seq_item();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ mode_2.upper() }} '{{ segment.name.upper() }}' Sequence Item:\n%s", {{ mode_2 }}_{{ segment.name }}_seq_item.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_seq_item_cg.sample();

endfunction : sample_{{ mode_2 }}_{{ segment.name }}_seq_item


{% endfor %}{% for segment in segments %}function void uvme_{{ name }}_st_cov_model_c::sample_{{ mode_1 }}_{{ segment.name }}_mon_trn();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ mode_1.upper() }} '{{ segment.name.upper() }}' Monitor Transaction:\n%s", {{ mode_1 }}_{{ segment.name }}_mon_trn.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ mode_1 }}_{{ segment.name }}_mon_trn_cg.sample();

endfunction : sample_{{ mode_1 }}_{{ segment.name }}_mon_trn


{% endfor %}{% for segment in segments %}function void uvme_{{ name }}_st_cov_model_c::sample_{{ mode_2 }}_{{ segment.name }}_mon_trn();

   `uvm_info("UVME_{{ name.upper() }}_ST_COV_MODEL", $sformatf("Sampling {{ mode_2.upper() }} '{{ segment.name.upper() }}' Monitor Transaction:\n%s", {{ mode_2 }}_{{ segment.name }}_mon_trn.sprint()), UVM_DEBUG)
   {{ name }}_st_{{ mode_2 }}_{{ segment.name }}_mon_trn_cg.sample();

endfunction : sample_{{ mode_2 }}_{{ segment.name }}_mon_trn


{% endfor %}`endif // __UVME_{{ name.upper() }}_ST_COV_MODEL_SV__
