// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_ENV_SV__
`define __UVME_{{ name.upper() }}_ST_ENV_SV__


/**
 * Top-level component that encapsulates, builds and connects all other {{ full_name }} UVM Agent Self-Test Environment
 * (uvme_{{ name }}_st_env_c) components.
 * @ingroup uvme_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_env_c extends uvml_env_c;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Configuration handle.  Must be provided by component instantiating this environment.
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Context handle.  Can be provided by component instantiating this environment.
   /// @}

{% if mm %}   /// @name Register Abstraction Layer (RAL)
   /// @{
   uvme_{{ name }}_st_reg_block_c  reg_block  ; ///< Mock Register Block used to test #reg_adapter.
   uvma_{{ name }}_reg_adapter_c   reg_adapter; ///< Converts between UVM 'register ops' and Sequence Items.
   /// @}

{% endif %}   /// @name Agents
   /// @{
   uvma_{{ name }}_agent_c  {{ mode_1 }}_agent; ///< {{ mode_1.upper() }} Agent instance.
   uvma_{{ name }}_agent_c  {{ mode_2 }}_agent; ///< {{ mode_2.upper() }} Agent instance.
   uvma_{{ name }}_agent_c  passive_agent; ///< Passive Agent instance.
   /// @}

   /// @name Components
   /// @{
   uvme_{{ name }}_st_vsqr_c  vsequencer; ///< Virtual Sequencer on which Virtual Sequences are run.
   uvme_{{ name }}_st_prd_c   predictor ; ///< Feeds #scoreboard's expected ports with monitor transactions.
   uvme_{{ name }}_st_sb_c    scoreboard; ///< Ensures that monitored transactions from data streams match.
   /// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_env_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_env", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null
    * 2. Assigns #cfg and #cntxt sub-handles
    * 3. Builds all components
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * 1. Connects agents to predictor
    * 2. Connects predictor & agents to scoreboard
    * 3. Assembles virtual sequencer handles
    * 4. Connects agents to coverage model
    */
   extern virtual function void connect_phase(uvm_phase phase);

   {% if mm %}/**
    * Starts memory handler Virtual Sequence.
    */
   extern virtual task run_phase(uvm_phase phase);
{% endif %}   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Assigns configuration handles to components using uvm_config_db.
    */
   extern function void assign_cfg();

   /**
    * Assigns context handles to components using uvm_config_db.
    */
   extern function void assign_cntxt();

   /**
    * Creates agent components.
    */
   extern function void create_agents();

{% if mm %}   /**
    * Creates #reg_adapter.
    */
   extern function void create_reg_adapter();
{% endif %}   /**
    * Creates additional (non-agent) environment components (and objects).
    */
   extern function void create_env_components();

   /**
    * Creates Virtual Sequencer.
    */
   extern function void create_vsequencer();

   /**
    * Connects agents to predictor.
    */
   extern function void connect_predictor();

   /**
    * Connects scoreboards to agents/predictor.
    */
   extern function void connect_scoreboard();

   {% if mm %}/**
    * Connects #reg_model to #{{ mode_1 }}_agent.
    */
   extern function void connect_reg_block();
{% endif %}   /**
    * Assembles virtual sequencer from agent sequencers.
    */
   extern function void assemble_vsequencer();

endclass : uvme_{{ name }}_st_env_c


function uvme_{{ name }}_st_env_c::new(string name="uvme_{{ name }}_st_env", uvm_component parent=null);

   super.new(name, parent);
   set_type_override_by_type(uvma_{{ name }}_cov_model_c::get_type(), uvme_{{ name }}_st_cov_model_c::get_type());

endfunction : new


function void uvme_{{ name }}_st_env_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg();
   if (cfg.enabled) begin
      get_cntxt            ();
      assign_cfg           ();
      assign_cntxt         ();
      create_agents        ();
      {% if mm %}create_reg_adapter   ();
      {% endif %}create_env_components();
      if (cfg.is_active) begin
         create_vsequencer();
      end
   end

endfunction : build_phase


function void uvme_{{ name }}_st_env_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   if (cfg.enabled) begin
      {% if mm %}connect_reg_block();
      {% endif %}assemble_vsequencer();
      if (cfg.scoreboarding_enabled) begin
         connect_predictor ();
         connect_scoreboard();
      end
   end

endfunction: connect_phase


{% if mm %}task uvme_{{ name }}_st_env_c::run_phase(uvm_phase phase);

   uvma_{{ name }}_{{ mode_2 }}_handler_mem_vseq_c  {{ mode_2 }}_mem_vseq;
   super.run_phase(phase);
   {{ mode_2 }}_mem_vseq = uvma_{{ name }}_{{ mode_2 }}_handler_mem_vseq_c::type_id::create("{{ mode_2 }}_mem_vseq");
   {{ mode_2 }}_mem_vseq.start(vsequencer.{{ mode_2 }}_vsequencer);

endtask : run_phase


{% endif %}function void uvme_{{ name }}_st_env_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_ENV", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_env_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_ENV", "Context handle is null")
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_env_c::assign_cfg();

   uvm_config_db#(uvme_{{ name }}_st_cfg_c)::set(this, "*", "cfg", cfg);
   uvm_config_db#(uvma_{{ name }}_cfg_c   )::set(this, "{{ mode_1 }}_agent", "cfg", cfg.{{ mode_1 }}_cfg);
   uvm_config_db#(uvma_{{ name }}_cfg_c   )::set(this, "{{ mode_2 }}_agent", "cfg", cfg.{{ mode_2 }}_cfg);
   uvm_config_db#(uvma_{{ name }}_cfg_c   )::set(this, "passive_agent", "cfg", cfg.passive_cfg);

endfunction: assign_cfg


function void uvme_{{ name }}_st_env_c::assign_cntxt();

   uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::set(this, "*", "cntxt", cntxt);
   uvm_config_db#(uvma_{{ name }}_cntxt_c   )::set(this, "{{ mode_1 }}_agent", "cntxt", cntxt.{{ mode_1 }}_cntxt);
   uvm_config_db#(uvma_{{ name }}_cntxt_c   )::set(this, "{{ mode_2 }}_agent", "cntxt", cntxt.{{ mode_2 }}_cntxt);
   uvm_config_db#(uvma_{{ name }}_cntxt_c   )::set(this, "passive_agent", "cntxt", cntxt.passive_cntxt);

endfunction: assign_cntxt


function void uvme_{{ name }}_st_env_c::create_agents();

   {{ mode_1 }}_agent = uvma_{{ name }}_agent_c::type_id::create("{{ mode_1 }}_agent", this);
   {{ mode_2 }}_agent = uvma_{{ name }}_agent_c::type_id::create("{{ mode_2 }}_agent", this);
   passive_agent = uvma_{{ name }}_agent_c::type_id::create("passive_agent", this);

endfunction: create_agents


function void uvme_{{ name }}_st_env_c::create_env_components();

   if (cfg.scoreboarding_enabled) begin
      predictor  = uvme_{{ name }}_st_prd_c::type_id::create("predictor" , this);
      scoreboard = uvme_{{ name }}_st_sb_c ::type_id::create("scoreboard", this);
   end

endfunction: create_env_components


{% if mm %}function void uvme_{{ name }}_st_env_c::create_reg_adapter();

   reg_adapter = uvma_{{ name }}_reg_adapter_c::type_id::create("reg_adapter");
   reg_block   = cfg.reg_block;

endfunction: create_reg_adapter


{% endif %}function void uvme_{{ name }}_st_env_c::create_vsequencer();

   vsequencer = uvme_{{ name }}_st_vsqr_c::type_id::create("vsequencer", this);

endfunction: create_vsequencer


function void uvme_{{ name }}_st_env_c::connect_predictor();

   {% if mm %}// Agents -> Predictor
   {{ mode_1 }}_agent.seq_item_ap.connect(predictor.{{ mode_1 }}_in_export);
   {{ mode_2 }}_agent.{{ mode_2 }}_{{ segments[0].name }}_seq_item_ap.connect(predictor.{{ mode_2 }}_in_export);
   {{ mode_1 }}_agent.mon_trn_ap.connect(predictor.e2e_in_export);
   {% else %}{% if duplex %}// Agents -> Predictor
   {{ mode_1 }}_agent.seq_item_ap.connect(predictor.{{ mode_1 }}_in_export);
   {{ mode_2 }}_agent.seq_item_ap.connect(predictor.{{ mode_2 }}_in_export);
   {{ mode_1 }}_agent.{{ dir_1 }}_mon_trn_ap.connect(predictor.{{ dir_1 }}_in_export);
   {{ mode_2 }}_agent.{{ dir_2 }}_mon_trn_ap.connect(predictor.{{ dir_2 }}_in_export);
   {% else %}// Agents -> Predictor
   {{ mode_1 }}_agent.seq_item_ap.connect(predictor.agent_in_export);
   {{ mode_2 }}_agent.mon_trn_ap .connect(predictor.e2e_in_export  );
{% endif %}{% endif %}
endfunction: connect_predictor


function void uvme_{{ name }}_st_env_c::connect_scoreboard();

   {% if mm %}// Agents -> Scoreboard
   passive_agent.mon_trn_ap.connect(scoreboard.{{ mode_1 }}_act_export);
   passive_agent.mon_trn_ap.connect(scoreboard.{{ mode_2 }}_act_export);
   {{ mode_2 }}_agent.mon_trn_ap.connect(scoreboard.e2e_act_export);
   // Predictor -> Scoreboard
   predictor.{{ mode_1 }}_out_ap.connect(scoreboard.{{ mode_1 }}_exp_export);
   predictor.{{ mode_2 }}_out_ap.connect(scoreboard.{{ mode_2 }}_exp_export);
   predictor.e2e_out_ap.connect(scoreboard.e2e_exp_export);
   {% else %}{% if duplex %}// Agents -> Scoreboard
   passive_agent.{{ dir_1 }}_mon_trn_ap.connect(scoreboard.{{ mode_1 }}_act_export);
   passive_agent.{{ dir_2 }}_mon_trn_ap.connect(scoreboard.{{ mode_2 }}_act_export);
   {{ mode_2 }}_agent.{{ dir_1 }}_mon_trn_ap.connect(scoreboard.{{ dir_1 }}_act_export);
   {{ mode_1 }}_agent.{{ dir_2 }}_mon_trn_ap.connect(scoreboard.{{ dir_2 }}_act_export);
   // Predictor -> Scoreboard
   predictor.{{ mode_1 }}_out_ap.connect(scoreboard.{{ mode_1 }}_exp_export);
   predictor.{{ mode_2 }}_out_ap.connect(scoreboard.{{ mode_2 }}_exp_export);
   predictor.{{ dir_1 }}_out_ap.connect(scoreboard.{{ dir_1 }}_exp_export);
   predictor.{{ dir_2 }}_out_ap.connect(scoreboard.{{ dir_2 }}_exp_export);
   {% else %}// Agents -> Scoreboard
   passive_agent.mon_trn_ap.connect(scoreboard.agent_act_export);
   {{ mode_2 }}_agent.mon_trn_ap.connect(scoreboard.e2e_act_export);
   // Predictor -> Scoreboard
   predictor.agent_out_ap.connect(scoreboard.agent_exp_export);
   predictor.e2e_out_ap  .connect(scoreboard.e2e_exp_export  );
{% endif %}{% endif %}
endfunction: connect_scoreboard


{% if mm %}function void uvme_{{ name }}_st_env_c::connect_reg_block();

   reg_block.connect();
   reg_block.default_map.set_sequencer({{ mode_1 }}_agent.vsequencer, reg_adapter);
   reg_block.default_map.set_auto_predict(1);

endfunction: connect_reg_block


{% endif %}function void uvme_{{ name }}_st_env_c::assemble_vsequencer();

   vsequencer.{{ mode_1 }}_vsequencer = {{ mode_1 }}_agent.vsequencer;
   vsequencer.{{ mode_2 }}_vsequencer = {{ mode_2 }}_agent.vsequencer;
   vsequencer.passive_vsequencer = passive_agent.vsequencer;

endfunction: assemble_vsequencer


`endif // __UVME_{{ name.upper() }}_ST_ENV_SV__
