// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_PRD_SV__
`define __UVME_{{ name.upper() }}_ST_PRD_SV__


/**
 * Component implementing transaction-based prediction for {{ full_name }} UVM Agent Self-Testing Environment.
 * Predicts how the Passive Agent will observe stimulus from {{ mode_1.upper() }} & {{ mode_2.upper() }} Agents as well as end-to-end.
 * @ingroup uvme_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_prd_c extends uvm_component;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Environment configuration handle
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Environment context handle
   /// @}

   /// @name TLM
   /// @{
   uvm_analysis_export   #(uvma_{{ name }}_seq_item_c)  {{ mode_1 }}_in_export; ///< Input Port for Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_analysis_export   #(uvma_{{ name }}_{{ mode_2 }}_{{ segments[0].name }}_seq_item_c)  {{ mode_2 }}_in_export; ///<
   uvm_analysis_export   #(uvma_{{ name }}_mon_trn_c)  e2e_in_export; ///< Input Port for Monitor Transactions from {{ mode_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_seq_item_c)  {{ mode_1 }}_in_fifo; ///< FIFO of Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_{{ mode_2 }}_{{ segments[0].name }}_seq_item_c)  {{ mode_2 }}_in_fifo; ///< FIFO of {{ segments[0].name.upper() }} Sequence Items from {{ mode_2.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c)  e2e_in_fifo; ///< Input Port for Monitor Transactions from {{ mode_1.upper() }} Agent.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c)  {{ mode_1 }}_out_ap; ///< Output Port for Monitor Transactions for {{ mode_1.upper() }} Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c)  {{ mode_2 }}_out_ap; ///< Output Port for Monitor Transactions for {{ mode_2.upper() }} Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c)  e2e_out_ap; ///< Output Port for Monitor Transactions for end-to-end Scoreboard Expected.
   /// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_prd_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_prd", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null
    * 2. Creates TLM objects
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects Exports to FIFOs.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Forks all processing tasks.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates TLM FIFOs and Analysis Ports.
    */
   extern function void create_tlm_objects();

   /**
    * Connects Exports to FIFOs.
    */
   extern function void connect_ports();

   /**
    * Predicts {{ mode_1.upper() }} data path.
    */
   extern task predict_{{ mode_1 }}();

   /**
    * Predicts {{ mode_2.upper() }} data path.
    */
   extern task predict_{{ mode_2 }}();

   /**
    * Predicts end-to-end data path.
    */
   extern task predict_e2e();

endclass : uvme_{{ name }}_st_prd_c


function uvme_{{ name }}_st_prd_c::new(string name="uvme_{{ name }}_st_prd", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvme_{{ name }}_st_prd_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg           ();
   get_cntxt         ();
   create_tlm_objects();

endfunction : build_phase


function void uvme_{{ name }}_st_prd_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_ports();

endfunction: connect_phase


task uvme_{{ name }}_st_prd_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   fork
      predict_{{ mode_1 }}();
      predict_{{ mode_2 }}();
      predict_e2e();
   join_none

endtask: run_phase


function void uvme_{{ name }}_st_prd_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_PRD", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_prd_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_PRD", "Context handle is null")
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_prd_c::create_tlm_objects();

   {{ mode_1 }}_in_export = new("{{ mode_1 }}_in_export", this);
   {{ mode_2 }}_in_export = new("{{ mode_2 }}_in_export", this);
   e2e_in_export = new("e2e_in_export", this);
   {{ mode_1 }}_in_fifo   = new("{{ mode_1 }}_in_fifo"  , this);
   {{ mode_2 }}_in_fifo   = new("{{ mode_2 }}_in_fifo"  , this);
   e2e_in_fifo = new("e2e_in_fifo", this);
   {{ mode_1 }}_out_ap    = new("{{ mode_1 }}_out_ap"   , this);
   {{ mode_2 }}_out_ap    = new("{{ mode_2 }}_out_ap"   , this);
   e2e_out_ap = new("e2e_out_ap", this);

endfunction : create_tlm_objects


function void uvme_{{ name }}_st_prd_c::connect_ports();

   {{ mode_1 }}_in_export.connect({{ mode_1 }}_in_fifo.analysis_export);
   {{ mode_2 }}_in_export.connect({{ mode_2 }}_in_fifo.analysis_export);
   e2e_in_export.connect(e2e_in_fifo.analysis_export);

endfunction : connect_ports


task uvme_{{ name }}_st_prd_c::predict_{{ mode_1 }}();

   uvma_{{ name }}_seq_item_c  in_trn;
   uvma_{{ name }}_mon_trn_c   out_trn;
   forever begin
      {{ mode_1 }}_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.access_type = in_trn.access_type;
      out_trn.address     = in_trn.address    ;
      {% if sbe %}out_trn.be          = in_trn.be         ;
      {% endif %}out_trn.data        = in_trn.data       ;
      out_trn.set_initiator(in_trn.get_initiator());
      if (cntxt.passive_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      {{ mode_1 }}_out_ap.write(out_trn);
   end

endtask : predict_{{ mode_1 }}


task uvme_{{ name }}_st_prd_c::predict_{{ mode_2 }}();

   uvma_{{ name }}_{{ mode_2 }}_{{ segments[0].name }}_seq_item_c  in_trn;
   uvma_{{ name }}_mon_trn_c  out_trn;

   forever begin
      {{ mode_2 }}_in_fifo.get(in_trn);
      // TODO Implement uvme_{{ name }}_st_prd_c::predict_{{ mode_2 }}()
      //      Ex: if ((in_trn.req == 1'b1) && (in_trn.gnt === 1'b1)) begin
      //             out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      //             out_trn.data = in_trn.{{ segments[0].name }}data;
      //             out_trn.set_initiator(in_trn.get_initiator());
      //             if (cntxt.{{ mode_1 }}_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
      //                out_trn.set_may_drop(1);
      //             end
      //             if (in_trn.has_error()) begin
      //                out_trn.set_may_drop(1);
      //             end
      //             {{ mode_2 }}_out_ap.write(out_trn);
      //          end
   end

endtask : predict_{{ mode_2 }}


task uvme_{{ name }}_st_prd_c::predict_e2e();

   uvma_{{ name }}_mon_trn_c  in_trn, out_trn;
   forever begin
      e2e_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.copy(in_trn);
      out_trn.set_initiator(in_trn.get_initiator());
      if (cntxt.{{ mode_2 }}_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      e2e_out_ap.write(out_trn);
   end

endtask : predict_e2e


`endif // __UVME_{{ name.upper() }}_ST_PRD_SV__
