// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_PRD_SV__
`define __UVME_{{ name.upper() }}_ST_PRD_SV__


/**
 * Component implementing transaction-based prediction for {{ full_name }} UVM Agent Self-Testing Environment.
 * Predicts how the Passive Agent will observe stimulus from {{ mode_1.upper() }} & {{ mode_2.upper() }} Agents as well as end-to-end.
 * @ingroup uvme_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_prd_c extends uvm_component;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Environment configuration handle
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Environment context handle
   /// @}

   /// @name TLM
   /// @{
   {% if duplex %}uvm_analysis_export   #(uvma_{{ name }}_seq_item_c)  {{ mode_1 }}_in_export; ///< Input Port for Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_analysis_export   #(uvma_{{ name }}_seq_item_c)  {{ mode_2 }}_in_export; ///< Input Port for Sequence Items from {{ mode_2.upper() }} Agent.
   uvm_analysis_export   #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_in_export; ///< Input Port for Monitor Transactions from {{ dir_1.upper() }} Agent.
   uvm_analysis_export   #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_in_export; ///< Input Port for Monitor Transactions from {{ dir_2.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_seq_item_c)  {{ mode_1 }}_in_fifo; ///< FIFO of Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_seq_item_c)  {{ mode_2 }}_in_fifo; ///< FIFO of Sequence Items from {{ mode_2.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_in_fifo; ///< FIFO of Sequence Items from {{ dir_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_in_fifo; ///< FIFO of Sequence Items from {{ dir_2.upper() }} Agent.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  {{ mode_1 }}_out_ap; ///< Output Port for Monitor Transactions for {{ mode_1.upper() }} Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  {{ mode_2 }}_out_ap; ///< Output Port for Monitor Transactions for {{ mode_2.upper() }} Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  {{ dir_1 }}_out_ap; ///< Output Port for Monitor Transactions for {{ dir_1.upper() }} Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  {{ dir_2 }}_out_ap; ///< Output Port for Monitor Transactions for {{ dir_2.upper() }} Scoreboard Expected.
   {% else %}uvm_analysis_export   #(uvma_{{ name }}_seq_item_c)  agent_in_export; ///< Input Port for Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_analysis_export   #(uvma_{{ name }}_mon_trn_c )  e2e_in_export  ; ///< Input Port for Monitor Transactions from {{ mode_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_seq_item_c)  agent_in_fifo  ; ///< Input Port for Sequence Items from {{ mode_1.upper() }} Agent.
   uvm_tlm_analysis_fifo #(uvma_{{ name }}_mon_trn_c )  e2e_in_fifo    ; ///< Input Port for Monitor Transactions from {{ mode_1.upper() }} Agent.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  agent_out_ap   ; ///< Output Port for Monitor Transactions for Agent Scoreboard Expected.
   uvm_analysis_port     #(uvma_{{ name }}_mon_trn_c )  e2e_out_ap     ; ///< Output Port for Monitor Transactions for end-to-end Scoreboard Expected.
   {% endif %}/// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_prd_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_prd", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null
    * 2. Creates TLM objects
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects Exports to FIFOs.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Forks all processing tasks.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates TLM FIFOs and Analysis Ports.
    */
   extern function void create_tlm_objects();

   /**
    * Connects Exports to FIFOs.
    */
   extern function void connect_ports();

   {% if duplex %}/**
    * Predicts {{ mode_1.upper() }} data path.
    */
   extern task predict_{{ mode_1 }}();

   /**
    * Predicts {{ mode_2.upper() }} data path.
    */
   extern task predict_{{ mode_2 }}();

   /**
    * Predicts {{ dir_1.upper() }} data path.
    */
   extern task predict_{{ dir_1 }}();

   /**
    * Predicts {{ dir_2.upper() }} data path.
    */
   extern task predict_{{ dir_2 }}();
{% else %}/**
    * Predicts agent data path.
    */
   extern task predict_agent();

   /**
    * Predicts end-to-end data path.
    */
   extern task predict_e2e();
{% endif %}
endclass : uvme_{{ name }}_st_prd_c


function uvme_{{ name }}_st_prd_c::new(string name="uvme_{{ name }}_st_prd", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvme_{{ name }}_st_prd_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg           ();
   get_cntxt         ();
   create_tlm_objects();

endfunction : build_phase


function void uvme_{{ name }}_st_prd_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_ports();

endfunction: connect_phase


task uvme_{{ name }}_st_prd_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   fork
{% if duplex %}      predict_{{ mode_1 }}();
      predict_{{ mode_2 }}();
      predict_{{ dir_1 }}();
      predict_{{ dir_2 }}();
{% else %}      predict_agent();
      predict_e2e  ();
{% endif %}   join_none

endtask: run_phase


function void uvme_{{ name }}_st_prd_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_PRD", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_prd_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_PRD", "Context handle is null")
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_prd_c::create_tlm_objects();

   {% if duplex %}{{ mode_1 }}_in_export = new("{{ mode_1 }}_in_export", this);
   {{ mode_2 }}_in_export = new("{{ mode_2 }}_in_export", this);
   {{ dir_1 }}_in_export = new("{{ dir_1 }}_in_export", this);
   {{ dir_2 }}_in_export = new("{{ dir_2 }}_in_export", this);
   {{ mode_1 }}_in_fifo = new("{{ mode_1 }}_in_fifo", this);
   {{ mode_2 }}_in_fifo = new("{{ mode_2 }}_in_fifo", this);
   {{ dir_1 }}_in_fifo = new("{{ dir_1 }}_in_fifo", this);
   {{ dir_2 }}_in_fifo = new("{{ dir_2 }}_in_fifo", this);
   {{ mode_1 }}_out_ap = new("{{ mode_1 }}_out_ap", this);
   {{ mode_2 }}_out_ap = new("{{ mode_2 }}_out_ap", this);
   {{ dir_1 }}_out_ap = new("{{ dir_1 }}_out_ap", this);
   {{ dir_2 }}_out_ap = new("{{ dir_2 }}_out_ap", this);
   {% else %}agent_in_export = new("agent_in_export", this);
   e2e_in_export   = new("e2e_in_export"  , this);
   agent_in_fifo   = new("agent_in_fifo"  , this);
   e2e_in_fifo     = new("e2e_in_fifo"    , this);
   agent_out_ap    = new("agent_out_ap"   , this);
   e2e_out_ap      = new("e2e_out_ap"     , this);
{% endif %}
endfunction : create_tlm_objects


function void uvme_{{ name }}_st_prd_c::connect_ports();

   {% if duplex %}{{ mode_1 }}_in_export.connect({{ mode_1 }}_in_fifo.analysis_export);
   {{ mode_2 }}_in_export.connect({{ mode_2 }}_in_fifo.analysis_export);
   {{ dir_1 }}_in_export.connect({{ dir_1 }}_in_fifo.analysis_export);
   {{ dir_2 }}_in_export.connect({{ dir_2 }}_in_fifo.analysis_export);
   {% else %}agent_in_export.connect(agent_in_fifo.analysis_export);
   e2e_in_export  .connect(e2e_in_fifo  .analysis_export);
{% endif %}
endfunction : connect_ports


{% if duplex %}task uvme_{{ name }}_st_prd_c::predict_{{ mode_1 }}();

   uvma_{{ name }}_seq_item_c  in_trn ;
   uvma_{{ name }}_mon_trn_c   out_trn;
   forever begin
      {{ mode_1 }}_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.data_size = in_trn.data_size;
      out_trn.data      = new[in_trn.data_size];
      foreach (out_trn.data[ii]) begin
         out_trn.data[ii] = in_trn.data[ii];
      end
      out_trn.set_initiator(this);
      if (cntxt.passive_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      {{ mode_1 }}_out_ap.write(out_trn);
   end

endtask : predict_{{ mode_1 }}


task uvme_{{ name }}_st_prd_c::predict_{{ mode_2 }}();

   uvma_{{ name }}_seq_item_c  in_trn ;
   uvma_{{ name }}_mon_trn_c   out_trn;
   forever begin
      {{ mode_2 }}_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.data_size = in_trn.data_size;
      out_trn.data      = new[in_trn.data_size];
      foreach (out_trn.data[ii]) begin
         out_trn.data[ii] = in_trn.data[ii];
      end
      out_trn.set_initiator(this);
      if (cntxt.passive_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      {{ mode_2 }}_out_ap.write(out_trn);
   end

endtask : predict_{{ mode_2 }}


task uvme_{{ name }}_st_prd_c::predict_{{ dir_1 }}();

   uvma_{{ name }}_mon_trn_c  in_trn, out_trn;
   forever begin
      {{ dir_1  }}_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.copy(in_trn);
      out_trn.set_initiator(this);
      if (cntxt.{{ mode_2 }}_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      {{ dir_1 }}_out_ap.write(out_trn);
   end

endtask : predict_{{ dir_1 }}


task uvme_{{ name }}_st_prd_c::predict_{{ dir_2 }}();

   uvma_{{ name }}_mon_trn_c  in_trn, out_trn;
   forever begin
      {{ dir_2  }}_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.copy(in_trn);
      out_trn.set_initiator(this);
      if (cntxt.{{ mode_1 }}_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      {{ dir_2 }}_out_ap.write(out_trn);
   end

endtask : predict_{{ dir_2 }}


{% else %}task uvme_{{ name }}_st_prd_c::predict_agent();

   uvma_{{ name }}_seq_item_c  in_trn ;
   uvma_{{ name }}_mon_trn_c   out_trn;
   forever begin
      agent_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.data_size = in_trn.data_size;
      out_trn.data      = new[in_trn.data_size];
      foreach (out_trn.data[ii]) begin
         out_trn.data[ii] = in_trn.data[ii];
      end
      out_trn.set_initiator(this);
      if (cntxt.passive_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      agent_out_ap.write(out_trn);
   end

endtask : predict_agent


task uvme_{{ name }}_st_prd_c::predict_e2e();

   uvma_{{ name }}_mon_trn_c  in_trn, out_trn;
   forever begin
      e2e_in_fifo.get(in_trn);
      out_trn = uvma_{{ name }}_mon_trn_c::type_id::create("out_trn");
      out_trn.copy(in_trn);
      out_trn.set_initiator(this);
      if (cntxt.{{ mode_2 }}_cntxt.reset_state != UVML_RESET_STATE_POST_RESET) begin
         out_trn.set_may_drop(1);
      end
      if (in_trn.has_error()) begin
         out_trn.set_may_drop(1);
      end
      e2e_out_ap.write(out_trn);
   end

endtask : predict_e2e


{% endif %}`endif // __UVME_{{ name.upper() }}_ST_PRD_SV__
