// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_SB_SV__
`define __UVME_{{ name.upper() }}_ST_SB_SV__


/**
 * Component encapsulating scoreboarding components for {{ full_name }} UVM Agent Self-Testing Environment.
 * @ingroup uvme_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_sb_c extends uvm_component;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Environment configuration handle
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Environment context handle
   /// @}

   /// @name Components
   /// @{
   {% if duplex %}uvme_{{ name }}_st_sb_simplex_c            sb_{{ mode_1 }}; ///< {{ mode_1.upper() }} Scoreboard.
   uvme_{{ name }}_st_sb_simplex_c            sb_{{ mode_2 }}; ///< {{ mode_2.upper() }} Scoreboard.
   uvme_{{ name }}_st_sb_simplex_c            sb_{{ dir_1 }}; ///< {{ dir_1.upper() }} Scoreboard.
   uvme_{{ name }}_st_sb_simplex_c            sb_{{ dir_2 }}; ///< {{ dir_2.upper() }} Scoreboard.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_{{ mode_1 }}; ///< Delay for #sb_{{ mode_1 }} actual to avoid race condition.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_{{ mode_2 }}; ///< Delay for #sb_{{ mode_2 }} actual to avoid race condition.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_{{ dir_1 }}; ///< Delay for #sb_{{ dir_1 }} actual to avoid race condition.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_{{ dir_2 }}; ///< Delay for #sb_{{ dir_2 }} actual to avoid race condition.
   {% else %}uvme_{{ name }}_st_sb_simplex_c            sb_agent   ; ///< Agent Scoreboard.
   uvme_{{ name }}_st_sb_simplex_c            sb_e2e     ; ///< End-to-end Scoreboard.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_agent; ///< Delay for #sb_agent actual to avoid race condition.
   uvml_delay_c #(uvma_{{ name }}_mon_trn_c)  delay_e2e  ; ///< Delay for #sb_e2e actual to avoid race condition.
   {% endif %}/// @}

   /// @name TLM Ports
   /// @{
   {% if duplex %}uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ mode_1 }}_act_export; ///< Input Port for #sb_{{ mode_1 }} Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ mode_2 }}_act_export; ///< Input Port for #sb_{{ mode_2 }} Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_1 }}_act_export; ///< Input Port for #sb_{{ dir_1 }} Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_2 }}_act_export; ///< Input Port for #sb_{{ dir_2 }} Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ mode_1 }}_exp_export; ///< Input Port for #sb_{{ mode_1 }} Expected Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ mode_2 }}_exp_export; ///< Input Port for #sb_{{ mode_2 }} Expected Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_1 }}_exp_export; ///< Input Port for #sb_{{ dir_1 }} Expected Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  {{ dir_2 }}_exp_export; ///< Input Port for #sb_{{ dir_2 }} Expected Transactions.
   {% else %}
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  agent_act_export; ///< Input Port for #sb_agent Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  e2e_act_export  ; ///< Input Port for #sb_e2e Actual Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  agent_exp_export; ///< Input Port for #sb_agent Expected Transactions.
   uvm_analysis_export #(uvma_{{ name }}_mon_trn_c)  e2e_exp_export  ; ///< Input Port for #sb_e2e Expected Transactions.
   {% endif %}/// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_sb_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_sb", uvm_component parent=null);

   /**
    * Ensures #cfg & #cntxt handles are not null.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects exports to scoreboards'.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Assigns configuration handles to components using uvm_config_db.
    */
   extern function void assign_cfg();

   /**
    * Assigns context handles to components using uvm_config_db.
    */
   extern function void assign_cntxt();

   /**
    * Creates scoreboard components.
    */
   extern function void create_components();

   /**
    * Connects scoreboard TLM ports and configures delays.
    */
   extern function void connect_ports();

   /**
    * Sets delay duration on delay components.
    */
   extern function void configure_delays();

endclass : uvme_{{ name }}_st_sb_c


function uvme_{{ name }}_st_sb_c::new(string name="uvme_{{ name }}_st_sb", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvme_{{ name }}_st_sb_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg          ();
   get_cntxt        ();
   assign_cfg       ();
   assign_cntxt     ();
   create_components();

endfunction : build_phase


function void uvme_{{ name }}_st_sb_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_ports   ();
   configure_delays();

endfunction : connect_phase


function void uvme_{{ name }}_st_sb_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (cfg == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_SB", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_sb_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (cntxt == null) begin
      `uvm_fatal("{{ name.upper() }}_ST_SB", "Context handle is null")
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_sb_c::assign_cfg();

   {% if duplex %}uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_{{ mode_1 }}", "cfg", cfg.sb_{{ mode_1 }}_cfg);
   uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_{{ mode_2 }}", "cfg", cfg.sb_{{ mode_2 }}_cfg);
   uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_{{ segments[0] }}", "cfg", cfg.sb_{{ dir_1 }}_cfg);
   uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_{{ segments[1] }}", "cfg", cfg.sb_{{ dir_2 }}_cfg);
   {% else %}uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_agent", "cfg", cfg.sb_agent_cfg);
   uvm_config_db#(uvml_sb_simplex_cfg_c)::set(this, "sb_e2e"  , "cfg", cfg.sb_e2e_cfg  );
{% endif %}
endfunction: assign_cfg


function void uvme_{{ name }}_st_sb_c::assign_cntxt();

   {% if duplex %}uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_{{ mode_1 }}", "cntxt", cntxt.sb_{{ mode_1 }}_cntxt);
   uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_{{ mode_2 }}", "cntxt", cntxt.sb_{{ mode_2 }}_cntxt);
   uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_{{ dir_1 }}", "cntxt", cntxt.sb_{{ dir_1 }}_cntxt);
   uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_{{ dir_2 }}", "cntxt", cntxt.sb_{{ dir_2 }}_cntxt);
   {% else %}uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_agent", "cntxt", cntxt.sb_agent_cntxt);
   uvm_config_db#(uvml_sb_simplex_cntxt_c)::set(this, "sb_e2e"  , "cntxt", cntxt.sb_e2e_cntxt  );
{% endif %}
endfunction: assign_cntxt


function void uvme_{{ name }}_st_sb_c::create_components();

   {% if duplex %}sb_{{ mode_1 }} = uvme_{{ name }}_st_sb_simplex_c::type_id::create("sb_{{ mode_1 }}", this);
   sb_{{ mode_2 }} = uvme_{{ name }}_st_sb_simplex_c::type_id::create("sb_{{ mode_2 }}", this);
   sb_{{ dir_1 }} = uvme_{{ name }}_st_sb_simplex_c::type_id::create("sb_{{ dir_1 }}", this);
   sb_{{ dir_2 }} = uvme_{{ name }}_st_sb_simplex_c::type_id::create("sb_{{ dir_2 }}", this);
   delay_{{ mode_1 }} = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_{{ mode_1 }}", this);
   delay_{{ mode_2 }} = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_{{ mode_2 }}", this);
   delay_{{ dir_1 }} = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_{{ dir_1 }}", this);
   delay_{{ dir_2 }} = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_{{ dir_2 }}", this);
   {% else %}sb_agent    = uvme_{{ name }}_st_sb_simplex_c          ::type_id::create("sb_agent"   , this);
   sb_e2e      = uvme_{{ name }}_st_sb_simplex_c          ::type_id::create("sb_e2e"     , this);
   delay_agent = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_agent", this);
   delay_e2e   = uvml_delay_c #(uvma_{{ name }}_mon_trn_c)::type_id::create("delay_e2e"  , this);
{% endif %}
endfunction: create_components


function void uvme_{{ name }}_st_sb_c::connect_ports();

   {% if duplex %}{{ mode_1 }}_act_export = delay_{{ mode_1 }}.in_export;
   {{ mode_2 }}_act_export = delay_{{ mode_2 }}.in_export ;
   {{ dir_1 }}_act_export = delay_{{ dir_1 }}.in_export;
   {{ dir_2 }}_act_export = delay_{{ dir_2 }}.in_export;
   {{ mode_1 }}_exp_export = sb_{{ mode_1 }}.exp_export;
   {{ mode_2 }}_exp_export = sb_{{ mode_2 }}.exp_export;
   {{ dir_1 }}_exp_export = sb_{{ dir_1 }}.exp_export;
   {{ dir_2 }}_exp_export = sb_{{ dir_2 }}.exp_export;
   delay_{{ mode_1 }}.out_ap.connect(sb_{{ mode_1 }}.act_export);
   delay_{{ mode_2 }}.out_ap.connect(sb_{{ mode_2 }}.act_export);
   delay_{{ dir_1 }}.out_ap.connect(sb_{{ dir_1 }}.act_export);
   delay_{{ dir_2 }}.out_ap.connect(sb_{{ dir_2 }}.act_export);
   {% else %}agent_act_export = delay_agent.in_export;
   e2e_act_export   = delay_e2e  .in_export;
   agent_exp_export = sb_agent   .exp_export;
   e2e_exp_export   = sb_e2e     .exp_export;
   delay_agent.out_ap.connect(sb_agent.act_export);
   delay_e2e  .out_ap.connect(sb_e2e  .act_export);
{% endif %}
endfunction: connect_ports


function void uvme_{{ name }}_st_sb_c::configure_delays();

   {% if duplex %}delay_{{ mode_1 }}.set_duration(uvme_{{ name }}_st_sb_delay_cons);
   delay_{{ mode_2 }}.set_duration(uvme_{{ name }}_st_sb_delay_cons);
   delay_{{ dir_1 }}.set_duration(uvme_{{ name }}_st_sb_delay_cons);
   delay_{{ dir_2 }}.set_duration(uvme_{{ name }}_st_sb_delay_cons);
   {% else %}delay_agent.set_duration(uvme_{{ name }}_st_sb_delay_cons);
   delay_e2e  .set_duration(uvme_{{ name }}_st_sb_delay_cons);
{% endif %}
endfunction: configure_delays


`endif // __UVME_{{ name.upper() }}_ST_SB_SV__
