// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_CFG_SV__
`define __UVME_{{ name.upper() }}_ST_CFG_SV__


/**
 * Object encapsulating all parameters for creating, connecting and running {{ full_name }} UVM Agent Self-Testing
 * Environment (uvme_{{ name }}_st_env_c) components.
 * @ingroup uvme_{{ name }}_st_obj
 */
class uvme_{{ name }}_st_cfg_c extends uvml_cfg_c;

   /// @name Integrals
   /// @{
   rand bit                      enabled              ; ///< Enables/disables creation of all sub-components.
   rand uvm_active_passive_enum  is_active            ; ///< Sets all agents as active/passive.
   rand uvml_reset_type_enum     reset_type           ; ///< Sets the type of reset the agent is sensitive to.
   rand bit                      scoreboarding_enabled; ///< Enables/disables creation of Predictor and Scoreboard.
   rand bit                      cov_model_enabled    ; ///< Enables/disables functional coverage models.
   rand bit                      trn_log_enabled      ; ///< Enables/disables transaction logging.
   {% if mm %}rand int unsigned             addr_width           ; ///< Address bus width.
{% endif %}rand int unsigned             data_width           ; ///< Data bus width.
   /// @}

   /// @name Objects
   /// @{
   rand uvma_{{ name }}_cfg_c  {{ mode_1 }}_cfg; ///< {{ mode_1.upper() }} Agent configuration.
   rand uvma_{{ name }}_cfg_c  {{ mode_2 }}_cfg; ///< {{ mode_2.upper() }} Agent configuration.
   rand uvma_{{ name }}_cfg_c  passive_cfg; ///< Passive Agent configuration.
   {% if mm %}rand uvml_sb_simplex_cfg_c  sb_{{ mode_1 }}_cfg; ///<  {{ mode_1.upper() }} Agent configuration.
   rand uvml_sb_simplex_cfg_c  sb_{{ mode_2 }}_cfg; ///< {{ mode_2.upper() }} Agent configuration.
   rand uvml_sb_simplex_cfg_c  sb_e2e_cfg; ///< End-to-end Scoreboard configuration.
   rand uvme_{{ name }}_st_reg_block_c  reg_block; ///< Mock register model for testing uvma_{{ name }}_reg_adapter_c.
   {% else %}{% if duplex %}rand uvml_sb_simplex_cfg_c  sb_{{ mode_1 }}_cfg; ///< {{ mode_1.upper() }} Agent configuration.
   rand uvml_sb_simplex_cfg_c  sb_{{ mode_2 }}_cfg; ///< {{ mode_2.upper() }} Agent configuration.
   rand uvml_sb_simplex_cfg_c  sb_{{ dir_1 }}_cfg; ///< {{ dir_1.upper() }} Agent configuration.
   rand uvml_sb_simplex_cfg_c  sb_{{ dir_2 }}_cfg; ///< {{ dir_2.upper() }} Agent configuration.
   {% else %}rand uvml_sb_simplex_cfg_c  sb_agent_cfg; ///< Agent Scoreboard configuration.
   rand uvml_sb_simplex_cfg_c  sb_e2e_cfg  ; ///<  End-to-end Scoreboard configuration.
{% endif %}{% endif %}   /// @}


   `uvm_object_utils_begin(uvme_{{ name }}_st_cfg_c)
      `uvm_field_int (                         enabled              , UVM_DEFAULT)
      `uvm_field_enum(uvm_active_passive_enum, is_active            , UVM_DEFAULT)
      `uvm_field_enum(uvml_reset_type_enum   , reset_type           , UVM_DEFAULT)
      `uvm_field_int (                         scoreboarding_enabled, UVM_DEFAULT)
      `uvm_field_int (                         cov_model_enabled    , UVM_DEFAULT)
      `uvm_field_int (                         trn_log_enabled      , UVM_DEFAULT)
      `uvm_field_object({{ mode_1 }}_cfg, UVM_DEFAULT)
      `uvm_field_object({{ mode_2 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(passive_cfg, UVM_DEFAULT)
      {% if mm %}`uvm_field_object(sb_{{ mode_1 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_{{ mode_2 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_e2e_cfg, UVM_DEFAULT)
      {% else %}{% if duplex %}`uvm_field_object(sb_{{ mode_1 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_{{ mode_2 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_{{ dir_1 }}_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_{{ dir_2 }}_cfg, UVM_DEFAULT)
      {% else %}`uvm_field_object(sb_agent_cfg, UVM_DEFAULT)
      `uvm_field_object(sb_e2e_cfg, UVM_DEFAULT)
{% endif %}{% endif %}   `uvm_object_utils_end


   /**
    * Sets likeliest, safe defaults.
    */
   constraint defaults_cons {
      soft enabled                == 1;
      soft is_active              == UVM_ACTIVE;
      soft scoreboarding_enabled  == 1;
      soft cov_model_enabled      == 0;
      soft trn_log_enabled        == 1;
      {% if mm %}soft addr_width  == uvma_{{ name }}_default_addr_width;
      soft data_width == uvma_{{ name }}_default_data_width;
   {% else %}   soft data_width == uvma_{{ name }}_default_data_width;
   {% endif %}}

   /**
    * Sets all configuration fields for agent configurations.
    */
   constraint agents_cfg_cons {
      {{ mode_1 }}_cfg.drv_mode == UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_1.upper() }};
      {{ mode_2 }}_cfg.drv_mode == UVMA_{{ name.upper() }}_DRV_MODE_{{ mode_2.upper() }};
      {{ mode_1 }}_cfg.drv_idle == UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS;
      {{ mode_2 }}_cfg.drv_idle == UVMA_{{ name.upper() }}_DRV_IDLE_ZEROS;
      {{ mode_1 }}_cfg.bypass_mode == 0;
      {{ mode_2 }}_cfg.bypass_mode == 0;
      passive_cfg.bypass_mode == 0;
      {{ mode_1 }}_cfg.reset_type == reset_type;
      {{ mode_2 }}_cfg.reset_type == reset_type;
      passive_cfg.reset_type == reset_type;
      {% if mm %}{{ mode_1 }}_cfg.addr_width == addr_width;
{% endif %}      {{ mode_1 }}_cfg.data_width == data_width;
      {% if mm %}{{ mode_2 }}_cfg.addr_width == addr_width;
{% endif %}      {{ mode_2 }}_cfg.data_width == data_width;
      {% if mm %}passive_cfg.addr_width == addr_width;
{% endif %}      passive_cfg.data_width == data_width;
      if (enabled) {
         {{ mode_1 }}_cfg.enabled == 1;
         {{ mode_2 }}_cfg.enabled == 1;
         passive_cfg.enabled == 1;
      }
      else {
         {{ mode_1 }}_cfg.enabled == 0;
         {{ mode_2 }}_cfg.enabled == 0;
         passive_cfg.enabled == 0;
      }
      passive_cfg.is_active == UVM_PASSIVE;
      if (is_active == UVM_ACTIVE) {
         {{ mode_1 }}_cfg.is_active == UVM_ACTIVE;
         {{ mode_2 }}_cfg.is_active == UVM_ACTIVE;
      }
      else {
         {{ mode_1 }}_cfg.is_active == UVM_PASSIVE;
         {{ mode_2 }}_cfg.is_active == UVM_PASSIVE;
      }
      if (cov_model_enabled) {
         {{ mode_1 }}_cfg.cov_model_enabled == 1;
         {{ mode_2 }}_cfg.cov_model_enabled == 1;
         passive_cfg.cov_model_enabled == 1;
      }
      else {
         {{ mode_1 }}_cfg.cov_model_enabled == 0;
         {{ mode_2 }}_cfg.cov_model_enabled == 0;
         passive_cfg.cov_model_enabled == 0;
      }
      if (trn_log_enabled) {
         {{ mode_1 }}_cfg.trn_log_enabled == 1;
         {{ mode_2 }}_cfg.trn_log_enabled == 1;
         passive_cfg.trn_log_enabled == 1;
      }
      else {
         {{ mode_1 }}_cfg.trn_log_enabled == 0;
         {{ mode_2 }}_cfg.trn_log_enabled == 0;
         passive_cfg.trn_log_enabled == 0;
      }
   }

   /**
    * Sets all configuration fields for Scoreboard configurations.
    */
   constraint sb_e2e_cfg_cons {
      {% if mm %}sb_{{ mode_1 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_{{ mode_2 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_e2e_cfg.mode == UVML_SB_MODE_IN_ORDER;
      {% else %}{% if duplex %}sb_{{ mode_1 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_{{ mode_2 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_{{ dir_1 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_{{ dir_2 }}_cfg.mode == UVML_SB_MODE_IN_ORDER;
      {% else %}sb_agent_cfg.mode == UVML_SB_MODE_IN_ORDER;
      sb_e2e_cfg.mode == UVML_SB_MODE_IN_ORDER;
{% endif %}{% endif %}      if (scoreboarding_enabled) {
         {% if mm %}sb_{{ mode_1 }}_cfg.enabled == 1;
         sb_{{ mode_2 }}_cfg.enabled == 1;
         sb_e2e_cfg.enabled == 1;
         {% else %}{% if duplex %}sb_{{ mode_1 }}_cfg.enabled == 1;
         sb_{{ mode_2 }}_cfg.enabled == 1;
         sb_{{ dir_1 }}_cfg.enabled == 1;
         sb_{{ dir_2 }}_cfg.enabled == 1;
         {% else %}sb_agent_cfg.enabled == 1;
         sb_e2e_cfg.enabled == 1;
{% endif %}{% endif %}      }
      else {
         {% if mm %}sb_{{ mode_1 }}_cfg.enabled == 0;
         sb_{{ mode_2 }}_cfg.enabled == 0;
         sb_e2e_cfg.enabled == 0;
         {% else %}{% if duplex %}sb_{{ mode_1 }}_cfg.enabled == 0;
         sb_{{ mode_2 }}_cfg.enabled == 0;
         sb_{{ dir_1 }}_cfg.enabled == 0;
         sb_{{ dir_2 }}_cfg.enabled == 0;
         {% else %}sb_agent_cfg.enabled == 0;
         sb_e2e_cfg.enabled == 0;
      {% endif %}{% endif %}}
   }


   /**
    * Creates objects.
    */
   extern function new(string name="uvme_{{ name }}_st_cfg");

endclass : uvme_{{ name }}_st_cfg_c


function uvme_{{ name }}_st_cfg_c::new(string name="uvme_{{ name }}_st_cfg");

   super.new(name);
   {{ mode_1 }}_cfg = uvma_{{ name }}_cfg_c::type_id::create("{{ mode_1 }}_cfg");
   {{ mode_2 }}_cfg = uvma_{{ name }}_cfg_c::type_id::create("{{ mode_2 }}_cfg");
   passive_cfg = uvma_{{ name }}_cfg_c::type_id::create("passive_cfg");
{% if mm %}   sb_{{ mode_1 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ mode_1 }}_cfg");
   sb_{{ mode_2 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ mode_2 }}_cfg");
   sb_e2e_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_e2e_cfg" );
   reg_block = uvme_{{ name }}_st_reg_block_c::type_id::create("reg_block");
   reg_block.build();
{% else %}{% if duplex %}   sb_{{ mode_1 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ mode_1 }}_cfg");
   sb_{{ mode_2 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ mode_2 }}_cfg");
   sb_{{ dir_1 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ dir_1 }}_cfg");
   sb_{{ dir_2 }}_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_{{ dir_2 }}_cfg");
{% else %}   sb_agent_cfg = uvml_sb_simplex_cfg_c::type_id::create("sb_agent_cfg");
   sb_e2e_cfg   = uvml_sb_simplex_cfg_c::type_id::create("sb_e2e_cfg"  );
{% endif %}{% endif %}   // TODO Add scoreboard logs
   //      Ex: sb_{{ mode_1 }}_cfg.log_entry_metadata_q.push_back(0);
   //          sb_{{ mode_1 }}_cfg.log_entry_metadata_q.push_back(1);

endfunction : new


`endif // __UVME_{{ name.upper() }}_ST_CFG_SV__
