// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__
`define __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__


/**
 * Virtual Sequence sending a set number of random Sequence items (accesses) to {{ mode_1 }} Virtual Sequencer
 * with knobs for reads/writes ratio and min/max gap control.
 * @ingroup uvme_{{ name }}_st_seq
 */
class uvme_{{ name }}_st_rand_stim_vseq_c extends uvme_{{ name }}_st_base_vseq_c;

   /// @name Knobs
   /// @{
   rand int unsigned  num_access; ///< Number of accesses to generate.
   rand int unsigned  pct_reads ; ///< Percentage of accesses which are UVMA_{{ name.upper() }}_ACCESS_READ
   rand int unsigned  pct_writes; ///< Percentage of accesses which are UVMA_{{ name.upper() }}_ACCESS_WRITE
   rand int unsigned  min_gap   ; ///< Minimum number of cycles between accesses.
   rand int unsigned  max_gap   ; ///< Maximum number of cycles between accesses.
   /// @}

   /// @name Sequences
   /// @{
   rand uvma_{{ name }}_rand_stim_vseq_c  rand_stim_vseq; ///< Sequence for {{ mode_1.upper() }} Agent.
   /// @}


   `uvm_object_utils_begin(uvme_{{ name }}_st_rand_stim_vseq_c)
      `uvm_field_int(num_access, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(pct_reads , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(pct_writes, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_gap   , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_gap   , UVM_DEFAULT + UVM_DEC)
   `uvm_object_utils_end


   /**
    * Sets safe defaults.
    */
   constraint defaults_cons {
      soft num_access == uvme_{{ name }}_st_rand_stim_vseq_default_num_access;
      soft min_gap    == uvme_{{ name }}_st_rand_stim_vseq_default_min_gap   ;
      soft max_gap    == uvme_{{ name }}_st_rand_stim_vseq_default_max_gap   ;
   }

   /**
    * Links Sequences knobs.
    */
   constraint limits_cons {
      rand_stim_vseq.num_access == local::num_access;
      rand_stim_vseq.pct_reads  == local::pct_reads ;
      rand_stim_vseq.pct_writes == local::pct_writes;
      rand_stim_vseq.min_gap    == local::min_gap   ;
      rand_stim_vseq.max_gap    == local::max_gap   ;
   }


   /**
    * Creates Sequences.
    */
   extern function new(string name="uvme_{{ name }}_st_rand_stim_vseq");

   /**
    * Generates Sequence Item according to knobs valuess.
    */
   extern virtual task body();

endclass : uvme_{{ name }}_st_rand_stim_vseq_c


function uvme_{{ name }}_st_rand_stim_vseq_c::new(string name="uvme_{{ name }}_st_rand_stim_vseq");

   super.new(name);
   rand_stim_vseq = uvma_{{ name }}_rand_stim_vseq_c::type_id::create("rand_stim_vseq");

endfunction : new


task uvme_{{ name }}_st_rand_stim_vseq_c::body();

   rand_stim_vseq.start(p_sequencer.{{ mode_1 }}_vsequencer);

endtask : body


`endif // __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__
