// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__
`define __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__


/**
 * {% if duplex %}Virtual Sequence sending a set number of random Sequence Items (packets) to {{ mode_1 }} and {{ mode_2 }} Virtual Sequencers
 * {% else %}Virtual Sequence sending a set number of random Sequence Items (packets) to {{ mode_1 }} Virtual Sequencer% endif %}
 * {% endif %}with knobs for min/max packet size and gap.
 * @ingroup uvme_{{ name }}_st_seq
 */
class uvme_{{ name }}_st_rand_stim_vseq_c extends uvme_{{ name }}_st_base_vseq_c;

   /// @name Knobs
   /// @{
   rand int unsigned  num_pkts; ///< Number of packets to generate.
   rand int unsigned  min_size; ///< Mininum packet size.
   rand int unsigned  max_size; ///< Maximum packet size.
   rand int unsigned  min_gap ; ///< Minimum gap.
   rand int unsigned  max_gap ; ///< Maximum gap.
   /// @}

   /// @name Sequences
   /// @{
   rand uvma_{{ name }}_rand_stim_vseq_c  {{ mode_1 }}_vseq; ///< Packet Sequence for {{ mode_1.upper() }} Agent.
   {% if duplex %}rand uvma_{{ name }}_rand_stim_vseq_c  {{ mode_2 }}_vseq; ///< Packet Sequence for {{ mode_2.upper() }} Agent.
{% endif %}   /// @}


   `uvm_object_utils_begin(uvme_{{ name }}_st_rand_stim_vseq_c)
      `uvm_field_int(num_pkts, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_size, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_size, UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(min_gap , UVM_DEFAULT + UVM_DEC)
      `uvm_field_int(max_gap , UVM_DEFAULT + UVM_DEC)
      `uvm_field_object({{ mode_1 }}_vseq, UVM_DEFAULT)
      {% if duplex %}`uvm_field_object({{ mode_2 }}_vseq, UVM_DEFAULT)
{% endif %}   `uvm_object_utils_end


   /**
    * Sets safe defaults.
    */
   constraint defaults_cons {
      soft num_pkts == uvme_{{ name }}_st_rand_stim_vseq_default_num_pkts;
      soft min_size == uvme_{{ name }}_st_rand_stim_vseq_default_min_size;
      soft max_size == uvme_{{ name }}_st_rand_stim_vseq_default_max_size;
      soft min_gap  == uvme_{{ name }}_st_rand_stim_vseq_default_min_gap ;
      soft max_gap  == uvme_{{ name }}_st_rand_stim_vseq_default_max_gap ;
   }

   /**
    * Links Sequences knobs.
    */
   constraint limits_cons {
      {{ mode_1 }}_vseq.num_pkts == local::num_pkts;
      {{ mode_1 }}_vseq.min_size == local::min_size;
      {{ mode_1 }}_vseq.max_size == local::max_size;
      {{ mode_1 }}_vseq.min_gap  == local::min_gap ;
      {{ mode_1 }}_vseq.max_gap  == local::max_gap ;
{% if duplex %}      {{ mode_2 }}_vseq.num_pkts == local::num_pkts;
      {{ mode_2 }}_vseq.min_size == local::min_size;
      {{ mode_2 }}_vseq.max_size == local::max_size;
      {{ mode_2 }}_vseq.min_gap  == local::min_gap ;
      {{ mode_2 }}_vseq.max_gap  == local::max_gap ;
{% endif %}   }


   /**
    * Creates Sequences.
    */
   extern function new(string name="uvme_{{ name }}_st_rand_stim_vseq");

   /**
    * {% if duplex %}Generates Sequence Items in both directions according to knobs values.
    * {% else %}Generates Sequence Items according to knobs values.{% endif %}
    */
   extern virtual task body();

endclass : uvme_{{ name }}_st_rand_stim_vseq_c


function uvme_{{ name }}_st_rand_stim_vseq_c::new(string name="uvme_{{ name }}_st_rand_stim_vseq");

   super.new(name);
   {{ mode_1 }}_vseq = uvma_{{ name }}_rand_stim_vseq_c::type_id::create("{{ mode_1 }}_vseq");
   {% if duplex %}{{ mode_2 }}_vseq = uvma_{{ name }}_rand_stim_vseq_c::type_id::create("{{ mode_2 }}_vseq");
{% endif %}
endfunction : new


task uvme_{{ name }}_st_rand_stim_vseq_c::body();

   `uvm_info("{{ name.upper() }}_ST_RAND_STIM_VSEQ", "Starting sequence", UVM_MEDIUM)
{% if duplex %}   fork
      {{ mode_1 }}_vseq.start(p_sequencer.{{ mode_1 }}_vsequencer);
      {{ mode_2 }}_vseq.start(p_sequencer.{{ mode_2 }}_vsequencer);
   join
{% else %}{{ mode_1 }}_vseq.start(p_sequencer.{{ mode_1 }}_vsequencer);
{% endif %}
endtask : body


`endif // __UVME_{{ name.upper() }}_ST_RAND_STIM_VSEQ_SV__
