# Hello from your Code Generator!
Thank you for using the Moore.io Advanced UVM Agent Code Template for Serial protocols v1.0!

Your parameters are:
* Name: '{{ name }}'
* Full Name: '{{ full_name }}'
* Mode 1: '{{ mode_1 }}'
* Mode 2: '{{ mode_2 }}'
* Direction 1: '{{ tx }}'
* Direction 2: '{{ rx }}'
* Clocking: {% if ddr %}DDR{% else %}SDR{% endif %}
* Interface Symmetry: {% if symmetric %}{{ tx.upper() }} and {{ rx.upper() }} are the same{% else %}{{ tx.upper() }} and {{ rx.upper() }} are NOT the same{% endif %}

If this is incorrect, it is recommended to delete the generated IP and re-generate with the correct parameters.

This code template is based on the ['Advanced' UVM methodology](https://www.linkedin.com/pulse/advanced-uvm-brian-hunter/).
This agent uses multiple sequencers and virtual sequences to harness the full power of UVM and elegantly implement serial
protocols.  It can also be combined with other Advanced UVM template instances to create VIP 'stacks'.

As opposed to most other Moore.io UVM code templates, this one has a protocol implemented out of the box.
This serial protocol is as simple as possible, while providing enough material to quickly get you angled towards your particular
protocol implementation challenges.  It has a simple "training sequence", an edge "syncing" stage which locks upon 36b
frames:  2b header/sync, 32b data, 2b tail.  There is error checking on the frame itself, but once a serial link is
synced, it does not lose sync again regardless of errors.

What follows is a short guide for developers of this new UVM Agent.

 ## 1 - Modify the physical interface
 1. - [ ] Physical Interface - `uvma_{{ name }}_if.sv` - Modify the signals to match your protocol's physical interface.
{% if symmetric %} 1. - [ ] Phy Monitor Transaction - `obj/uvma_{{ name }}_phy_mon_trn.sv` - Modify the fields to match your interface definition.
 1. - [ ] Phy Sequence Item - `seq/uvma_{{ name }}_phy_seq_item.sv` - Modify the fields to match your interface definition.
 1. - [ ] Phy Driver - `comps/uvma_{{ name }}_phy_drv.sv` - Modify the code driving the modport signals to match your interface definition.
 1. - [ ] Phy Driver Virtual Sequence - `seq/uvma_{{ name }}_{{ tx }}_phy_vseq.sv` - Modify the sequence to match your interface definition.
 1. - [ ] Monitor - `comps/uvma_{{ name }}_mon.sv` - Modify the code sampling the modports signals to match your interface definition.
 1. - [ ] Monitor Virtual Sequence - `seq/uvma_{{ name }}_mon_vseq.sv` - Modify the code pushing data from Phy Monitor Transactions to match your new interface definition.
{% else %} 1. - [ ] {{ tx.upper() }} Phy Monitor Transaction - `obj/uvma_{{ name }}_{{ tx }}_mon_trn.sv` - Modify the fields to match your interface definition.
 1. - [ ] {{ rx.upper() }} Phy Monitor Transaction - `obj/uvma_{{ name }}_{{ rx }}_mon_trn.sv` - Modify the fields to match your interface definition.
 1. - [ ] {{ tx.upper() }} Phy Sequence Item - `seq/uvma_{{ name }}_{{ tx }}_seq_item.sv` - Modify the fields to match your interface definition.
 1. - [ ] {{ rx.upper() }} Phy Sequence Item - `seq/uvma_{{ name }}_{{ rx }}_seq_item.sv` - Modify the fields to match your interface definition.
 1. - [ ] {{ tx.upper() }} Phy Driver - `comps/uvma_{{ name }}_{{ tx }}_drv.sv` - Modify the code driving the modports signals to match your interface definition.
 1. - [ ] {{ rx.upper() }} Phy Driver - `comps/uvma_{{ name }}_{{ rx }}_drv_c` - Modify the code driving the modports signals to match your interface definition.
 1. - [ ] {{ tx.upper() }} Phy Driver Virtual Sequence - `seq/uvma_{{ name }}_{{ tx }}_drv_vseq.sv` - Modify the code to match your interface definition.
 1. - [ ] {{ rx.upper() }} Phy Driver Virtual Sequence - `seq/uvma_{{ name }}_{{ rx }}_drv_vseq.sv` - Modify the code to match your interface definition.
 1. - [ ] Monitor - `comps/uvma_{{ name }}_mon.sv` - Modify the code sampling the modports signals to match your interface definition.
 1. - [ ] Monitor Virtual Sequence - `seq/uvma_{{ name }}_mon_vseq.sv` - Modify the code pushing data from Phy Monitor Transactions to match your new interface definition.
{% endif %}
 ## 2 - Logical Transaction Model
 1. - [ ] Sequence Item - `seq/uvma_{{ name }}_seq_item.sv` - Modify the fields to model your protocol frame structure for stimulus generation.
 1. - [ ] Monitor Transaction - `obj/uvma_{{ name }}_mon_trn.sv` - Modify the fields to model your protocol frame structure for rebuilding from sampled data.

 ## 3 - Protocol Implementation
 1. - [ ] Configuration - `obj/uvma_{{ name }}_cfg.sv` - Add parameters if needed for protocol options.
 1. - [ ] FSM Definition - `obj/uvma_{{ name }}_mon_fsm_cntxt.sv` - Add fields and modify types if needed to store the Monitor Finite State Machine (FSM) data.
 1. - [ ] Monitor FSM - `obj/uvma_{{ name }}_mon_vseq.sv` - Modify the code to implement your protocol's serial FSM.
 1. - [ ] IDLE Generation - `seq/uvma_{{ name }}_idle_vseq.sv` - Modify the sequence to implement IDLE generation for your protocol.


## Have fun!
The codebase is ready to test using the other 2 IPs generated along with this agent:
* `uvme_{{ name }}_st`
* `uvmt_{{ name }}_st`




# {{ name_of_copyright_owner }} ({{ name.upper() }}) UVM Agent

# About
This IP contains the {{ name_of_copyright_owner }} {{ full_name }} UVM Agent.
TODO Describe {{ full_name }} UVM Agent

# Block Diagram
![alt text](./docs/agent_block_diagram.svg "{{ full_name }} UVM Agent Block Diagram")

# Directory Structure
* `bin` - Scripts, metadata and other miscellaneous files
* `docs` - Documents describing the {{ full_name }} UVM Agent
* `examples` - Code samples for using and extending this agent
* `src` - Source code


# Dependencies
It is dependent on the following IP:

* `uvm`
* `uvml`
* `uvml_logs`
