// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name_uppercase }}_ST_COV_MODEL_SV__
`define __UVME_{{ name_uppercase }}_ST_COV_MODEL_SV__


/**
 * Component encapsulating {{ name_normal_case }} UVM Library Self-Test functional coverage model.
 * @ingroup uvma_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_cov_model_c extends uvm_component;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Environment configuration handle
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Environment context handle
   /// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_cov_model_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Library configuration functional coverage.
    */
   covergroup {{ name }}_st_cfg_cg;
      enabled_cpt   : coverpoint cfg.enabled  ;
      is_active_cpt : coverpoint cfg.is_active;
   endgroup : {{ name }}_st_cfg_cg

   /**
    * Library context functional coverage.
    */
   covergroup {{ name }}_st_cntxt_cg;
   endgroup : {{ name }}_st_cntxt_cg


   /**
    * Creates all covergroups.
    */
   extern function new(string name="uvme_{{ name }}_st_cov_model", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null.
    * 2. Creates fifos.
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * Connects exports to fifos.
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Forks all sampling loops.
    */
   extern virtual task run_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Creates TLM FIFOs and Analysis Ports.
    */
   extern function void create_tlm_objects();

   /**
    * Connects Exports to FIFOs.
    */
   extern function void connect_ports();

   /**
    * Samples #{{ name }}_st_cfg_cg
    */
   extern virtual function void sample_cfg();

   /**
    * Samples #{{ name }}_st_cntxt_cg
    */
   extern virtual function void sample_cntxt();

endclass : uvme_{{ name }}_st_cov_model_c


function uvme_{{ name }}_st_cov_model_c::new(string name="uvme_{{ name }}_st_cov_model", uvm_component parent=null);

   super.new(name, parent);
   {{ name }}_st_cfg_cg   = new();
   {{ name }}_st_cntxt_cg = new();

endfunction : new


function void uvme_{{ name }}_st_cov_model_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg           ();
   get_cntxt         ();
   create_tlm_objects();

endfunction : build_phase


function void uvme_{{ name }}_st_cov_model_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   connect_ports();

endfunction : connect_phase


task uvme_{{ name }}_st_cov_model_c::run_phase(uvm_phase phase);

   super.run_phase(phase);
   fork
      // Configuration
      forever begin
         cntxt.sample_cfg_e.wait_trigger();
         sample_cfg();
      end
      // Context
      forever begin
         cntxt.sample_cntxt_e.wait_trigger();
         sample_cntxt();
      end
   join_none

endtask : run_phase


function void uvme_{{ name }}_st_cov_model_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (!cfg) begin
      `uvm_fatal("{{ name_uppercase }}_ST_COV_MODEL", "Configuration handle is null")
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_cov_model_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (!cntxt) begin
      `uvm_fatal("{{ name_uppercase }}_ST_COV_MODEL", "Context handle is null")
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_cov_model_c::create_tlm_objects();



endfunction : create_tlm_objects


function void uvme_{{ name }}_st_cov_model_c::connect_ports();



endfunction : connect_ports


function void uvme_{{ name }}_st_cov_model_c::sample_cfg();

  {{ name }}_st_cfg_cg.sample();

endfunction : sample_cfg


function void uvme_{{ name }}_st_cov_model_c::sample_cntxt();

   {{ name }}_st_cntxt_cg.sample();

endfunction : sample_cntxt


`endif // __UVME_{{ name_uppercase }}_ST_COV_MODEL_SV__
