// Copyright {{ year }} {{ name_of_copyright_owner }}
// {{ license }}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


`ifndef __UVME_{{ name_uppercase }}_ST_ENV_SV__
`define __UVME_{{ name_uppercase }}_ST_ENV_SV__


/**
 * Top-level component that encapsulates, builds and connects all other {{ name_normal_case }} UVM Library Self-Test Environment
 * (uvme_{{ name }}_st_env_c) components.
 * @ingroup uvma_{{ name }}_st_comps
 */
class uvme_{{ name }}_st_env_c extends uvml_env_c;

   /// @name Objects
   /// @{
   uvme_{{ name }}_st_cfg_c    cfg  ; ///< Configuration handle.  Must be provided by component instantiating this environment.
   uvme_{{ name }}_st_cntxt_c  cntxt; ///< Context handle.  Can be provided by component instantiating this environment.
   /// @}

   /// @name Components
   /// @{
   uvme_{{ name }}_st_cov_model_c  cov_model ; /// < Functional coverage model.
   uvme_{{ name }}_st_vsqr_c       vsequencer; ///< Virtual sequencer on which virtual sequences are run.
   uvme_{{ name }}_st_prd_c        predictor ; ///< Feeds #scoreboard's expected ports.
   uvme_{{ name }}_st_sb_c         scoreboard; ///< Top-level scoreboarding component.
   /// @}


   `uvm_component_utils_begin(uvme_{{ name }}_st_env_c)
      `uvm_field_object(cfg  , UVM_DEFAULT)
      `uvm_field_object(cntxt, UVM_DEFAULT)
   `uvm_component_utils_end


   /**
    * Default constructor.
    */
   extern function new(string name="uvme_{{ name }}_st_env", uvm_component parent=null);

   /**
    * 1. Ensures #cfg & #cntxt handles are not null
    * 2. Assigns #cfg and #cntxt handles
    * 3. Creates all components via create_<x>()
    */
   extern virtual function void build_phase(uvm_phase phase);

   /**
    * 1. Connects agents to predictor
    * 2. Connects predictor & agents to scoreboard
    * 3. Assembles virtual sequencer handles
    * 4. Connects agents to coverage model
    */
   extern virtual function void connect_phase(uvm_phase phase);

   /**
    * Uses uvm_config_db to retrieve cfg.
    */
   extern function void get_cfg();

   /**
    * Uses uvm_config_db to retrieve cntxt.
    */
   extern function void get_cntxt();

   /**
    * Assigns configuration handles to components using UVM Configuration Database.
    */
   extern function void assign_cfg();

   /**
    * Assigns context handles to components using UVM Configuration Database.
    */
   extern function void assign_cntxt();

   /**
    * Creates additional (non-agent) environment components (and objects).
    */
   extern function void create_env_components();

   /**
    * Creates environment's virtual sequencer.
    */
   extern function void create_vsequencer();

   /**
    * Connects agents to predictor.
    */
   extern function void connect_predictor();

   /**
    * Connects scoreboards components to agents/predictor.
    */
   extern function void connect_scoreboard();

   /**
    * Assembles virtual sequencer from agent sequencers.
    */
   extern function void assemble_vsequencer();

endclass : uvme_{{ name }}_st_env_c


function uvme_{{ name }}_st_env_c::new(string name="uvme_{{ name }}_st_env", uvm_component parent=null);

   super.new(name, parent);

endfunction : new


function void uvme_{{ name }}_st_env_c::build_phase(uvm_phase phase);

   super.build_phase(phase);
   get_cfg              ();
   get_cntxt            ();
   assign_cfg           ();
   assign_cntxt         ();
   create_env_components();
   create_vsequencer    ();

endfunction : build_phase


function void uvme_{{ name }}_st_env_c::connect_phase(uvm_phase phase);

   super.connect_phase(phase);
   assemble_vsequencer();
   connect_predictor  ();
   if (cfg.scoreboarding_enabled) begin
      connect_scoreboard();
   end

endfunction: connect_phase


function void uvme_{{ name }}_st_env_c::get_cfg();

   void'(uvm_config_db#(uvme_{{ name }}_st_cfg_c)::get(this, "", "cfg", cfg));
   if (!cfg) begin
      `uvm_fatal("{{ name_uppercase }}_ST_ENV", "Configuration handle is null")
   end
   else begin
      `uvm_info("{{ name_uppercase }}_ST_ENV", $sformatf("Found configuration handle:\n%s", cfg.sprint()), UVM_DEBUG)
   end

endfunction : get_cfg


function void uvme_{{ name }}_st_env_c::get_cntxt();

   void'(uvm_config_db#(uvme_{{ name }}_st_cntxt_c)::get(this, "", "cntxt", cntxt));
   if (!cntxt) begin
      `uvm_info("{{ name_uppercase }}_ST_ENV", "Context handle is null; creating.", UVM_DEBUG)
      cntxt = uvme_{{ name }}_st_cntxt_c::type_id::create("cntxt");
   end

endfunction : get_cntxt


function void uvme_{{ name }}_st_env_c::assign_cfg();

   uvm_config_db #(uvme_{{ name }}_st_cfg_c)::set(this, "*", "cfg", cfg);

endfunction: assign_cfg


function void uvme_{{ name }}_st_env_c::assign_cntxt();

   uvm_config_db #(uvme_{{ name }}_st_cntxt_c)::set(this, "*", "cntxt", cntxt);

endfunction: assign_cntxt


function void uvme_{{ name }}_st_env_c::create_env_components();

   cov_model  = uvme_{{ name }}_st_cov_model_c::type_id::create("cov_model" , this);
   predictor  = uvme_{{ name }}_st_prd_c      ::type_id::create("predictor" , this);
   scoreboard = uvme_{{ name }}_st_sb_c       ::type_id::create("scoreboard", this);

endfunction: create_env_components


function void uvme_{{ name }}_st_env_c::create_vsequencer();

   vsequencer = uvme_{{ name }}_st_vsqr_c::type_id::create("vsequencer", this);

endfunction: create_vsequencer


function void uvme_{{ name }}_st_env_c::connect_predictor();

   // Connect agent -> predictor

endfunction: connect_predictor


function void uvme_{{ name }}_st_env_c::connect_scoreboard();

   // Connect ? -> scoreboard
   // Connect predictor -> scoreboard

endfunction: connect_scoreboard


function void uvme_{{ name }}_st_env_c::assemble_vsequencer();



endfunction: assemble_vsequencer


`endif // __UVME_{{ name_uppercase }}_ST_ENV_SV__
